
/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createProdDspTrn.c
* Description:  Contains routines for the product files handling
*
* History:      
* 14-Jun-04     (csabet)
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <sys/stat.h>
#include <stdio.h>
#include <cpl.h>
#include <time.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "errorHandling.h"
#include "midiFitsUtility.h"
#include "fileHandling.h"
#include "createProdDspTrn.h"
#include "qfits.h"

/**********************************************************
*   Global Variables 
**********************************************************/

/*============================ C O D E    A R E A ===========================*/






/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createDspTrnProd
* Input/Output: See function arguments to avoid duplication
* Description:  Creates MIDI product files
*
* History:      
* 15-Jun-04     (csabet) Created
******************************************************************************/
void createDspTrnProd (
    MidiFiles       *fileNames,    // In: Pointer to the file structure
    ImageFormat        *format,    // In: Image format
    DispersiveTrans *trans,     // In: Dispersive transmission data structure
    int                *error)    // Ou: Error status
{
  
    /*  Local Declarations
    --------------------*/
    const char  routine[] = "createDspTrnProd";
    char    *classification, *stringTemp;
    FILE    *inFitsBatchPtr;
    float    *dspTrnImage;
    int        i, j, k, l, arraySize;
    qfitsdumper    qdDspTrn;
   
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nCreating Product files for DSP_TRN from batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"--------------------------------------------- \n");
    fprintf (midiReportPtr, "\nCreating Product for DSP_TRN from batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "--------------------------------------- \n");

    // Reset status
    *error = 0;
    arraySize = 300;
    
    //    Create the primary header extension
    createDspTrnPrimHead (fileNames, format, arraySize, trans, error);
    if (*error)
    {
    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create Primary Header extension");
    return;
    }

    //    Allocate memory
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    dspTrnImage = (float *) calloc (arraySize * arraySize, sizeof (float));

    //    Convert Transmissivity vector into an image and write into the FITS file
    for (i = 0; i < 3; i++)
    {
        for (j = 0; j < 100; j++)
        {
            for (k = 0; k < arraySize; k++)
            {
                l = (i * 100 + j) * arraySize + k;
                dspTrnImage[l] = trans->transmission[i];
            }
        }
    }
    
    qdDspTrn.filename  = fileNames->outFitsName;
    qdDspTrn.npix      = arraySize * arraySize;
    qdDspTrn.ptype     = PTYPE_FLOAT;
    qdDspTrn.fbuf      = dspTrnImage;
    qdDspTrn.out_ptype = BPP_IEEE_FLOAT;
    qfits_pixdump (&qdDspTrn);

    free (dspTrnImage);

    //  Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        *error = 1;
        return;
    }


    //  Read the name of the first file in the list and get it's full path name
    fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr);
    sprintf (classification, "%s", "");
    sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);

    // Copy keywords from the raw FITS file into the QC log
    createDspTrnQcLog (fileNames->inFitsName, error);
    if (*error)
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot copy keywords to QC log");

    //    Add product info to QC log
    addProdInfoToDspTrnQcLog (format, fileNames, trans, error);

    //  Close files
    fclose (inFitsBatchPtr);

    //  Release memory
    free (classification);
    free (stringTemp);
 
    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createDspTrnPrimHead
* Input/Output: See function arguments to avoid duplication
* Description:  Creates the primary header
*
*
* History:
* 24-Aug-06     (csabet) Created
******************************************************************************/
void createDspTrnPrimHead (
    MidiFiles       *fileNames,    // In: Pointer to the file structure
    ImageFormat        *format,    // In: Image format
    int                arraySize,    // In: size of the image
    DispersiveTrans    *trans,// In: Dispersive transmission data structure
    int                *error)        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "createDspTrnPrimHead";
    qfits_header    *outFitsHeader;
    FILE            *inFitsBatchPtr=NULL, *outFitsPtr;
    int             i;
    char            *textBuff, *stringQfits, *messageBuffer, *currentTime, *cleanString,
                    *stringTemp, *classification, *sWidthX, *sWidthY, *tech;
    time_t          now;
    struct tm       *newTime;
    struct stat     buf;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset parameters
    *error = 0;

    //    Open list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        *error = 1;
        return;
    }

    //    Allocate memory
    tech = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    textBuff = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    messageBuffer = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sWidthX = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sWidthY = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    //    Get current time/date
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    //    Read the name of the first file in the list and get it's full path name
    fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr);
    sprintf (classification, "%s", "");
    sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);

    //    Copy primary header from the raw file to the output header
    outFitsHeader = qfits_header_read (fileNames->inFitsName);
    if (outFitsHeader == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load header from the input FITS file");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        free (sWidthX);
        free (sWidthY);
        return;
    }

    //    Write all NAXIS values as character strings
    sprintf (sWidthY, "%d", arraySize);
    sprintf (sWidthX, "%d", arraySize);

    //    Updates header for the DetRon output file
    qfits_header_mod (outFitsHeader, "BITPIX", "-32", "number of bits per pixel"); 
    qfits_header_mod (outFitsHeader, "NAXIS", "2", "number of data axes"); 
    qfits_header_add (outFitsHeader, "NAXIS1", sWidthX, "", NULL);
    qfits_header_add (outFitsHeader, "NAXIS2", sWidthY, "", NULL);
    qfits_header_mod (outFitsHeader, "INSTRUME", "MIDI", "MIDI Raw Data Display FITS created by DRS pipeline" );  

    //    Now write all the required product keaywords to the output header
    //    -----------------------------------------------------------------
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO TYPE", format->obsType, "MIDI pipeline product type", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO CATG", "REDUCED_DSPTRN", "Pipeline product category", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO ARCFILE", fileNames->archFileName, "Arcfile name of first raw file", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO PIPEDATE", currentTime, "Pipeline run date", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO VERSION", MIDI_PIPE_VERSION, "Pipeline version", NULL);
    qfits_header_add (outFitsHeader, "PIPEFILE", fileNames->pipeFileName, "Pipeline product file name", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO DID", MIDI_QC_DIC_VERSION, "QC dictionary version", NULL);

    //    Rewind the list of files
    rewind (inFitsBatchPtr);
    
    //    Loop through the list of files and write into the primary header
    i = 0;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", messageBuffer, classification);

        //    Get Technique
        stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR TECH");
        if (stringQfits == NULL)
        {
            sprintf (tech, "%s", "UNKNOWN");
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Technique");
        }
        else
        {
            cleanUpString (stringQfits, cleanString);
            sprintf (tech, "%s", cleanString);
        }
        
        //    If classification is not given get it from the raw file
        if (strcmp (classification, "") == 0)
        {
            stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR CATG");
            if (stringQfits == NULL)
            {
                sprintf (classification, "%s", "UNKNOWN");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Category");
            }
            else
            {
                cleanUpString (stringQfits, cleanString);
                sprintf (classification, "%s", cleanString);
            }
        }
        removePathName (messageBuffer, midiReportPtr);
        sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d NAME", i+1);
        qfits_header_add (outFitsHeader, textBuff, messageBuffer, "FITS file name", "");
        sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d CATG", i+1);
        qfits_header_add (outFitsHeader, textBuff, classification, "Observation Categoty", "");
        i++;
    }
    fclose (inFitsBatchPtr);

    //    Add Image parameters
    for (i = 0; i < trans->numOfFiles; i++)
    {
        if (strcmp (trans->grismId[i], "OPEN") == 0)
        {
            sprintf (messageBuffer, "%s", trans->grismId[i]);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d IM NAME", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Grism Name", "");
            sprintf (messageBuffer, "%s", trans->filterName[i]);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d IM FIL", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Filter Name", "");
            sprintf (messageBuffer, "%0.2f", trans->target[i].xCoord);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d IM X", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "x coordinate", "");
            sprintf (messageBuffer, "%0.2f", trans->target[i].yCoord);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d IM Y", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "y coordinate", "");
            sprintf (messageBuffer, "%0.2f", trans->target[i].dxCoord);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d IM DX", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Shift x coordinate", "");
            sprintf (messageBuffer, "%0.2f", trans->target[i].dyCoord);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d IM DY", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Shift y coordinate", "");
            sprintf (messageBuffer, "%0.4f", trans->integTime[i]);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d IM EXT", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Integration time", "");
            sprintf (messageBuffer, "%0.2f", trans->meanFlux[i]);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d IM CNT", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Mean flux", "");
        }
        else if (strcmp (trans->grismId[i], "GRISM") == 0)
        {
            sprintf (messageBuffer, "%s", trans->grismId[i]);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d GR NAME", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Grism Name", "");
            sprintf (messageBuffer, "%s", trans->filterName[i]);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d GR FIL", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Filter Name", "");
            sprintf (messageBuffer, "%0.2f", trans->target[i].xCoord);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d GR X", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "x coordinate", "");
            sprintf (messageBuffer, "%0.2f", trans->target[i].yCoord);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d GR Y", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "y coordinate", "");
            sprintf (messageBuffer, "%0.2f", trans->target[i].dxCoord);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d GR DX", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Shift x coordinate", "");
            sprintf (messageBuffer, "%0.2f", trans->target[i].dyCoord);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d GR DY", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Shift y coordinate", "");
            sprintf (messageBuffer, "%0.4f", trans->integTime[i]);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d GR EXT", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Integration time", "");
            sprintf (messageBuffer, "%0.2f", trans->meanFlux[i]);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d GR CNT", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Mean flux", "");
            if (i == 3 || i == 4 || i == 5)
            {
                sprintf (messageBuffer, "%0.6f", trans->transmission[i-3]);
                sprintf (textBuff, "HIERARCH ESO QC TRN F%d GR TRANS", i+1);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transmissivity", "");
            }
        }
        else if (strcmp (trans->grismId[i], "PRISM") == 0)
        {
            sprintf (messageBuffer, "%s", trans->grismId[i]);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d PR NAME", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Grism Name", "");
            sprintf (messageBuffer, "%s", trans->filterName[i]);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d PR FIL", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Filter Name", "");
            sprintf (messageBuffer, "%0.2f", trans->target[i].xCoord);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d PR X", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "x coordinate", "");
            sprintf (messageBuffer, "%0.2f", trans->target[i].yCoord);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d PR Y", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "y coordinate", "");
            sprintf (messageBuffer, "%0.2f", trans->target[i].dxCoord);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d PR DX", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Shift x coordinate", "");
            sprintf (messageBuffer, "%0.2f", trans->target[i].dyCoord);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d PR DY", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Shift y coordinate", "");
            sprintf (messageBuffer, "%0.4f", trans->integTime[i]);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d PR EXT", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Integration time", "");
            sprintf (messageBuffer, "%0.2f", trans->meanFlux[i]);
            sprintf (textBuff, "HIERARCH ESO QC TRN F%d PR CNT", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Mean flux", "");
            if (i == 3 || i == 4 || i == 5)
            {
                sprintf (messageBuffer, "%0.6f", trans->transmission[i-3]);
                sprintf (textBuff, "HIERARCH ESO QC TRN F%d PR TRANS", i+1);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transmissivity", "");
            }
        }
    }

    //    Place Holders
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH1", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH2", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH3", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH4", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH5", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH6", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH7", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH8", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH9", "TBD", "TBD", "");
    
    //    Create the output fits file. If the file exist delete it
    if (stat (fileNames->outFitsName, &buf) == 0) remove (fileNames->outFitsName);

    outFitsPtr = fopen (fileNames->outFitsName, "w");
    if (!outFitsPtr)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create output FITS file");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        free (sWidthX);
        free (sWidthY);        
        return;
    }

   cpl_msg_info(cpl_func,"Created Product FITS file: %s \n", fileNames->outFitsName);
    fprintf (midiReportPtr, "Created Product FITS file: %s \n", fileNames->outFitsName);

    //    Write header into the output file
    qfits_header_sort (&outFitsHeader);
    qfits_header_dump (outFitsHeader, outFitsPtr);
    fclose (outFitsPtr);
    qfits_header_destroy (outFitsHeader);

    //    release memory
    free (messageBuffer);
    free (currentTime);
    free (cleanString);
    free (stringTemp);
    free (classification);
    free (tech);
    free (textBuff);
    free (sWidthX);    
    free (sWidthY);

    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    createDspTrnQcLog
* Input/Output:    See function arguments to avoid duplication
* Description:    Copies keywords from the raw input FITS primary header to the 
*               QC log and adds additional product keywords
*
* History:        
* 30-Aug-04        (csabet) Created
******************************************************************************/
void createDspTrnQcLog (
    char    *inFitsName,
    int      *error)
    
{

    /*    Local Declarations
    --------------------*/
    const char         routine[] = "createDspTrnQcLog";
    char            *qfitsString, *emptyString, *cleanString;

    /*    Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /* Reset status */
    *error = 0;

    /* Allocate memory */
    emptyString = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    /*    Initialise empty string */
    strcpy (emptyString, "\"UNKNOWN\"");

    /* Get all the keywords */
    qfitsString = qfits_query_hdr (inFitsName, "ARCFILE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ARCFILE             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ARCFILE             \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "TELESCOP");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "TELESCOP            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "TELESCOP            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "INSTRUME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INSTRUME            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INSTRUME            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "MJD-OBS");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "MJD-OBS             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "MJD-OBS             \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "DATE-OBS");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DATE-OBS            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DATE-OBS            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "UTC");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "UTC                 %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "UTC                 %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "LST");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "LST                 %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "LST                 %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET CHIP NX");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.CHIP.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.CHIP.NX         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET CHIP NY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.CHIP.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.CHIP.NY         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET DIT");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.DIT             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.DIT             %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET DITDELAY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.DITDELAY        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.DITDELAY        %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET INT MODE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.INT.MODE        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.INT.MODE        \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET NDIT");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.NDIT            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.NDIT            %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET NRTS MODE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.NRTS.MODE       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.NRTS.MODE       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN1 NX");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN1.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN1.NX         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN1 NY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN1.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN1.NY         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN2 NX");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN2.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN2.NX         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN2 NY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN2.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN2.NY         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR CATG");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DPR.CATG            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DPR.CATG            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR TECH");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DPR.TECH            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DPR.TECH            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR TYPE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DPR.TYPE            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DPR.TYPE            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS CAM NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.CAM.NAME        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.CAM.NAME        \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS FIL NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.FILT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.FILT.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS GRIS NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.GRIS.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.GRIS.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS MODE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.MODE            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.MODE            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS OPT1 NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.OPT1.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.OPT1.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS OPT1 TYPE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.OPT1.TYPE       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.OPT1.TYPE       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS SHUT NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.SHUT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.SHUT.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS SLIT NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.SLIT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.SLIT.NAME       \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.ID              %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.ID              \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.NAME            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.NAME            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PI-COI ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.PI-COI.ID       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.PI-COI.ID       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PI-COI NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.PI-COI.NAME     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.PI-COI.NAME     \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PROG ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.PROG.ID         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.PROG.ID         \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.START           %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.START           \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS TARG NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.TARG.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.TARG.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OCS EXPO1 FNAME1");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OCS.EXPO1.FNAME1    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OCS.EXPO1.FNAME1    \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO TPL ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "TPL.ID              %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "TPL.ID              \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO TPL START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "TPL.START           %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "TPL.START           \"%s\" \n", cleanString);}
    
    
    /* Release memory */
    free (emptyString);
    free (cleanString);
    
    return;
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    addProdInfoToDspTrnQcLog
* Input/Output:    See function arguments to avoid duplication
* Description:    Adds product information to QC log
*
*
* History:        
* 16-Jan-04        (csabet) Created
******************************************************************************/
void addProdInfoToDspTrnQcLog (
    ImageFormat        *format,    // In: Image format
    MidiFiles        *fileNames,
    DispersiveTrans    *trans,// In: Dispersive transmission data structure
    int                *error)
{

    /*    Local Declarations
    --------------------*/
    const char         routine[] = "addProdInfoToDspTrnQcLog";
    int             i;
    FILE            *inFitsBatchPtr;
    char            *stringQfits, *messageBuffer, *currentTime, *textBuff, *cleanString,
                    *stringTemp, *classification;
    time_t          now;
    struct tm        *newTime;
    
    /*    Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /* Reset parameters */
    *error = 0;

    /*    Allocate memory */
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    textBuff = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    messageBuffer = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    
    /*    Get current time/date */
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    //    Add keyword to QC log file
    fprintf (midiQcLogPtr, "PRO.TYPE            \"%s\" \n", format->obsType);
    fprintf (midiQcLogPtr, "PRO.CATG            \"REDUCED_DSPTRN\" \n");
    fprintf (midiQcLogPtr, "PRO.ARCFILE         \"%s\" \n", fileNames->archFileName);
    fprintf (midiQcLogPtr, "PRO.PIPEDATE        \"%s\" \n", currentTime);
    fprintf (midiQcLogPtr, "PRO.VERSION         \"%s\" \n", MIDI_PIPE_VERSION);
    fprintf (midiQcLogPtr, "PRO.PIPEFILE        \"%s\" \n", fileNames->pipeFileName);
    fprintf (midiQcLogPtr, "PRO.DID             \"%s\" \n", MIDI_QC_DIC_VERSION);

    /*  Open the list of files */
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    /* Loop through the list of files and write into the QC log */
    i = 1;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", messageBuffer, classification);
        
        /* If classification is not given get it from the raw file */
        if (strcmp (classification, "") == 0)
        {
            stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR CATG");
            if (stringQfits == NULL) 
            {
                sprintf (classification, "%s", "UNKNOWN");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Category");
            }
            else
            {
                cleanUpString (stringQfits, cleanString);
                sprintf (classification, "%s", cleanString);
            }
        }
        removePathName (messageBuffer, midiReportPtr);
        fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.NAME  \"%s\" \n", i, messageBuffer);
        fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.CATG  \"%s\" \n", i++, classification);
    }
    fclose (inFitsBatchPtr);

    for (i = 0; i < trans->numOfFiles; i++)
    {
        if (strcmp (trans->grismId[i], "OPEN") == 0)
        {
            fprintf (midiQcLogPtr, "QC.TRN.F%d.IM.NAME \"%s\" \n", i+1, trans->grismId[i]);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.IM.FIL \"%s\" \n", i+1, trans->filterName[i]);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.IM.X   %0.2f \n", i+1, trans->target[i].xCoord);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.IM.Y   %0.2f \n", i+1, trans->target[i].yCoord);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.IM.DX  %0.2f \n", i+1, trans->target[i].dxCoord);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.IM.DY  %0.2f \n", i+1, trans->target[i].dyCoord);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.IM.EXT %0.4f \n", i+1, trans->integTime[i]);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.IM.CNT %0.2f \n", i+1, trans->meanFlux[i]);
        }
        else if (strcmp (trans->grismId[i], "PRISM") == 0)
        {
            fprintf (midiQcLogPtr, "QC.TRN.F%d.PR.NAME \"%s\" \n", i+1, trans->grismId[i]);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.PR.FIL \"%s\" \n", i+1, trans->filterName[i]);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.PR.X   %0.2f \n", i+1, trans->target[i].xCoord);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.PR.Y   %0.2f \n", i+1, trans->target[i].yCoord);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.PR.DX  %0.2f \n", i+1, trans->target[i].dxCoord);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.PR.DY  %0.2f \n", i+1, trans->target[i].dyCoord);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.PR.EXT %0.4f \n", i+1, trans->integTime[i]);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.PR.CNT %0.2f \n", i+1, trans->meanFlux[i]);
            if (i == 3 || i == 4 || i == 5) 
                fprintf (midiQcLogPtr, "QC.TRN.F%d.PR.TRANS %0.6f \n", i+1, trans->transmission[i-3]);
        }
        else if (strcmp (trans->grismId[i], "GRISM") == 0)
        {
            fprintf (midiQcLogPtr, "QC.TRN.F%d.GR.NAME \"%s\" \n", i+1, trans->grismId[i]);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.GR.FIL \"%s\" \n", i+1, trans->filterName[i]);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.GR.X   %0.2f \n", i+1, trans->target[i].xCoord);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.GR.Y   %0.2f \n", i+1, trans->target[i].yCoord);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.GR.DX  %0.2f \n", i+1, trans->target[i].dxCoord);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.GR.DY  %0.2f \n", i+1, trans->target[i].dyCoord);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.GR.EXT %0.4f \n", i+1, trans->integTime[i]);
            fprintf (midiQcLogPtr, "QC.TRN.F%d.GR.CNT %0.2f \n", i+1, trans->meanFlux[i]);
            if (i == 3 || i == 4 || i == 5) 
                fprintf (midiQcLogPtr, "QC.TRN.F%d.GR.TRANS %0.6f \n", i+1, trans->transmission[i-3]);
        }
        else
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown Dispersive element");
            *error = 1;
            free (messageBuffer);
            free (currentTime);
            free (cleanString);
            free (stringTemp);
            free (classification);
            free (textBuff);
            return;
        }
    }

    /*    release memory */
    free (messageBuffer);
    free (currentTime);
    free (textBuff);
    free (cleanString);
    free (stringTemp);
    free (classification);
        
    return;
}
/*****************************************************************************/

