/******************************************************************************
*******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software
*
* Module name:    fft.c
* Description:    This is the new COMPILER INDEPENDENT fft module for NRTS etc.
*
* History:        
* 03-Feb-03        (jmeisner) Created
* 03-Mar-03        (csabet) Added title, directives and included in the MIDI pipeline
*******************************************************************************
******************************************************************************/

/******************************************************************************
*    Compiler directives
******************************************************************************/

/******************************************************************************
*    Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include <stdlib.h>
#include <math.h>
#include "complex_midi.h"
#include "compiler.h"
#include "fft.h"

/**********************************************************
*    Global Variables 
**********************************************************/
//    This number (power of 2) is the maximum array size supported:
//    NOW IN FFT.H:    #define MAXFFTSIZE 1024
//    was called: BIGGER 256   #define BIGGER2 BIGGER/2
int FFTsize = 2;           //    (Formerly XX)
int FFThalfsize = 1;    //    (Formerly XX2)
int FFTlevel = 1;        //    FFTsize = 2^FFTlevel
int shuffle[MAXFFTSIZE];

//    New, array in which FFT is processed, and results delivered:
struct Complex *FFTarray ; // Changed to dynamic memory (csabet 11-Jan-2007)

float ScaleFFTby;    // Applies to entire fft
                    // REVERSED: Now ScaleFFTby is used by the FFT and NOT by the IFT!! 
                    // Only used for IFT, ==1 if forward FT

struct Complex Twiddle[MAXFFTSIZE/2];    // Now always set for forward xfrm
struct Complex Rotated;
// Now local to DoFFT:   int skip,groups,TwiddleIndex;



/*============================ C O D E    A R E A ===========================*/

// This function now returns all bit reversals
// First, the QC x86 inline assembly code:

#ifdef QUICK_C /*---------------------------------------*/

static int BitReverse(int x,int NumBits)
{
    _asm
    {
        mov cx, NumBits;
        mov dx, x;
        sub ax, ax;
TOP:
        rcr dx,1;
        rcl ax,1;
        loop TOP;
        }
}          // Ignore compiler warning about "no return value"
#else /*---------------------------------------*/

static int BitReverse(int x, int NumBits)
{
   int result = 0;
   int i;
   for(i=0; i<NumBits; i++)
   {
      result *= 2;
      if(x&1)
     result++;
      x /= 2 ;
   }
   return result;
}
#endif /*---------------------------------------*/


// used to initialize the shuffled index
// and twiddle arrays:
void FFTInit(int level) // Now only runs when needed!!    // no longer:  , int inverse)
{
   int i;
   float angle=0.F, incr;

   if(level==FFTlevel)  // If already correctly set...
      return;           // then nothing to do!!

   FFTlevel = level;

   FFTsize = 1 << level;
   FFThalfsize = FFTsize/2 ;

   if(FFTsize>MAXFFTSIZE)
      ERROR(-111)  ; // not!: "requested FFT level exceeds max!");

// NEW NORMALIZATION SCHEME, Constant division ONLY for INVERSE xfrm:
// Now, only USED by IFT, set regardless:

//   if(inverse) // REVERSED: Now ScaleFFTby is used by the FFT and NOT by the IFT!!

   ScaleFFTby = 1.F/((double) FFTsize) ;

//   else
//      ScaleFFTby = 1.F;

//   Num2shuffle = 0;

   for (i=0; i<FFTsize; i++)
   {
      shuffle[i] = BitReverse(i, level);
   }

   incr = -6.2831853F / ((float) FFTsize) ;  //  (1 << level) ;

// NAH:    if(inverse) incr = -incr ;

   for (i=0;i < FFThalfsize; i++) // this array needs only be 1/2 the array size
   {
       GetEtoJtimes(angle, &Twiddle[i]);
       angle += incr;
   }
}

           // FOR FORWARD XFRM:
static void ButterflyXfwd (int AIndex, int BIndex, int TwiddleIndex,
                    struct Complex arr1[])
//  AIndex and BIndex are indices pointing to which elements
//  are to be butterflied
{
    MultiplyCptr(&Rotated, &arr1[BIndex],  &Twiddle[TwiddleIndex] );
    SubtractCptr( &arr1[BIndex] , &arr1[AIndex] , &Rotated);
    AddCptr( &arr1[AIndex] , &arr1[AIndex] , &Rotated);
}
           // FOR INVERSE XFRM:
static void ButterflyXinv (int AIndex, int BIndex, int TwiddleIndex,
                    struct Complex arr1[])
//  AIndex and BIndex are indices pointing to which elements
//  are to be butterflied
{
    MultiplyConjPtr(&Rotated, &arr1[BIndex],  &Twiddle[TwiddleIndex] );
    SubtractCptr( &arr1[BIndex] , &arr1[AIndex] , &Rotated);
    AddCptr( &arr1[AIndex] , &arr1[AIndex] , &Rotated);
}

     // For type2 algorithm:
static void Type2ButterflyXinv (int AIndex, int BIndex, int TwiddleIndex,
                    struct Complex arr1[])
//  AIndex and BIndex are indices pointing to which elements
//  are to be butterflied
{
    struct Complex temp;
    SubtractCptr(&temp,  &arr1[AIndex], &arr1[BIndex]);  // To become a index
    AddCptr(&arr1[AIndex], &arr1[AIndex] , &arr1[BIndex]);
    MultiplyConjPtr(&arr1[BIndex], &temp, &Twiddle[TwiddleIndex] );
}



void DoFFT(void) // This runs the actual math, after data is in FFTarray:
{   // REVERSED: Now ScaleFFTby is used by the FFT and NOT by the IFT!! (must be done by calling program)
   int i, j, k;
   int skip=1;
   int groups = FFTsize ;
   int TwiddleIndex, IndexA, IndexB;
   // these three nested loops are the core of the FFT

   for (k=0;k<FFTlevel;k++)  // loop #1  for all the levels
   {
   /******** Now these are set differently:
      skip= (1<<k); //         (int)pow(2,k);
      groups= FFTsize/(2*skip); //  why  divided by 2 ?  because for a 16 element -
   *************/                    //  the first one is 8(groups) x 1(skip)
      groups /= 2 ; //  <<<< Here for groups

      // loop #2 --"group" loop -- XX/2 at first,  1 at the end
      for (j=0;j<groups;j++)    //= skip * 2)
      {
     TwiddleIndex=0;
     IndexA = j * skip * 2;
     IndexB = IndexA + skip;

          // and here is loop # 3 - twiddle loops for each group
     for (i=0;i<skip;i++)
     {
        ButterflyXfwd(IndexA++, IndexB++, TwiddleIndex, FFTarray);
    // Was: j * skip * 2 +i, j * skip * 2 + skip + i , TwiddleIndex, FFTarray);
        TwiddleIndex+= groups;
     }
      }
      skip *= 2 ; /// <<<< And here!
   }
}


void DoIFT(void) //  Identical to above, except uses ButterflyXinv
{            // IFT must also (in calling program) apply the scale ScaleFFTby  <<< NOT
   int i, j, k;
   int skip=1;
   int groups = FFTsize ;
   int TwiddleIndex, IndexA, IndexB;
   // these three nested loops are the core of the FFT

   for (k=0;k<FFTlevel;k++)  // loop #1  for all the levels
   {
      groups /= 2 ; //  <<<< Here for groups

      // loop #2 --"group" loop -- XX/2 at first,  1 at the end
      for (j=0;j<groups;j++)    //= skip * 2)
      {
     TwiddleIndex=0;
     IndexA = j * skip * 2;
     IndexB = IndexA + skip;

          // and here is loop # 3 - twiddle loops for each group
     for (i=0;i<skip;i++)
     {
        ButterflyXinv(IndexA++, IndexB++, TwiddleIndex, FFTarray);
        TwiddleIndex+= groups;
     }
      }
      skip *= 2 ; /// <<<< And here!
   }
}


void Type2DoIFT(void) //  NEW!   NEW!
{            // IFT must also (in calling program) apply the scale ScaleFFTby <<< NOT
   int i, j, k;
   int skip = FFTsize;   // BACKWARDS: =1;
   int groups = 1;   // not: FFTsize ;
   int TwiddleIndex, IndexA, IndexB;
   // these three nested loops are the core of the FFT

   for (k=0;k<FFTlevel;k++)  // loop #1  for all the levels
   {
      skip /= 2;  // <<<< HERE now
 // not:     groups /= 2 ; //  <<<< Here for groups
   // but, below:   groups *= 2;

      // BACKWARDS:  // loop #2 --"group" loop -- XX/2 at first,  1 at the end
      for (j=0;j<groups;j++)    //= skip * 2)
      {
     TwiddleIndex=0;
     IndexA = j * skip * 2;
     IndexB = IndexA + skip;

          // and here is loop # 3 - twiddle loops for each group
     for (i=0;i<skip;i++)
     {
        Type2ButterflyXinv(IndexA++, IndexB++, TwiddleIndex, FFTarray);
        TwiddleIndex+= groups;
     }
      }
     // BACKWARDS:  skip *= 2 ; /// <<<< And here!
   // but above:   skip /= 2;
      groups *= 2;  // <<< NOW HERE
   }
}

// USER LEVEL FUNCTIONS:

void FFT(struct Complex *Input, int newlevel)       // Vector1[], int level)
{
    int i;

    FFTInit(newlevel);

    for(i=0; i<FFTsize; i++)  // Shuffle the data:
    {
       FFTarray[shuffle[i]] = Scalec(Input[i], ScaleFFTby); // rather than Input[i] ;
    }

    DoFFT();   // <<< And actually do it!
}

void IFT(struct Complex *Input, int newlevel)       // Vector1[], int level)
{
    int i;

    if(newlevel != FFTlevel)

    for(i=0; i<FFTsize; i++)  // Shuffle the data:
    {                     // AND SCALE since its the IFT:
       FFTarray[shuffle[i]] = Input[i] ; // NO LONGER: Scalec(Input[i], ScaleFFTby);
    }

    DoIFT();   // <<< And actually do it!
}

/* static void Type2IFT(struct Complex *Output, int newlevel)       // Vector1[], int level) */
/* { */
/*     int i; */

/*     FFTInit(newlevel); */

/*         // Input in fftarray, so... */
/*     Type2DoIFT();   // <<< And actually do it! */

/*     for(i=0; i<FFTsize; i++)  // Shuffle the data: */
/*     {                     // AND SCALE since its the IFT: */
/*        Output[shuffle[i]] = FFTarray[i] ; // No longer: Scalec(FFTarray[i], ScaleFFTby);  */
/*     } */
/* } */

/* static void Type2IFTtoReal(float *Output, int newlevel)       // Vector1[], int level) */
/* { */
/*     int i; */

/*     FFTInit(newlevel); */

/*         // Input in fftarray, so... */
/*     Type2DoIFT();   // <<< And actually do it! */

/*     for(i=0; i<FFTsize; i++)  // Shuffle the data: */
/*     {                     // AND SCALE since its the IFT: */
/*        Output[shuffle[i]] = FFTarray[i].r ; // NO LONGER:   * ScaleFFTby; */
/*     } */
/* } */

/*********** Set FFTlevel & init so just large enough for InputSize *******/
void SetFFTsize(int InputSize)
{
    int newFFTlevel = FFTlevel;

    while(InputSize > FFTsize)
    {
       newFFTlevel++;
       FFTsize *= 2;
    }

    while(2*InputSize <= FFTsize)
    {
       newFFTlevel--;
       FFTsize /= 2;
    }
    
    FFTInit(newFFTlevel);   
}

/************ FURTHER USER LEVEL FUNCTIONS ****************/

void FFTofReal(float *Input, int InputSize)   // Enter size of array to be xfrmd
{
    int i;
    SetFFTsize(InputSize);

    for(i=0; i<InputSize; i++)
    {
       FFTarray[shuffle[i]].r = Input[i]*ScaleFFTby; // YES NEEDED // NOT NEEDED, =1 for forward FFT: *ScaleFFTby;
       FFTarray[shuffle[i]].i = 0.F;
    }

    for(i=InputSize; i<FFTsize; i++)
    {
       FFTarray[shuffle[i]].r = FFTarray[shuffle[i]].i = 0.F;
    }

    DoFFT();
}


// For this version, FFTlevel/size must be set in advance & FFTinit() done:
// Produces even (symetric) function over full range & produces cosine xfrm

void FFTfullReal(float *Input, int InputSize, float scale)   // Enter size of array to be xfrmd
{
    int i;
    struct Complex Ctemp;    
// NOW A GLOBAL VAR:        int FFThalfsize = FFTsize/2;
    float inputscale = ((float)InputSize) /((float)FFThalfsize) ;  
    
    scale *= ScaleFFTby ; // Include this factor here, now.

    Ctemp.i = 0.F;

// NO LONGER NEEDED:        if(FFTinverseMode)  // in case it got switched somehow!!
//           FFTInit(FFTlevel, 0);

    for(i=0; i<FFThalfsize; i++)
    {
       Ctemp.r = scale*Input[((int) (((float)i) * inputscale))] ;

// SHOULD BE:
       FFTarray[shuffle[i]] = FFTarray[shuffle[FFTsize-i]] = Ctemp;
    }
     // NEW:
    FFTarray[1].r = scale*Input[InputSize-1] ; // To make continuous!!
    FFTarray[1].i = 0.F ; // This is the FFThalfsize pt.

    DoFFT();
}


    // Does NOT attempt to reduce the size of the fft array!!:
    // Should have run SetFFTsize() first!
/* static void FFTofShortArray(struct Complex *Input, int InputSize) */
/* { */
/*     int i; */

/*     for(i=0; i<InputSize; i++) */
/*     { */
/*        FFTarray[shuffle[i]] = Scalec(Input[i], ScaleFFTby);  */
/*     } */

/*     for(i=InputSize; i<FFTsize; i++) */
/*     { */
/*        FFTarray[shuffle[i]].r = FFTarray[shuffle[i]].i = 0.F; */
/*     } */

/*     DoFFT(); */
/* } */

/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  determineFFTsize
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 01-Sep-03     (csabet) Integrated
* 17-Jul-03     (jmeisner) Completed the code
******************************************************************************/
int determineFFTsize (
    int    framesperscan)
{
    int  result=1;

    while ((0.7F * ((float) result)) < ((float) framesperscan))
        result = 2 * result;
    return result;
}
/*****************************************************************************/

