/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*               VLTI Data Reduction Software
*
* File name:    fileHanding.c
* Description:  Contains the routines for general file handing
*               and freeing
*
*
* History:
* 05-Jul-04     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <math.h>
#include <unistd.h>
#include <stdio.h>
#include <cpl.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "errorHandling.h"
#include "fileHandling.h"
#include "qfits.h"

/******************************************************************************
*   Global Variables
******************************************************************************/

/******************************************************************************
*   Prototypes
******************************************************************************/

/*============================ C O D E    A R E A ===========================*/





/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  removePathName
* Input/Output: See function arguments to avoid duplication
* Description:  Returns a clean file name without the path name
*
*
* History:
* 26-Nov-03     (csabet) Created
******************************************************************************/
void removePathName(
    char    *fileName,  /*  IO: Name of string possibly containig path names */
    FILE    *filePtr)   /*  In: Pointer to an open temporary file */
{
    /*  Local Declarations
    --------------------*/
    const char      routine[] = "removePathName";
    FILE            *temp1Ptr, *temp2Ptr;
    size_t          stringLength;
    unsigned int    i, indexCount = 0;
    char            nextChar;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (filePtr, "Invoking      routine   '%s' \n", routine);
    

    stringLength = strlen (fileName);
    temp1Ptr = fopen("temp1.grb", "w");
    fprintf (temp1Ptr, "%s\n", fileName);
    fclose (temp1Ptr);
    temp1Ptr = fopen("temp1.grb", "r");
    for (i = 0; i < stringLength; i++)
    {
        int ret = fscanf (temp1Ptr, "%c", &nextChar);
        cpl_msg_debug(cpl_func,"fscanf returned %d input items", ret);
        if (memcmp (&nextChar, "/", 1) == 0) indexCount = i+1;
    }
    rewind (temp1Ptr);
    temp2Ptr = fopen("temp2.grb", "w");
    for (i = 0; i < stringLength; i++)
    {
        nextChar = fgetc (temp1Ptr);
        if (i >= indexCount) fputc (nextChar, temp2Ptr);
    }
    fclose (temp1Ptr);
    fclose (temp2Ptr);
    temp2Ptr = fopen("temp2.grb", "r");
    int ret = fscanf (temp2Ptr, "%s\n", fileName);
    cpl_msg_debug(cpl_func,"fscanf returned %d input items", ret);
    fclose (temp2Ptr);
    remove ("temp1.grb");
    remove ("temp2.grb");

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  getFirstCharacter
* Input/Output: See function arguments to avoid duplication
* Description:  Returns the first character of a string
*
*
* History:
* 13-Jan-04     (csabet) Created
******************************************************************************/
char getFirstCharacter (    /*  Ou: First character of the input string */
    char    *string)        /*  In: String to be examined */
{
    /*  Local Declarations
    --------------------*/
    const char      routine[] = "getFirstCharacter";
    char            firstChar;
    FILE            *tempPtr;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    
    if (string == NULL) return (' ');

    tempPtr = fopen ("getFirstChar.grb", "w");
    fprintf (tempPtr, "%s\n", string);
    fclose (tempPtr);
    tempPtr = fopen ("getFirstChar.grb", "r");
    int ret = fscanf (tempPtr, "%c", &firstChar);
    cpl_msg_debug(cpl_func,"fscanf returned %d input items", ret);
    fclose (tempPtr);
    remove ("getFirstChar.grb");

    return (firstChar);
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  getFitsNames
* Input/Output: See function arguments to avoid duplication
* Description:  It looks into a given directory and writes names of FITS files
*               into an output file
*
* History:
* 17-Oct-03     (csabet) Created
******************************************************************************/
void getFitsNames (
    MidiFiles   *fileNames, /*  IO: Pointer to midi files structure */
    FILE        *filePtr)
{
    /*  Local Declarations
    --------------------*/
    const char  routine[] = "getFitsNames";
    char        *argument;
    FILE        *inFitsClassifiedPtr;
    struct stat buf;
    int         numOfFiles;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (filePtr, "Invoking      routine   '%s' \n", routine);

    /*  Allocate memory */
    argument = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    /*  Remove existing list */
    if (stat (fileNames->inFitsClassified, &buf) == 0)  /*  If the file exist delete it */
        remove (fileNames->inFitsClassified);

/*  TO_DO_MIDI_DRS  This section shall be replaced by an appropriate C code. (csabet 26-Oct-03) */
    strcat (argument, "ls -1 ");
    strcat (argument, fileNames->inFileDir);
    strcat (argument, "MIDI*.fits ");
    strcat (argument, "> ");
    strcat (argument, fileNames->inFitsClassified);
    int ret = system (argument);
    if (ret != 0) {
         cpl_msg_warning(cpl_func,"Failed to call the system command");
     }

    /*  Open the file for reading */
    inFitsClassifiedPtr = fopen (fileNames->inFitsClassified, "r");
    numOfFiles = 0;
    while (fscanf (inFitsClassifiedPtr, "%s\n", argument) != EOF)
    {
        numOfFiles++;
    }
    if (numOfFiles == 0)
    {
        sprintf (midiMessage, "Cannot find any valid MIDI FITS files in ... %s", fileNames->inFileDir);
        midiReportError (filePtr, routine, __FILE__, __LINE__, midiMessage);
    }

    fclose (inFitsClassifiedPtr);

    /*  Release memory */
    free (argument);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  cleanUpString
* Input/Output: See function arguments to avoid duplication
* Description:  Returns a clean string without spaces and quotes
*
*
* History:
* 11-Nov-03     (csabet) Created
******************************************************************************/
void cleanUpString (
    char    *rawString,        // In: String to be cleaned up
    char    *cleanString)    // Ou: Cleaned up string
{
    /*  Local Declarations
    --------------------*/
    char    *string=NULL, *qfitsStr=NULL;
    
    //    Algorithm
    //-----------
    
    //    Allocate memory
    string = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    sprintf (string, "%s", rawString);
    qfitsStr = qfits_pretty_string (string);
    sprintf (cleanString, "%s", qfitsStr);

    //    Release memory
    free (string);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  removeNewLine
* Input/Output: See function arguments to avoid duplication
* Description:  Returns a clean string without new line
*
*
* History:
* 29-Apr-05     (csabet) Created
******************************************************************************/
void removeNewLine (
    char    *rawString,        // In: String to be cleaned up
    char    *cleanString)    // Ou: Cleaned up string
{
    /*  Local Declarations
    --------------------*/
    char    lastChar, nextChar;
    FILE    *tempPtr=NULL;
    int        i;
    char    *tempSrt=NULL;
    size_t    newLength, stringLength;
    
    //    Algorithm
    //-----------
    tempSrt = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringLength = strlen (rawString);    
    tempPtr = fopen ("temp.grb", "w");
    fprintf (tempPtr, "%s\n", rawString);
    fclose (tempPtr);
    tempPtr = fopen ("temp.grb", "r");

    newLength = 0;
    for (i = 0; i < (int)stringLength; i++)
    {
        nextChar = fgetc (tempPtr);
        if (nextChar == '\n' || nextChar == ' ') break;
        lastChar = nextChar;
        newLength++;
    }
    if (lastChar == ' ') newLength--;
    fclose (tempPtr);
    strncpy (tempSrt, rawString, newLength);
    sprintf (cleanString, "%s", tempSrt);
    remove ("temp.grb");

    free (tempSrt);
    
    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createQcLog
* Input/Output: See function arguments to avoid duplication
* Description:  Copies keywords from the raw input FITS primary header to the
*               QC log
*
* History:
* 23-Feb-04     (csabet) Created
******************************************************************************/
void createQcLog (
    char    *inFitsName,
    int        *error)

{

    /*  Local Declarations
    --------------------*/
    const char      routine[] = "createQcLog";
    char            *qfitsString, *emptyString, *cleanString;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /* Reset status */
    *error = 0;

    /* Allocate memory */
    emptyString = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    /*  Initialise empty string */
    strcpy (emptyString, "\"UNKNOWN\"");

    /* Get all the keywords */
    qfitsString = qfits_query_hdr (inFitsName, "ARCFILE");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ARCFILE             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ARCFILE             \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "TELESCOP");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "TELESCOP            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "TELESCOP            \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "INSTRUME");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "INSTRUME            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "INSTRUME            \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "MJD-OBS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "MJD-OBS             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "MJD-OBS             \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "DATE-OBS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DATE-OBS            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DATE-OBS            \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "UTC");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "UTC                 %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "UTC                 %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "LST");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "LST                 %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "LST                 %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU GUID DEC");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.GUID.DEC        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.GUID.DEC        %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU GUID RA");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.GUID.RA         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.GUID.RA         %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU GUID MAG");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "COU.GUID.MAG        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "COU.GUID.MAG        %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 STREHL_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.STREHL_MEAN %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.STREHL_MEAN %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 STREHL_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.STREHL_MEAN %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.STREHL_MEAN %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 STREHL_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.STREHL_RMS  %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.STREHL_RMS  %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 STREHL_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.STREHL_RMS  %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.STREHL_RMS  %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 WFE_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.WFE_MEAN    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.WFE_MEAN    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 WFE_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.WFE_MEAN    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.WFE_MEAN    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 WFE_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.WFE_RMS     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.WFE_RMS     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 WFE_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.WFE_RMS     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.WFE_RMS     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 ENC_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.ENC_MEAN    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.ENC_MEAN    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 ENC_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.ENC_MEAN    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.ENC_MEAN    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 ENC_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.ENC_RMS     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.ENC_RMS     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 ENC_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.ENC_RMS     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.ENC_RMS     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 FWHM_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.FWHM_MEAN   %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.FWHM_MEAN   %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 FWHM_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.FWHM_MEAN   %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.FWHM_MEAN   %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 FWHM_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.FWHM_RMS    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.FWHM_RMS    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 FWHM_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.FWHM_RMS    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.FWHM_RMS    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 L0_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.L0_MEAN     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.L0_MEAN     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 L0_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.L0_MEAN     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.L0_MEAN     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 L0_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.L0_RMS      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.L0_RMS      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 L0_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.L0_RMS      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.L0_RMS      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 R0_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.R0_MEAN     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.R0_MEAN     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 R0_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.R0_MEAN     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.R0_MEAN     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 R0_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.R0_RMS      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.R0_RMS      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 R0_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.R0_RMS      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.R0_RMS      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 T0_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.T0_MEAN     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.T0_MEAN     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 T0_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.T0_MEAN     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.T0_MEAN     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 T0_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.T0_RMS      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.T0_RMS      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 T0_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.T0_RMS      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.T0_RMS      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET CHIP NX");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DET.CHIP.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DET.CHIP.NX         %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET CHIP NY");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DET.CHIP.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DET.CHIP.NY         %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET DIT");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DET.DIT             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DET.DIT             %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET DITDELAY");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DET.DITDELAY        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DET.DITDELAY        %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET INT MODE");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DET.INT.MODE        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DET.INT.MODE        \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET NDIT");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DET.NDIT            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DET.NDIT            %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET NRTS MODE");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DET.NRTS.MODE       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DET.NRTS.MODE       \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN1 NX");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DET.WIN1.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DET.WIN1.NX         %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN1 NY");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DET.WIN1.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DET.WIN1.NY         %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN2 NX");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DET.WIN2.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DET.WIN2.NX         %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN2 NY");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DET.WIN2.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DET.WIN2.NY         %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR CATG");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DPR.CATG            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DPR.CATG            \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR TECH");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DPR.TECH            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DPR.TECH            \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR TYPE");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "DPR.TYPE            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "DPR.TYPE            \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS CAM NAME");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "INS.CAM.NAME        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "INS.CAM.NAME        \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS FILT NAME");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "INS.FILT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "INS.FILT.NAME       \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS GRIS NAME");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "INS.GRIS.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "INS.GRIS.NAME       \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS MODE");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "INS.MODE            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "INS.MODE            \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS OPT1 NAME");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "INS.OPT1.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "INS.OPT1.NAME       \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS OPT1 TYPE");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "INS.OPT1.TYPE       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "INS.OPT1.TYPE       \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS SHUT NAME");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "INS.SHUT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "INS.SHUT.NAME       \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS SLIT NAME");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "INS.SLIT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "INS.SLIT.NAME       \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AIRM END");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.AIRM.END        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.AIRM.END        %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AIRM START");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.AIRM.START      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.AIRM.START      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS ALT");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.ALT             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.ALT             %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI FWHM END");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.AMBI.FWHM.END   %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.AMBI.FWHM.END   %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI FWHM START");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.AMBI.FWHM.START %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.AMBI.FWHM.START %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI PRES");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.AMBI.PRES       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.AMBI.PRES       %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI RHUM");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.AMBI.RHUM       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.AMBI.RHUM       %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI TAU0 END");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.AMBI.TAU0.END   %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.AMBI.TAU0.END   %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI TAU0 START");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.AMBI.TAU0.START %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.AMBI.TAU0.START %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI TEMP");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.AMBI.TEMP       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.AMBI.TEMP       %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI WINDDIR");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.AMBI.WINDDIR    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.AMBI.WINDDIR    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI WINDSP");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.AMBI.WINDSP     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.AMBI.WINDSP     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AZ");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.AZ              %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.AZ              %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF STATION1");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.CONF.STATION1   %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.CONF.STATION1   \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF STATION2");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.CONF.STATION2   %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.CONF.STATION2   \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF T1X");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.CONF.T1X        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.CONF.T1X        %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF T1Y");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.CONF.T1Y        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.CONF.T1Y        %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF T1Z");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.CONF.T1Z        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.CONF.T1Z        %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF T2X");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.CONF.T2X        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.CONF.T2X        %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF T2Y");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.CONF.T2Y        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.CONF.T2Y        %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF T2Z");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.CONF.T2Z        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.CONF.T2Z        %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS PARANG END");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.PARANG.END      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.PARANG.END      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS PARANG START");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.PARANG.START    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.PARANG.START    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS PBL12 END");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.PBL12.END       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.PBL12.END       %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS PBL12 START");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "ISS.PBL12.START     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "ISS.PBL12.START     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS ID");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "OBS.ID              %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "OBS.ID              \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS NAME");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "OBS.NAME            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "OBS.NAME            \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PI-COI ID");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "OBS.PI-COI.ID       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "OBS.PI-COI.ID       \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PI-COI NAME");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "OBS.PI-COI.NAME     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "OBS.PI-COI.NAME     \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PROG ID");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "OBS.PROG.ID         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "OBS.PROG.ID         \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS START");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "OBS.START           %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "OBS.START           \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS TARG NAME");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "OBS.TARG.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "OBS.TARG.NAME       \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OCS EXPO1 FNAME1");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "OCS.EXPO1.FNAME1    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "OCS.EXPO1.FNAME1    \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO TPL ID");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "TPL.ID              %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "TPL.ID              \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO TPL START");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "TPL.START           %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "TPL.START           \"%s\" \n", cleanString);}


    /* Release memory */
    free (emptyString);
    free (cleanString);

    return;
}
/*****************************************************************************/
