/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  midicplutils.c
* Description:  reads a table column into an imagelist
*
* History:
* 01-Mai-09     (agabasch) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/

#include "midi_cplutils.h"
#include <cpl.h>
#include <string.h>
#include "midi_cplupgrade.h"

int timetable_to_cpltable(const char * columname,
                          cpl_table * intable,
                          cpl_table * outtable)
{
   
   int              dimen_in=0, dimen_out=0;
   cpl_table      * dummy_table=NULL;
   cpl_errorstate   prestate = cpl_errorstate_get();
   
   /* Load extension  Imaging Data  */ 
   
   
   dimen_out=cpl_table_get_nrow(outtable);  	   
   dimen_in =cpl_table_get_nrow(intable);  	   
   
   dummy_table=cpl_table_new(dimen_in);
   cpl_table_duplicate_column(dummy_table, columname, intable, columname);
   cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());   

/*    cpl_table_dump_structure  	(intable,NULL);	 */
/*    cpl_table_dump_structure  	(outtable,NULL);	 */
/*    cpl_table_dump_structure  	(dummy_table,NULL);	 */

   cpl_table_insert(outtable, dummy_table, dimen_out+1);   
   if(dummy_table!=NULL){
      cpl_table_delete(dummy_table);
   }
   cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
   
   return (int)cpl_error_get_code();
}




int table_to_imglst(const char * columname,
                    cpl_imagelist * imglst,
                    cpl_table * table)
{
   
   int             dimenDATA=0; 
   int             i=0, counter=0;
   cpl_array     * array_data=NULL;
   cpl_image     * image_data_int=NULL;
   cpl_errorstate  prestate = cpl_errorstate_get();
   cpl_type        ctype;   
   
   /* Load extension  Imaging Data  */ 
   
   dimenDATA=cpl_table_get_column_dimensions(table, columname);
   cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
   if (dimenDATA != 2) {
      return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                        "DATA has a wrong dimension");
   }
   ctype=cpl_table_get_column_type(table, columname);   
/*    cpl_msg_info(cpl_func, "Type of the table column: %d",ctype); */
   
   counter=cpl_imagelist_get_size(imglst);


   for (i=0; i<cpl_table_get_nrow(table);i++){
      
      array_data=(cpl_array *)cpl_table_get_array(table,columname, i); 
      if(ctype&CPL_TYPE_INT){
         image_data_int=cpl_image_wrap_int(cpl_table_get_column_dimension(table,columname,0 ), cpl_table_get_column_dimension(table,columname,1 ), cpl_array_get_data_int(array_data) );
      }
      if(ctype&CPL_TYPE_FLOAT){
         image_data_int=cpl_image_wrap_float(cpl_table_get_column_dimension(table,columname,0 ), cpl_table_get_column_dimension(table,columname,1 ), cpl_array_get_data_float(array_data) );
      }
      /* Cast the image to float */
/*       image_data=cpl_image_cast(image_data_int, CPL_TYPE_FLOAT); */
      
/*        Append the image to the imagelists */
      
      cpl_imagelist_set(imglst,cpl_image_cast(image_data_int, CPL_TYPE_FLOAT),counter++);

      /*Unwrap processed image manualy if nor Target nor Sky*/
      if(image_data_int!=NULL){
         cpl_image_unwrap(image_data_int);
      }
      
      
   }
   
   

   return (int)cpl_error_get_code();
}
/*----------------------------------------------------------------------------*/
/**
  @brief    checks if the SOF contains fitsfiles of a certain tag
  @param    frameset  The recipe frameset 
  @param    tag       The tag
  @return   number of fitsfiles of a certain tag
 */
/*----------------------------------------------------------------------------*/

int midi_check_sof(cpl_frameset * frameset, const char * tag)
{
   char                  * tag_file=NULL;
   cpl_frame             * cur_frame=NULL;
   int                     NumberOfFiles=0;
   
   cur_frame = cpl_frameset_get_position(frameset, 0);
   if (cur_frame == NULL) {
      return (int)cpl_error_set_message(cpl_func, CPL_ERROR_DATA_NOT_FOUND,
                                        "SOF does not have any file");
   }
   
/*    Walk through the whole Set of Frames SOF */
   
   NumberOfFiles=0;   
   cpl_frameset_iterator * it = cpl_frameset_iterator_new(frameset);
   cur_frame = cpl_frameset_iterator_get(it);

   while(cur_frame)
   {
      tag_file = (char*)cpl_frame_get_tag(cur_frame);
      
      if (strcmp(tag_file, tag)) {
          cpl_frameset_iterator_advance(it, 1);
          cur_frame = cpl_frameset_iterator_get(it);
         continue;
      }
      if (strcmp(tag_file, tag)==0) {
         NumberOfFiles+=1;
      }
      cpl_frameset_iterator_advance(it, 1);
      cur_frame = cpl_frameset_iterator_get(it);
   }
   cpl_frameset_iterator_delete(it);
   return NumberOfFiles;
   
}
/*----------------------------------------------------------------------------*/
/**
  NOTE: same behavior as midi_polynomial_fit_1d_create() in cpl 5.0 but
  encapsulated as depreciated

  @brief    Fit a 1D-polynomial to a 1D-signal in a least squares sense
  @param    x_pos   Vector of positions of the signal to fit.
  @param    values  Vector of values of the signal to fit.
  @param    degree  Non-negative polynomial degree.
  @param    mse     Iff mse is not null, the mean squared error on success
  @return   The fitted polynomial or NULL on error
  @see cpl_polynomial_fit()
  @note In the absence of rounding-errors the fit will succeed if more than
        degree distinct data-points are provided, but in reality this may not
        hold. Secondly, an increase in the polynomial degree will normally
        reduce the mean squared error. However, due to rounding errors and
        the limited accuracy of the solver of the normal equations, an increase
        in the polynomial degree may at some point cause the mse to _increase_.
        In some cases this happens with an increase of the polynomial degree
        from 9 to 10. Deprecated: Use cpl_polynomial_fit().

  This function fits a 1D-polynomial to a 1D-signal in a least squares sense.

  The input signal is given in x_pos and values which must be of the same size.

  x_pos must contain more than degree distinct values.

  mse may be NULL. If it is not, *mse is set to the mean squared error on
  success, while it is unchanged on error.

  The fit is done in the following steps:
  1) x_pos is first transformed into xhat = x_pos - mean(x_pos).
  2) The normal equations of the Vandermonde matrix are formed from xhat.
  3) The normal equations are solved using Cholesky factorization.
  4) The resulting polynomial in xhat is transformed back to x_pos.

  The call requires 6MN + N^3/3 + 7/2N^2 + O(M) FLOPs where M is the number
  of data points and where N is the number of polynomial coefficients,
  N = degree + 1.

  Possible #_cpl_error_code_ set in this function:
  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  - CPL_ERROR_ILLEGAL_INPUT if degree is negative.
  - CPL_ERROR_DATA_NOT_FOUND if the size of x_pos is less than the number of
    coefficients to be determined, degree + 1
  - CPL_ERROR_INCOMPATIBLE_INPUT if x_pos and values have different sizes
  - CPL_ERROR_SINGULAR_MATRIX if x_pos contains too few distinct values.
 */
/*----------------------------------------------------------------------------*/

cpl_polynomial * midi_polynomial_fit_1d_create(
		const cpl_vector    *   x_pos,
		const cpl_vector    *   values,
		int                     degree,
		double              *   mse)
{
	cpl_polynomial * fit1d;
	cpl_matrix     * samppos;
	cpl_vector     * fitresidual;

	cpl_size degree_tmp=0;

	fit1d = cpl_polynomial_new(1);
	samppos = cpl_matrix_wrap(1, cpl_vector_get_size(x_pos),
				cpl_vector_get_data((cpl_vector *)x_pos));
	fitresidual = cpl_vector_new(cpl_vector_get_size(x_pos));
	cpl_polynomial_fit(fit1d, samppos, NULL, values, NULL,
			CPL_FALSE, NULL, &degree_tmp);

	degree=(int)(degree_tmp);

	cpl_vector_fill_polynomial_fit_residual(fitresidual, values, NULL, fit1d,
                                        samppos, NULL);
	cpl_matrix_unwrap(samppos);

	if(mse!=NULL){
		*mse = cpl_vector_product(fitresidual, fitresidual)
					/(double)(cpl_vector_get_size(fitresidual));
	}
	cpl_vector_delete(fitresidual);
	return fit1d;
}
