/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  procFrgSP.c
* Description:  Contains routines for Fringe Track data post processing
*
* History:
* 03-Feb-05     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "memoryHandling.h"
#include "errorHandling.h"
#include "calibration.h"
#include "fitsAnalysisFrgSP.h"
#include "imageProcessing.h"
#include "createProdFrg.h"
#include "transferFunction.h"
#include "visibility.h"
#include "diagnostics.h"
#include "estimationFrg.h"
#include "photometry.h"
#include "preProcFrgSP.h"
#include "procFrgSP.h"
#include "procFrgHS.h"
#include "statistics.h"

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Global Variables
**********************************************************/

/*============================ C O D E    A R E A ===========================*/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  procFrgSP
* Input/Output: See function arguments to avoid duplication
* Description:    This is the main routine for processing the SCI_PHOT
*                computation. If this is a SCI_PHOT_CALIB or SCI_PHOT_SCIENCE template 
*                then we exepect ony Interferometry files with the following 
*                configurations:
*                
*                DATA1 = Photometry A     data
*                DATA2 = Interferometry     data
*                DATA3 = Interferometry    data
*                DATA4 = Photometry B    data
*
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void procFrgSP (
   MidiFiles         *fileNames,    // In: Pointer to midi files structure
   UserOptions       *options,    // In: Pointer to user options
   int               *error,      // Ou: Error status
   cpl_parameterlist *parlist,
   cpl_frameset      *frameset)
{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "procFrgSP";
    ImageFormat        *formatInterf=NULL;
    ImageFormat        *formatPhotomA=NULL;
    ImageFormat        *formatPhotomB=NULL;
    CompressedData    *compressedInterf=NULL, *compressedPhotomA=NULL, *compressedPhotomB=NULL;
    FilterData        *filterInfo=NULL;
    DispersedResult    *dispResult=NULL;
    CorrectedFrames		*corrFrames=NULL;
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;
    if (options->processing == UNDISPERSED)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "'UNDISPERSED' is an invalid processing request for SCI_PHOT data");
        return;
    }

    //    Allocate memory
    formatInterf = callocImageFormat ();
    formatPhotomA = callocImageFormat ();
    formatPhotomB = callocImageFormat ();
    corrFrames = cpl_calloc (1, sizeof (CorrectedFrames));

    corrFrames->CorrInterf=0;
    corrFrames->CorrPhotomA=0;
    corrFrames->CorrPhotomB=0;
    
    //    Analyse Input files
    //    -------------------
    analyseFitsFrgSP (fileNames, formatInterf, formatPhotomA, formatPhotomB, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot analyse FITS files");
        freeImageFormat (formatInterf);
        freeImageFormat (formatPhotomA);
        freeImageFormat (formatPhotomB);
        return;
    }

    //    Reduce Data
    //    -----------
    //  Allocate memory for data compression
    compressedInterf = callocMidiCompressed (formatInterf);
    compressedPhotomA = callocMidiCompressed (formatPhotomA);
    compressedPhotomB = callocMidiCompressed (formatPhotomB);
    badChannelList = (int *) calloc (formatInterf->iXWidth, sizeof (int)); 
    filterInfo = callocFilterInfo();


    //    Preprocessing
    preProcFrgSP (options, filterInfo, fileNames, compressedInterf, 
        compressedPhotomA, compressedPhotomB, formatInterf, formatPhotomA, formatPhotomB, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot compress FITS files");
        freeCompressedData (formatInterf, compressedInterf);
        freeCompressedData (formatPhotomA, compressedPhotomA);
        freeCompressedData (formatPhotomB, compressedPhotomB);
        free (badChannelList);
        freeImageFormat (formatInterf);
        freeImageFormat (formatPhotomA);
        freeImageFormat (formatPhotomB);
        freeFilterInfo (filterInfo);
        return;
    }

    //    Check Delay Lines for jumps within scans
    checkDelayLineJumps ("INTERF", formatInterf, compressedInterf);
    checkDelayLineJumps ("PHOTOMA", formatPhotomA, compressedPhotomA);
    checkDelayLineJumps ("PHOTOMB", formatPhotomB, compressedPhotomB);

    //    Check Delay Line Consistencies
    checkDelayLineConsistencies ("INTERF", "PHOTOMA", compressedInterf, compressedPhotomA, formatInterf, formatPhotomA, error);
    checkDelayLineConsistencies ("INTERF", "PHOTOMB", compressedInterf, compressedPhotomB, formatInterf, formatPhotomB, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "SCI_PHOT processing abandonned");
        freeCompressedData (formatInterf, compressedInterf);
        freeCompressedData (formatPhotomA, compressedPhotomA);
        freeCompressedData (formatPhotomB, compressedPhotomB);
        free (badChannelList);
        freeImageFormat (formatInterf);
        freeImageFormat (formatPhotomA);
        freeImageFormat (formatPhotomB);
        freeFilterInfo (filterInfo);
        return;
    }

    //    Check Chopping Statistics. for QCLOG diagnostics
    //    ------------------------------------------------
    reportInterfChopping ("INTERF", formatInterf, compressedInterf);
    reportPhotomChopping ("PHOTOMA", formatPhotomA, compressedPhotomA);
    reportPhotomChopping ("PHOTOMB", formatPhotomB, compressedPhotomB);
    displayInterfChoppingDisp ("INTERF", formatInterf, compressedInterf);
    displayPhotomChoppingDisp ("PHOTOMA", formatPhotomA, compressedPhotomA);
    displayPhotomChoppingDisp ("PHOTOMB", formatPhotomB, compressedPhotomB);
 
    //    Remove Sky Background and display Target with Sky removed
    dispResult = callocDispResult (formatInterf->iXWidth);
    cleanupFrgSP (options->processing, formatInterf, formatPhotomA, formatPhotomB, compressedInterf,
        compressedPhotomA, compressedPhotomB, error, corrFrames);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "SCI_PHOT processing abandonned");
        freeCompressedData (formatInterf, compressedInterf);
        freeCompressedData (formatPhotomA, compressedPhotomA);
        freeCompressedData (formatPhotomB, compressedPhotomB);
        free (badChannelList);
        freeImageFormat (formatInterf);
        freeImageFormat (formatPhotomA);
        freeImageFormat (formatPhotomB);
        freeFilterInfo (filterInfo);
        freeDispResult (dispResult);
        return;
    }

    //    Display Waterfall data
    prepareWaterfallDisplay (fileNames, formatInterf, compressedInterf);

    //    Apply optical compensation using the available Transfer Ratios
    applyOpticalCompensation (fileNames, formatInterf, compressedInterf, 
                              compressedPhotomA, compressedPhotomB, error, parlist, frameset);

    //    Frequency Calibration, if it is a CALIB batch
    //    ---------------------------------------------
    if (strcmp (formatInterf->obsCatg, "CALIB") == 0)
        calibrateWaveUsingPolynom (fileNames, formatInterf);

    //    Create UNDISPERSED allSpectrum and dcLevels. For SCI_PHOT this is not necessary
    //    -------------------------------------------
//    getUndispersedPowerSpectrum (formatInterf, compressedInterf);
        
    postProcFringe (fileNames, filterInfo, compressedInterf, compressedPhotomA,
                    compressedPhotomB, formatInterf, formatPhotomA,
                    formatPhotomB, dispResult, error,
                    parlist, frameset, corrFrames);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "SCI_PHOT processing abandonned");
        freeCompressedData (formatInterf, compressedInterf);
        freeCompressedData (formatPhotomA, compressedPhotomA);
        freeCompressedData (formatPhotomB, compressedPhotomB);
        free (badChannelList);
        freeImageFormat (formatInterf);
        freeImageFormat (formatPhotomA);
        freeImageFormat (formatPhotomB);
        freeFilterInfo (filterInfo);
        freeDispResult (dispResult);
        return;
    }

    //    Release memory used in this batch
    freeCompressedData (formatInterf, compressedInterf);
    freeCompressedData (formatPhotomA, compressedPhotomA);
    freeCompressedData (formatPhotomB, compressedPhotomB);
    free (badChannelList);
    freeImageFormat (formatInterf);
    freeImageFormat (formatPhotomA);
    freeImageFormat (formatPhotomB);
    freeFilterInfo (filterInfo);
    freeDispResult (dispResult);
    cpl_free(corrFrames);
    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  cleanupFrgSP
* Input/Output: See function arguments to avoid duplication
* Description:  Removes sky etc from the data
*
* History:
* 06-Dec-05     (csabet) Created
******************************************************************************/
void cleanupFrgSP (
    enum ProcessingMode    processing,
    ImageFormat            *formatInterf,        // In: Interf size parameters
    ImageFormat            *formatPhotomA,        // In: PhotA size parameters
    ImageFormat            *formatPhotomB,        // In: PhotB size parameters
    CompressedData        *compressedInterf,    // Ou: Pointer to the compressed interferometry data structure
    CompressedData        *compressedPhotomA,    // Ou: Pointer to the compressed photom A data structure
    CompressedData        *compressedPhotomB,    // Ou: Pointer to the compressed photom B data structure
    int                    *error,               // Ou: Error status
    CorrectedFrames			*corrFrames)         // Corrected/removed frames
{
 
    //    Local Declarations
    //    ------------------
    const char    routine[] = "cleanupFrgSP";
    int            transitions, f, R, X, n, localError;
    double        *array, fluxAve;
    float        *arrayF;
    char        *fileName, *title;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
 
   cpl_msg_info(cpl_func,"\nCleaning up SCI_PHOT files for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"------------------------------------ \n");
    fprintf (midiReportPtr, "\nCleaning up SCI_PHOT files for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "------------------------------------ \n");
 
    //    Initialise
    localError = 0;
    *error = 0;
    transitions = 0;
    
    //    Check Interferometry chopping cycle
    //    -----------------------------------    
    transitions = countTransitions ('T', formatInterf->numOfFrames, compressedInterf->tarType);
    sprintf (midiMessage, "Target Transitions for Interf   = %d", transitions); 
    midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    if (transitions < 10)
    {
        sprintf (midiMessage, "Interferometry data is not chopped"); 
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
    }
 
    //    Check PhotomA chopping cycle
    //    ----------------------------    
    transitions = countTransitions ('T', formatPhotomA->numOfFrames, compressedPhotomA->tarType);
    sprintf (midiMessage, "Target Transitions for Photom A = %d", transitions); 
    midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    if (transitions < 10)
    {
        sprintf (midiMessage, "Photometry A data is not chopped"); 
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
    }
 
    //    Check PhotomB chopping cycle
    //    ----------------------------
    transitions = countTransitions ('T', formatPhotomB->numOfFrames, compressedPhotomB->tarType);
    sprintf (midiMessage, "Target Transitions for Photom B = %d", transitions); 
    midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    if (transitions < 10)
    {
        sprintf (midiMessage, "Photometry B data is not chopped"); 
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
    }
 
    //    if No error so far
    if (!(*error))
    {
        formatInterf->chopped = 1;
        formatPhotomA->chopped = 1;
        formatPhotomB->chopped = 1;
    }
    else 
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot cleanup SCI_PHOT files");
        *error = 1;
        return;
    }
    
    //    Correct benign tarType inconsistencies
    //    --------------------------------------



    corrFrames->CorrInterf=correctTarType("INTERF", compressedInterf->tarType,
    		compressedInterf->time, formatInterf->numOfFrames, &localError);
    if (localError) *error = 1;

    corrFrames->CorrPhotomA=correctTarType("PHOTOMA", compressedPhotomA->tarType,
    		compressedPhotomA->time, formatPhotomA->numOfFrames, &localError);
    if (localError) *error = 1;

    corrFrames->CorrPhotomB=correctTarType("PHOTOMB", compressedPhotomB->tarType,
    		compressedPhotomB->time, formatPhotomB->numOfFrames, &localError);
    if (*error || localError) 
    {
        *error = 1;
        return;
    }

    //    Remove sky background
    //    ---------------------
    removeSkyBackground ("INTERF", processing, formatInterf, compressedInterf, &localError);
    if (localError) *error = 1;
    removeSkyBackground ("PHOTOMA", processing, formatPhotomA, compressedPhotomA, &localError);
    if (localError) *error = 1;
    removeSkyBackground ("PHOTOMB", processing, formatPhotomB, compressedPhotomB, &localError);
    if (formatInterf->numOfFrames != formatPhotomA->numOfFrames || 
        formatInterf->numOfFrames != formatPhotomB->numOfFrames || *error || localError)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unequal numOfFrames");
        *error = 1;
        return;
    }
 
    //    Reject frames whose flux are <= 0 or are not 'T'
    for (R = 0; R < formatInterf->numOfRegionsToProcess; R++)
    {
        for (X = 0; X < formatInterf->iXWidth; X++)
        {
            if (badChannelList[X])
                continue;
                    
            for (f = 0; f < formatInterf->numOfFrames; f++)
            {
/*                 if ((compressedInterf->tarType[f] == 'T') &&  */
/*                     (((compressedInterf->iDispFringe)[R])[X])[f] <= 0.0) */
/*                 { */
/*                     compressedInterf->rejectList[X][f] |= BSL_DATA_ERROR; */
/*                     if (diagnostic > 3)  */
/*                     { */
/*                         sprintf (midiMessage, "Negative Flux at: R %d    X %3d    F %4d          Interf %10.2f",  */
/*                             R, X, f, (((compressedInterf->iDispFringe)[R])[X])[f]); */
/*                         midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage); */
/*                     } */
/*                 } */
                //    Reject frames which are not 'T'
                if (compressedInterf->tarType[f] != 'T')
                    compressedInterf->rejectList[X][f] |= BSL_SKY_SCAN;
            }
        }
    }
 
    for (R = 0; R < formatPhotomA->numOfRegionsToProcess; R++)
    {
        for (X = 0; X < formatPhotomA->iXWidth; X++)
        {
            if (badChannelList[X])
                continue;
                    
            for (f = 0; f < formatPhotomA->numOfFrames; f++)
            {
/*                 if ((compressedPhotomA->tarType[f] == 'T') &&  */
/*                     (((compressedPhotomA->iDispFringe)[R])[X])[f] <= 0.0) */
/*                 { */
/*                     compressedPhotomA->rejectList[X][f] |= BSL_DATA_ERROR; */
/* /\*         If the photometry is bad in one of the channels, set also the interferometry to "BSL_DATA_ERROR" *\/ */
/*                     if(formatPhotomA->iXWidth == formatInterf->iXWidth && formatPhotomA->numOfFrames == formatInterf->numOfFrames)  */
/*                     { */
/*                        compressedInterf->rejectList[X][f] |= BSL_DATA_ERROR; */
/*                     } */
/*                     if (diagnostic > 3)  */
/*                     { */
/*                         sprintf (midiMessage, "Negative Flux at: R %d    X %3d    F %4d         PhotomA %10.2f",  */
/*                             R, X, f, (((compressedPhotomA->iDispFringe)[R])[X])[f]); */
/*                         midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage); */
/*                     } */
/*                 } */
                //    Reject frames which are not 'T'
                if (compressedPhotomA->tarType[f] != 'T')
                    compressedPhotomA->rejectList[X][f] |= BSL_SKY_SCAN;
            }
        }
    }
 
    for (R = 0; R < formatPhotomB->numOfRegionsToProcess; R++)
    {
        for (X = 0; X < formatPhotomB->iXWidth; X++)
        {
            if (badChannelList[X])
                continue;
                    
            for (f = 0; f < formatPhotomB->numOfFrames; f++)
            {
/*                 if ((compressedPhotomB->tarType[f] == 'T') &&  */
/*                     (((compressedPhotomB->iDispFringe)[R])[X])[f] <= 0.0) */
/*                 { */
/*                     compressedPhotomB->rejectList[X][f] |= BSL_DATA_ERROR; */
/* /\*         If the photometry is bad in one of the channels, set also the interferometry to "BSL_DATA_ERROR" *\/ */
/*                     if(formatPhotomB->iXWidth == formatInterf->iXWidth && formatPhotomB->numOfFrames == formatInterf->numOfFrames)  */
/*                     { */
/*                     compressedInterf->rejectList[X][f] |= BSL_DATA_ERROR; */
/*                     } */
/*                     if (diagnostic > 3)  */
/*                     { */
/*                         sprintf (midiMessage, "Negative Flux at: R %d    X %3d    F %4d         PhotomB %10.2f",  */
/*                             R, X, f, (((compressedPhotomB->iDispFringe)[R])[X])[f]); */
/*                         midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage); */
/*                     } */
/*                 } */
                //    Reject frames which are not 'T'
                if (compressedPhotomB->tarType[f] != 'T')
                    compressedPhotomB->rejectList[X][f] |= BSL_SKY_SCAN;
            }
        }
    }










    //    Display data with background removed
    if (plotFile)
    {
        array = (double *) calloc (formatInterf->numOfFrames, sizeof (double));
 
        n = 0;
        fluxAve = 0.0;
        for (f = 0; f < formatInterf->numOfFrames; f++)
        {
            if (compressedInterf->tarType[f] == 'T')
            {
                array[n] = compressedInterf->iFringe1[f];
                fluxAve += array[n];
                n++;
            }
        }
        fluxAve /= n;
       cpl_msg_info(cpl_func,"Interf DATA2 Average Flux = %f \n", fluxAve);
        midiCreatePlotFileDouble2D ("InterfCleanDATA2", "Interferometry DATA2 (Sky Removed)", 
            "Frame", "Flux", 0, array, 0, n, 1);
 
        n = 0;
        fluxAve = 0.0;
        for (f = 0; f < formatInterf->numOfFrames; f++)
        {
            if (compressedInterf->tarType[f] == 'T')
            {
                array[n] = compressedInterf->iFringe2[f];
                fluxAve += array[n];
                n++;
            }
        }
        fluxAve /= n;
       cpl_msg_info(cpl_func,"Interf DATA3 Average Flux = %f \n", fluxAve);
        midiCreatePlotFileDouble2D ("InterfCleanDATA3", "Interferometry DATA3 (Sky Removed)", 
            "Frame", "Flux", 0, array, 0, n, 1);
 
        n = 0;
        for (f = 0; f < formatInterf->numOfFrames; f++)
        {
            if (compressedInterf->tarType[f] == 'T')
            {
                array[n] = compressedInterf->iFringe[f];
                n++;
            }
        }
        midiCreatePlotFileDouble2D ("InterfCleanDATA2_DATA3", "Interferometry DATA2-DATA3 (Sky Removed)", 
            "Frame", "Flux", 0, array, 0, n, 1);
 
        n = 0;
        fluxAve = 0.0;
        for (f = 0; f < formatPhotomA->numOfFrames; f++)
        {
            if (compressedPhotomA->tarType[f] == 'T')
            {
                array[n] = compressedPhotomA->iFringe1[f];
                fluxAve += array[n];
                n++;
            }
        }
        fluxAve /= n;
       cpl_msg_info(cpl_func,"PhotomA DATA1 Average Flux = %f \n", fluxAve);
        midiCreatePlotFileDouble2D ("PhotomACleanDATA1", "Photometry A DATA1 (Sky Removed)", 
            "Frame", "Flux", 0, array, 0, n, 1);
 
        n = 0;
        fluxAve = 0.0;
        for (f = 0; f < formatPhotomB->numOfFrames; f++)
        {
            if (compressedPhotomB->tarType[f] == 'T')
            {
                array[n] = compressedPhotomB->iFringe1[f];
                fluxAve += array[n];
                n++;
            }
        }
        fluxAve /= n;
       cpl_msg_info(cpl_func,"PhotomB DATA4 Average Flux = %f \n", fluxAve);
        midiCreatePlotFileDouble2D ("PhotomBCleanDATA4", "Photometry B DATA4 (Sky Removed)", 
            "Frame", "Flux", 0, array, 0, n, 1);
 
        free (array);
    }

    //    Display for each channel
    if (plotFile && diagnostic > 4)
    {
        arrayF = (float *) calloc (formatInterf->numOfFrames, sizeof (float));
        fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        
        for (R = 0; R < formatInterf->numOfRegionsToProcess; R++)
        {
            for (X = 0; X < formatInterf->iXWidth; X++)
            {
                if (badChannelList[X])
                    continue;
                    
                n = 0;
                for (f = 0; f < formatInterf->numOfFrames; f++)
                {
                    if (compressedInterf->tarType[f] == 'T' && !(compressedInterf->rejectList[X][f]))
                    {
                        arrayF[n] = compressedInterf->iDispFringe[R][X][f];
                        n++;
                    }
                }
                sprintf (fileName, "InterfCleanDATA%d_X%d", R+2, X+1);
                sprintf (title, "Interferometry DATA%d for Channel %d (Sky Removed)", R+2, X+1);
                midiCreatePlotFile2D (fileName, title, "Frame", "Flux", 1, arrayF, 0, n, 1, 0);
            }
        }
        for (R = 0; R < formatPhotomA->numOfRegionsToProcess; R++)
        {
            for (X = 0; X < formatPhotomA->iXWidth; X++)
            {
                if (badChannelList[X])
                    continue;
                    
                n = 0;
                for (f = 0; f < formatPhotomA->numOfFrames; f++)
                {
                    if (compressedPhotomA->tarType[f] == 'T' && !(compressedPhotomA->rejectList[X][f]))
                    {
                        arrayF[n] = compressedPhotomA->iDispFringe[R][X][f];
                        n++;
                    }
                }
                sprintf (fileName, "PhotomACleanDATA1_X%d", X+1);
                sprintf (title, "Photometry A DATA1 for Channel %d (Sky Removed)", X+1);
                midiCreatePlotFile2D (fileName, title, "Frame", "Flux", 1, arrayF, 0, n, 1, 0);
            }
        }
        for (R = 0; R < formatPhotomB->numOfRegionsToProcess; R++)
        {
            for (X = 0; X < formatPhotomB->iXWidth; X++)
            {
                if (badChannelList[X])
                    continue;
                    
                n = 0;
                for (f = 0; f < formatPhotomB->numOfFrames; f++)
                {
                    if (compressedPhotomB->tarType[f] == 'T' && !(compressedPhotomB->rejectList[X][f]))
                    {
                        arrayF[n] = compressedPhotomB->iDispFringe[R][X][f];
                        n++;
                    }
                }
                sprintf (fileName, "PhotomBCleanDATA4_X%d", X+1);
                sprintf (title, "Photometry B DATA4 for Channel %d (Sky Removed)", X+1);
                midiCreatePlotFile2D (fileName, title, "Frame", "Flux", 1, arrayF, 0, n, 1, 0);
            }
        }
        free (fileName);
        free (title);
        free (arrayF);
    }



    //    Display the mean flux as a function of the channel

    if (plotFile)
    {	
       arrayF = (float *) calloc (formatInterf->iXWidth, sizeof (float));
       fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
       title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
       
       for (R = 0; R < formatInterf->numOfRegionsToProcess; R++)
       {
          for (X = 0; X < formatInterf->iXWidth; X++)
          {
             arrayF[X]=0;
             if (badChannelList[X])
                continue;
             
              n = 0;
              for (f = 0; f < formatInterf->numOfFrames; f++)
              {
                 if (compressedInterf->tarType[f] == 'T' && !(compressedInterf->rejectList[X][f]))
                 {
                    arrayF[X] += compressedInterf->iDispFringe[R][X][f];
                    n++;
                 }
              }
              arrayF[X] /=(float)n;
          }
          
          sprintf (fileName, "MeanInterfCleanDATA%d", R+1);
          sprintf (title, "Mean Interferometry DATA%d (Sky Removed)", R+1);
          midiCreatePlotFile2D (fileName, title, "Channel", "Flux", 0, arrayF, 0, formatInterf->iXWidth, 1, 0);
       }
       free (fileName);
       free (title);
       free (arrayF);
    }


    //    Finalise rejection list
    finaliseRejectListInterf ("Combined Interferometry", formatInterf, compressedInterf, error);
    if (*error) return;
    finaliseRejectListPhotom ("Photometry A", formatPhotomA, compressedPhotomA, error);
    if (*error) return;
    finaliseRejectListPhotom ("Photometry B", formatPhotomB, compressedPhotomB, error);
    if (*error) return;

    return;
    
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  applyOpticalCompensation
* Input/Output: See function arguments to avoid duplication
* Description:  Uses the existing Transfer Ratios to carry out optical compensation
*                If Transfer Ratios is not available in the current directory
*                it will extract the data from the static database
*
* History:
* 06-Dec-05     (csabet) Created
******************************************************************************/
void applyOpticalCompensation (
   MidiFiles         *fileNames,            // In: Pointer to midi files structure
   ImageFormat       *format,            // In: Interf size parameters
   CompressedData    *compressedInterf,    // Ou: Pointer to the compressed interferometry data structure
   CompressedData    *compressedPhotomA,    // Ou: Pointer to the compressed photom A data structure
   CompressedData    *compressedPhotomB,    // Ou: Pointer to the compressed photom B data structure
   int               *error,  // Ou: Error status
   cpl_parameterlist *parlist,
   cpl_frameset      *frameset)                
{
   
   //    Local Declarations
   //    ------------------
   const char      routine[] = "applyOpticalCompensation";
/*    FILE            *trrPtr=NULL; */
   TransferRatios  *trr;
   int             i, X, n;
/*    int             numOfRecords; */
/*    char            *fileString; */
   char            * tag=NULL;
   char            * softag=NULL;
   cpl_frame       * cur_frame;
   cpl_table       * table;
   float * kappamatrix11=NULL;  
   float * kappamatrix12=NULL;  
   float * kappamatrix21=NULL;  
   float * kappamatrix22=NULL;  
   float * kappamatrix11_error=NULL;  
   float * kappamatrix12_error=NULL;  
   float * kappamatrix21_error=NULL;  
   float * kappamatrix22_error=NULL;  
   int     dimenkappamatrix=0;  
   cpl_errorstate          prestate = cpl_errorstate_get();   
   
   
   //    Algorithm
   //    ---------
   if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
   if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
   
   //    Initialise
   *error = 0;
   
   //    Check if Transfer Ratios file is available. Try to open the file for reading
   cpl_msg_info(cpl_func,"\nApplying Optical Compensation for batch %d \n", batchNumber);
   cpl_msg_info(cpl_func,"--------------------------------------- \n");
   fprintf (midiReportPtr, "\nApplying Optical Compensation for batch %d \n", batchNumber);
   fprintf (midiReportPtr, "--------------------------------------- \n");


/* ---------------------------------------- */
   {
/*       Grism or Prism? */
      if (strcmp(format->grismId,"GRISM")==0){
         softag = cpl_sprintf("%s", "MIDI_KAPPAMATRIX_GRISM");
      }
      if (strcmp(format->grismId,"PRISM")==0){
         softag = cpl_sprintf("%s", "MIDI_KAPPAMATRIX_PRISM");
      }
      
      
      cur_frame = cpl_frameset_get_position(frameset, 0);
      if (cur_frame == NULL) {
         cpl_msg_error(cpl_func, "No frame found in the SOF");  
         return ;
         
      }
      
      cpl_frameset_iterator * it = cpl_frameset_iterator_new(frameset);
      cur_frame = cpl_frameset_iterator_get(it);
      while(cur_frame)
      {
         /*      Check the right tags */
         tag = (char*)cpl_frame_get_tag(cur_frame);
         if (strcmp(tag, softag)) {
             cpl_frameset_iterator_advance(it, 1);
             cur_frame = cpl_frameset_iterator_get(it);
            continue;
         }
         else{
            cpl_msg_info(cpl_func, "Kappamatrix found in the SOF: %s\n",cpl_frame_get_filename(cur_frame));   
            break;
         }
/*          cur_frame = cpl_frameset_get_next( frameset );     */
      }
      cpl_frameset_iterator_delete(it);
      
      if (strcmp(tag, softag)) {
         cpl_msg_info(cpl_func, "No Kappamatrix found in the SOF!");   
         cpl_msg_warning(cpl_func, "Subsequent computational values are without Optical Compensation");
         *error = 1;
         return;
      }

      cpl_free(softag);

      /* Load extension  MIDI_KAPPAMATRIX == extension 1  Do it more clever in the future*/ 

      prestate = cpl_errorstate_get();   
      if((table = cpl_table_load(cpl_frame_get_filename(cur_frame), 1, 1))==NULL)
      {
         cpl_msg_info(cpl_func, "No siutable table found in the file %s\n",cpl_frame_get_filename(cur_frame));   
         cpl_msg_warning(cpl_func, "Subsequent computational values are without Optical Compensation");
         cpl_errorstate_set(prestate);  
         *error = 1;
         return;
      }

      /* If all is Ok read the values */
      
      if (cpl_table_has_column(table, "kappam11"))
      {
         dimenkappamatrix=cpl_table_get_column_dimensions(table, "kappam11");
         kappamatrix11=cpl_table_get_data_float(table, "kappam11");
         kappamatrix12=cpl_table_get_data_float(table, "kappam12");
         kappamatrix21=cpl_table_get_data_float(table, "kappam21");
         kappamatrix22=cpl_table_get_data_float(table, "kappam22");
         kappamatrix11_error=cpl_table_get_data_float(table, "kappam11_error");
         kappamatrix12_error=cpl_table_get_data_float(table, "kappam12_error");
         kappamatrix21_error=cpl_table_get_data_float(table, "kappam21_error");
         kappamatrix22_error=cpl_table_get_data_float(table, "kappam22_error");
/*        cpl_msg_info(cpl_func, "kappam11: %f", kappamatrix11[0]); */
      }
      else{
         cpl_msg_info(cpl_func, "No siutable table column found in the file %s\n",cpl_frame_get_filename(cur_frame));   
         cpl_msg_warning(cpl_func, "Subsequent computational values are without Optical Compensation");
         *error = 1;
         return;
      }
      
      trr = callocTransferRatios (format->iXWidth);
      
      cpl_msg_info(cpl_func,"Dimension: %d",dimenkappamatrix);
/*       for (i=0; i<dimenkappamatrix;i++){ */
      for (i=0; i<format->iXWidth;i++){
         
         trr->ka1[i]=kappamatrix11[i] ;
         trr->siga1[i]=kappamatrix11_error[i] ; 
         trr->ka2[i]=kappamatrix12[i] ; 
         trr->siga2[i]=kappamatrix12_error[i] ; 
         trr->kb1[i]=kappamatrix21[i] ; 
         trr->sigb1[i]=kappamatrix21_error[i] ; 
         trr->kb2[i]=kappamatrix22[i] ; 
         trr->sigb2[i]=kappamatrix22_error[i] ;
         cpl_msg_info(cpl_func,"%3d        %6.2f %6.2f      %6.2f %6.2f      %6.2f %6.2f      %6.2f %6.2f \n", i, 
                      trr->ka1[i], trr->siga1[i], trr->ka2[i], trr->siga2[i], 
                      trr->kb1[i], trr->sigb1[i], trr->kb2[i], trr->sigb2[i]);
      }         

      for (i=0; i<format->iXWidth;i++){
         if (!(trr->ka1[i]) || !(trr->ka2[i]) || !(trr->kb1[i]) || !(trr->kb2[i]))
         {
            badChannelList[i] |= BSL_DATA_ERROR;
            cpl_msg_info(cpl_func,"%3d    <-- %s --> \n", i, UNAV);
            fprintf (midiReportPtr, "%3d    <-- %s --> QCLOG \n", i, UNAV);
         }
      }
      
      //    Compute compensated beams:
      
      //    Check for valid channels
      n = 0;
      for (X = 0; X < format->iXWidth; X++)
      {
         if (!(badChannelList[X]))
            n++;
      }        
      if (!n)
      {
         cpl_msg_warning(cpl_func, "Cannot find any valid Transfer Ratios");
         cpl_msg_warning(cpl_func, "Subsequent computational values are without Optical Compensation");
         *error = 1;
         freeTransferRatios (trr);
         return;
      }
      
      //    Get compensated beams
      getCompensatedBeams (trr, format, compressedInterf, compressedPhotomA, compressedPhotomB);
      freeTransferRatios (trr);
      cpl_table_delete(table);      
/*       *error = 1; */
      return;   
   }

/* ---------------------------------------- */



   
/*    if (diagnostic)cpl_msg_info(cpl_func,"Trying to extract Transfer Ratios from %s \n", fileNames->trrNameRead); */
/*    fprintf (midiReportPtr, "Trying to extract Transfer Ratios from %s \n", fileNames->trrNameRead); */
/*    if ((trrPtr = fopen (fileNames->trrNameRead, "r")) == NULL) */
/*    { */
/*       *error = 1; */
/*       sprintf (midiMessage, "Cannot open Transfer Ratios file %s in the local directory", fileNames->trrNameRead);  */
/*       midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage); */
/*    } */
   
/*    //    If Transfer Ratios file is not available in the local directory try the static database */
/*    if (*error) */
/*    { */
/*       *error = 0; */
/*       fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char)); */
/*       sprintf (fileString , "%s_%s.dat", fileNames->transferRatiosName, format->grismId); */
/*       if (diagnostic)cpl_msg_info(cpl_func,"Trying to extract Transfer Ratios from the static database %s \n", fileString); */
/*       fprintf (midiReportPtr, "Trying to extract Transfer Ratios from the static database %s \n", fileString); */
/*       if ((trrPtr = fopen (fileString, "r")) == NULL) */
/*       { */
/*          sprintf (midiMessage, "Cannot open Transfer Ratios file %s", fileString);  */
/*          midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage); */
/*          *error = 1; */
/*       } */
/*       free (fileString); */
/*    } */
   
/*    //    If all is well, read the supporting parameters */
/*    if (!(*error)) */
/*    { */
/*       //    Read number of record in the file */
/*       if (fscanf (trrPtr, "%d \n", &numOfRecords) == EOF) */
/*       { */
/*          *error = 1; */
/*          midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Number of Records"); */
/*          if (trrPtr) fclose (trrPtr); */
/*       } */
/*       else  */
/*       { */
/*          //    Check if number of records is consistent */
/*          if (numOfRecords != format->iXWidth) */
/*          { */
/*             *error = 1; */
/*             midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent Number of Records"); */
/*             cpl_msg_info(cpl_func,"Given Number of Records = %d. Expected %d \n", numOfRecords, format->iXWidth); */
/*             fprintf (midiReportPtr, "Given Number of Records = %d. Expected %d \n", numOfRecords, format->iXWidth); */
/*             if (trrPtr) fclose (trrPtr); */
/*          } */
/*       } */
      
/*       //    If no error, extract the data */
/*       if (!(*error)) */
/*       { */
/*          trr = callocTransferRatios (format->iXWidth); */
         
/*          //    Extract Transfer Ratios. File pointer should now be at the second line */
/*          cpl_msg_info(cpl_func,"Channel     ka1   siga1         ka2   siga2         kb1   sigb1         kb2   sigb2 \n"); */
/*          fprintf (midiReportPtr, "Channel    ka1   siga1         ka2   siga2         kb1   sigb1         kb2   sigb2 \n"); */


/*          for (i = 0; i < format->iXWidth; i++) */
/*          { */
/*             if (fscanf (trrPtr, "%f %f %f %f %f %f %f %f \n", &(trr->ka1[i]), &(trr->siga1[i]),  */
/*                         &(trr->ka2[i]), &(trr->siga2[i]), &(trr->kb1[i]), &(trr->sigb1[i]),  */
/*                         &(trr->kb2[i]), &(trr->sigb2[i])) == EOF) */
/*             { */
/*                *error = 1; */
/*                sprintf (midiMessage, "Extracted %d records. Expected %d", i, format->iXWidth); */
/*                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage); */
/*                if (trrPtr) fclose (trrPtr); */
/*                freeTransferRatios (trr); */
/*                break; */
/*             } */
/*             else */
/*             { */
/*                if (!(trr->ka1[i]) || !(trr->ka2[i]) || !(trr->kb1[i]) || !(trr->kb2[i])) */
/*                { */
/*                   badChannelList[i] |= BSL_DATA_ERROR; */
/*                   cpl_msg_info(cpl_func,"%3d    <-- %s --> \n", i, UNAV); */
/*                   fprintf (midiReportPtr, "%3d    <-- %s --> QCLOG \n", i, UNAV); */
/*                   if (diagnostic > 2) */
/*                   { */
/*                      sprintf (midiMessage, "Found invalid Transfer Ratios for channel %3d", i); */
/*                      midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage); */
/*                   } */
/*                } */
/*                else */
/*                { */
/*                   cpl_msg_info(cpl_func,"%3d        %6.2f %6.2f      %6.2f %6.2f      %6.2f %6.2f      %6.2f %6.2f \n", i,  */
/*                                trr->ka1[i], trr->siga1[i], trr->ka2[i], trr->siga2[i],  */
/*                                trr->kb1[i], trr->sigb1[i], trr->kb2[i], trr->sigb2[i]); */
/*                   fprintf (midiReportPtr, "%3d        %6.2f %6.2f      %6.2f %6.2f      %6.2f %6.2f      %6.2f %6.2f QCLOG \n", i,  */
/*                            trr->ka1[i], trr->siga1[i], trr->ka2[i], trr->siga2[i],  */
/*                            trr->kb1[i], trr->sigb1[i], trr->kb2[i], trr->sigb2[i]); */
/*                } */
/*             } */
/*          } */
         
/*          //    Compute compensated beams */
/*          if (!(*error)) */
/*          { */
/*             sprintf (midiMessage, "Extracted the above %d Transfer Ratios records \n", format->iXWidth); */
/*             midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage); */
/*             if (trrPtr) fclose (trrPtr); */
            
/*             //    Check for valid channels */
/*             n = 0; */
/*             for (X = 0; X < format->iXWidth; X++) */
/*             { */
/*                if (!(badChannelList[X])) */
/*                   n++; */
/*             }         */
/*             if (!n) */
/*             { */
/*                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find any valid Transfer Ratios"); */
/*                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__,  */
/*                                   "Subsequent computational values are without Optical Compensation"); */
/*                *error = 1; */
/*                freeTransferRatios (trr); */
/*                return; */
/*             } */
            
/*             //    Get compensated beams */
/*             getCompensatedBeams (trr, format, compressedInterf, compressedPhotomA, compressedPhotomB); */
/*             freeTransferRatios (trr); */
/*          } */
/*       } */
/*       else */
/*       { */
/*          midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__,  */
/*                             "Subsequent computational values are without Optical Compensation"); */
/*       } */
/*    } */
/*    else */
/*    { */
/*       midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__,  */
/*                          "Subsequent computational values are without Optical Compensation"); */
/*    } */
   
/*    return; */
}
/*****************************************************************************/


 
/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  getCompensatedBeams
* Input/Output: See function arguments to avoid duplication
* Description:  Computes compensated Interferometry channels
*
* History:
* 06-Mar-06     (csabet) Created
******************************************************************************/
void getCompensatedBeams (
    TransferRatios    *trr,                // In: Transfer Ratios
    ImageFormat        *format,            // In: Interf size parameters
    CompressedData    *compressedInterf,    // Ou: Pointer to the compressed interferometry data structure
    CompressedData    *compressedPhotomA,    // Ou: Pointer to the compressed photom A data structure
    CompressedData    *compressedPhotomB)    // Ou: Pointer to the compressed photom B data structure
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "getCompensatedBeams";
    int            f, X, R;
    double compensate; 
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    For all regions
    for (R = 0; R < format->numOfRegionsToProcess; R++)
    {
       //    For all valid channels (Wavelengths)
       for (X = 0; X < format->iXWidth; X++)
       {
          if (badChannelList[X])
             continue;
          
          for (f = 0; f < format->numOfFrames; f++)
          {
             //    Process good frames only
             if (!((compressedInterf->rejectList[X])[f]) )
             {
/*                 At the moment no compensation in the fringe */
/*                 channels should be done, but one has to compensate */
/*                 the photA and photB photometry. photA and photB are */
/*                 scaled to the total flux of telescope A and */
/*                 telescope B used to produce the fringes(!!), respectively. */

                if (R == 0)
                {
                   if(trr->ka1[X] > 0 && trr->ka2[X] > 0 && trr->kb1[X] > 0 && trr->kb2[X] > 0)
                   {
/*                       Total flux of telescope A and B */
/*                       ((compressedPhotomA->iDispFringe[0])[X])[f]=(((compressedPhotomA->iDispFringe[0])[X])[f])*(1.+ trr->ka1[X] + trr->kb1[X]); */
/*                       ((compressedPhotomB->iDispFringe[0])[X])[f]=(((compressedPhotomB->iDispFringe[0])[X])[f])*(1.+trr->ka2[X] + trr->kb2[X]); */

/*                       Total flux of telescope A and B used to produce the fringes (!) */
                      ((compressedPhotomA->iDispFringe[0])[X])[f]=(((compressedPhotomA->iDispFringe[0])[X])[f])*(trr->ka1[X] + trr->kb1[X]);
                      ((compressedPhotomB->iDispFringe[0])[X])[f]=(((compressedPhotomB->iDispFringe[0])[X])[f])*(trr->ka2[X] + trr->kb2[X]);
                   }
                   else
                   {
                      ((compressedPhotomA->iDispFringe[0])[X])[f]=(((compressedPhotomA->iDispFringe[0])[X])[f]);
                      ((compressedPhotomB->iDispFringe[0])[X])[f]=(((compressedPhotomB->iDispFringe[0])[X])[f]);
                   }
/*                      cpl_msg_info(cpl_func, "PhotA: %f PhotB: %f ka1: %f kb1 %f Fringe: %f", ((compressedPhotomA->iDispFringe[R])[X])[f], */
/*                                    ((compressedPhotomB->iDispFringe[R])[X])[f], trr->ka1[X], trr->kb1[X],((compressedInterf->iDispFringe[R])[X])[f]); */
                   
                }
                continue;
                
                if (R == 0)
                {
                   compensate=1. / ( (sqrt(((compressedPhotomA->iDispFringe[R])[X])[f] * 
                                           ((compressedPhotomB->iDispFringe[R])[X])[f])) *
                                     (sqrt(trr->ka1[X] * trr->kb1[X])) );
                   if(isnan(compensate) || isinf(compensate))
                   {
                      compensate=0.;
/*                       cpl_msg_info(cpl_func, "PhotA: %f PhotB: %f ka1: %f kb1 %f ", ((compressedPhotomA->iDispFringe[R])[X])[f], */
/*                                    ((compressedPhotomB->iDispFringe[R])[X])[f], trr->ka1[X], trr->kb1[X]); */
                   }
                   ((compressedInterf->iDispFringe[R])[X])[f] = ((compressedInterf->iDispFringe[R])[X])[f] * compensate;
                }
                else
                {
                   compensate=1. / ( (sqrt(((compressedPhotomA->iDispFringe[R-1])[X])[f] * 
                                           ((compressedPhotomB->iDispFringe[R-1])[X])[f])) *
                                     (sqrt(trr->ka2[X] * trr->kb2[X])) );
                   
                   if(isnan(compensate) || isinf(compensate))
/*                       if(1) */
                   {
                      compensate=0.;
/*                       cpl_msg_info(cpl_func, "PhotA: %f PhotB: %f ka2: %f kb2 %f ", ((compressedPhotomA->iDispFringe[R-1])[X])[f], */
/*                                    ((compressedPhotomB->iDispFringe[R-1])[X])[f], trr->ka2[X], trr->kb2[X]); */
                   }
                   
                   ((compressedInterf->iDispFringe[R])[X])[f] = 
                      ((compressedInterf->iDispFringe[R])[X])[f] * compensate;
                   
                }
             }
             else
             {
                if (diagnostic > 2)
                {
                   sprintf (midiMessage, "Interferometry data in region %d, channel %3d, frame %4d cannot be compensated", 
                            R, X, f);
                   midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                }
             }
          }
       }
    }
    
    return;
}
/*****************************************************************************/
