/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  statistics.c
* Description:  Contains routines for all statistical computations
*
* History:      
* 22-Dec-03     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <math.h>
#include <stdio.h>
#include <cpl.h>
#include <stdlib.h>
#include "errorHandling.h"
#include "midiGlobal.h"
#include "diagnostics.h"
#include "statistics.h"

/**********************************************************
*   Constant definitions
**********************************************************/
#define    ITMAX    (100)
#define EPS        (3.0e-7)
#define FPMIN     (1.0e-30)
static float sqrarg;
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)

/**********************************************************
*   Global Variables 
**********************************************************/

/*============================ C O D E    A R E A ===========================*/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:    makeStats
* Input/Output:    See function arguments to avoid duplication
* Description:
*
* History:
* 25-Aug-04        (JM) Created
* 16-Jan-05        (csabet) Cleaned up and integrated into MIDI pipeline
******************************************************************************/
void   makeStats(
    float    *data,
    int        numdata,
    float    *mean,
    float    *rms)
{

    /*  Local Declarations
    --------------------*/
    const char        routine[] = "makeStats";
    float accum = 0.F;
    float accum2 = 0.F;
    int i;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    *rms = 0.F;        // Usually changed below
    *mean = 0.F;    // Usually changed below
    if (numdata < 1) return;    // just in case
    for (i = 0; i < numdata; i++)
    {
        accum += data[i];
        accum2 += data[i] * data[i];
    }
    *mean = accum/numdata;
    if (numdata < 2) return;    // just in case
    accum2 -= *mean * *mean * numdata;
    if (accum2 < 0.F)  return;
    *rms = sqrt(accum2/((float) (numdata-1)));
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:    removeDc
* Input/Output:    See function arguments to avoid duplication
* Description:    Removes DC from the input array. The arrays are
*                1D linear. Hence the routine can be used for linear
*                images
*
* History:
* 19-May-06        (csabet) Created
******************************************************************************/
void removeDc (
    int        size,        // In: Size of the image
    float    *inArray,    // In: Input array
    float    *outArray)    // Ou: Output array with DC removed
{

    /*  Local Declarations
    --------------------*/
    const char    routine[] = "removeDc";
    int         i;
    float        max;
    
    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    max = 0.0;
    for (i = 0; i < size; i++)
    {
        if (inArray[i] > max) max = inArray[i];
    }
    for (i = 0; i < size; i++)
        outArray[i] = inArray[i] - max;
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software 
*
* Module name:  signalMean
* Input/Output: See function arguments to avoid duplication
* Description:  Computes mean of a signal
* History:      
* 22-Dec-03     (csabet) Created
******************************************************************************/
float signalMean (        /*    Ou:    Mean of the signal */
    float    *signal,    /*    In:    Pointer to an array containing the signal */
    int        start,        /*    In:    Start point */
    int        end)        /*    In:    End point */
{

    /*  Local Declarations
    --------------------*/
    const char    routine[] = "signalMean";
    float        mean = 0.0;
    int            i;
    
    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    for (i = start; i < end; i++) 
        mean += signal[i];
    
    if ((end - start) != 0)    /*    Singularity check */
        mean /= ((float) (end - start));
    else
        mean /= ((float) (VERY_SMALL_INT));
        

    return (mean);
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software 
*
* Module name:  signalPeak
* Input/Output: See function arguments to avoid duplication
* Description:  Computes peak and its index
* History:      
* 11-Apr-06     (csabet) Created
******************************************************************************/
float signalPeak (        //    Ou:    Peak of the signal
    float    *signal,    //    In:    Pointer to an array containing the signal
    int        start,        //    In:    Begin of array section
    int        end,        //    In: End of array section
    int        *peakIndex)    //    Ou:    Peak index
{
 
    //    Local Declarations
    //    ------------------
    const char    routine[] = "signalPeak";
    float        peak, *sigPtr;
    int            i, length;
    
    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //    Singularity check
    length = end - start;
    if (length <= 0) midiReportError (midiReportPtr, routine, __FILE__, __LINE__, "Invalid length");
 
    peak = 0.0;
    *peakIndex = 0;
    sigPtr = signal + start;
    for (i = 0; i < length; i++)
    {
        if (*sigPtr > peak)
        {
            peak = *sigPtr;
            *peakIndex = i;
        }
        sigPtr++;
    } 
 
    return (peak);
}
/*****************************************************************************/
 
 
/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software 
*
* Module name:  signalVariance
* Input/Output: See function arguments to avoid duplication
* Description:  Computes variance and standard deviation of a signal
* History:      
* 22-Dec-03     (csabet) Created
******************************************************************************/
float signalVariance (    /*    Ou:    Variance of the signal */
    float    *signal,    /*    In:    Pointer to an array containing the signal */
    int        start,        /*    In:    Start point */
    int        end,        /*    In:    End point */
    float    *standDev)    /*    Ou:    Standard deviation */
{

    /*  Local Declarations
    --------------------*/
    const char    routine[] = "signalVariance";
    float        mean, variance, diffCumSqr = 0.0, diff;
    int            i, length, length_1;
    
    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /*    Compute mean */
    mean = signalMean (signal, start, end);
    
    for (i = start; i < end; i++)
    {
        diff = signal[i] - mean;
        diffCumSqr += (diff * diff);
    }
    
    length = end - start;
    length_1 = end - start - 1;
    
    if (length == 0)    /*    Singularity check */
        length = VERY_SMALL_INT;
    if (length_1 == 0)    /*    Singularity check */
        length_1 = VERY_SMALL_INT;
        
    variance = diffCumSqr / (float) length_1;
    *standDev = sqrt (variance);

    return (variance);
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software 
*
* Module name:  signalMedian
* Input/Output: See function arguments to avoid duplication
* Description:  Computes median a signal
* History:      
* 22-Dec-03     (csabet) Created
******************************************************************************/
float signalMedian (    //    Ou:    Median of the signal
    float    *signal,    //    In:    Pointer to an array containing the signal
    int        start,        //    In:    Start point
    int        end)        //    In:    End point
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "signalMedian";
     float        remainder, median, *buffer, *sigPtr, *bufPtr;
     int            i, length, lengthHalf;
     
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Singularity check
    length = end - start;
    if (length <= 0) midiReportError (midiReportPtr, routine, __FILE__, __LINE__, "Invalid length");
 
    //    Allocate memory
    buffer = (float *) calloc (length, sizeof (float));
    bufPtr = buffer;
    sigPtr = signal + start;
    
    //    Load into buffer
    for (i = 0; i < length; i++)
        *bufPtr++ = *sigPtr++;
    
    //    Sort the signal
    signalSort (buffer, 0, length);
    
    //    Check if odd or even
    remainder = (length % 2);
    if (remainder)    // It is odd
    {
        lengthHalf = 0.5 * (length -1);
        median = buffer[lengthHalf];
    }
    else            // It is even
    {
        lengthHalf = 0.5 * length;
        median = 0.5 * (buffer[lengthHalf-1] + buffer[lengthHalf]);
    }
    
    //    Release memory
    free (buffer);
 
    return (median);
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software 
*
* Module name:  signalSortInt
* Input/Output: See function arguments to avoid duplication
* Description:  Sorts a signal
* History:      
* 22-Dec-03     (csabet) Created
******************************************************************************/
void signalSortInt (
    int    *signal,    /*    In:    Pointer to an array containing the signal */
    int    start,        /*    In:    Start point */
    int    end)        /*    In:    End point */
{

    /*  Local Declarations
    --------------------*/
    const char    routine[] = "signalSortInt";
    int            i, j, searchStart, foundIndex;
    int            min, signalStart;
    FILE        *tempFilePtr;
    char        *tempFileName;
     
    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    searchStart = start;
    foundIndex = start;
    for (j = start; j < end; j++)
    {
        signalStart = signal[j];
        min = signal[searchStart];
        for (i = searchStart; i < end; i++)
        {
            if (signal[i] <= min)
            {
                min = signal[i];
                foundIndex = i;
            }
        }
        searchStart++;
        signal[j] = signal[foundIndex];
        signal[foundIndex] = signalStart;
    }
    
    /*    Diagnostics */
    if (diagnostic > 4) 
    {
        /*    Allocate memory */
        tempFileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

        sprintf (tempFileName, "%s%s.MedianSortInt.log", outFileDir, outRootName);
        tempFilePtr = fopen(tempFileName, "w");    
        for (i = start; i < end; i++)
            fprintf (tempFilePtr, "%d\n", signal[i]);
    
        /*    Close file */
        fclose (tempFilePtr);
        
        /*    Release memory */
        free (tempFileName);
    }
    
    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software 
*
* Module name:  signalSort
* Input/Output: See function arguments to avoid duplication
* Description:  Sorts a signal
* History:      
* 22-Dec-03     (csabet) Created
******************************************************************************/
void signalSort (
    float    *signal,    //    In:    Pointer to an array containing the signal
    int        start,        //    In:    Start point
    int        end)        //    In:    End point
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "signalSort";
    int            i, j, searchStart, foundIndex;
    float        min, signalStart;
     
    //    Algorithm
    //-----------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    if ((end - start) <= 0) midiReportError (midiReportPtr, routine, __FILE__, __LINE__, "Invalid length");

    searchStart = start;
    foundIndex = start;
    for (j = start; j < end; j++)
    {
        signalStart = signal[j];
        min = signal[searchStart];
        for (i = searchStart; i < end; i++)
        {
            if (signal[i] <= min)
            {
                min = signal[i];
                foundIndex = i;
            }
        }
        searchStart++;
        signal[j] = signal[foundIndex];
        signal[foundIndex] = signalStart;
    }
    
    //    Diagnostics
    if (diagnostic > 4) midiCreatePlotFile2D ("SortedSignal", "Sorted Signal", "x", "y", 
        1, signal, start, end, 1, 0);
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  sqrtp
* Input/Output: See function arguments to avoid duplication
* Description:    
*
* History:
* 21-Jul-03     (JM)
* 20-Jan-05        (csabet) Integrated
******************************************************************************/
float sqr(float x)
{
    return x*x ;
}    
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  sqrtp
* Input/Output: See function arguments to avoid duplication
* Description:    
*
* History:
* 21-Jul-03     (JM)
* 20-Jan-05        (csabet) Integrated
******************************************************************************/
float sqrtp (
    float    x)
{
    if (x <= 0.F)  return 0.F;
    return (float) sqrt(x);
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  midiGetLinearFit
* Input/Output: See function arguments to avoid duplication
* Description:  This is a least-squares fit data to a straight line by
*                minimising chi-squared. It returns uncertainties for
*                all the coefficients, the goodness-of-fit and the value
*                of the chi-squared
*
* History:      
* 14-June-05    (csabet) Adapted from the Numerical Recipes
******************************************************************************/
void midiGetLinearFit (
    double    *x,        // In: Array of elements along X
    double    *y,        // In: array of elements along Y
    int        ndata,    // In: Number of input points
    float    sig,    // In: Spread (Standard Deviation) assumed identical for all points
    int        mwt,    // In: 0 or 1, 0 = Standard deviation not supplied
    float    *a,        // Ou: coefficient
    float    *b,        // Ou: coefficient
    float    *siga,    // Ou: Uncertainty
    float    *sigb,    // Ou: Uncertainty
    float    *chi2,    // Ou: Chi-squared
    float    *q,        // Ou: Goodness-of-fit
    int        *error)    // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "midiGetLinearFit";
    int         i;
    double         arg, wt, t, sxoss, sx=0.0, sy=0.0, st2=0.0, ss, sigdat;
    
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    *b = 0.0;
    if (mwt)
    {
        ss = 0.0;
        for (i = 0; i < ndata; i++) 
        {
            wt = 1.0 / SQR(sig);
            ss += wt;
            sx += x[i] * wt;
            sy += y[i] * wt;
        }
    } 
    else 
    {
        for (i = 0; i < ndata; i++) 
        {
            sx += x[i];
            sy += y[i];
        }
        ss = ndata;
    }
    sxoss = sx / ss;
    if (mwt) 
    {
        for (i = 0; i < ndata; i++) 
        {
            t = (x[i] - sxoss) / sig;
            st2 += t * t;
            *b += t * y[i] / sig;
        }
    } 
    else 
    {
        for (i = 0; i < ndata; i++) 
        {
            t = x[i] - sxoss;
            st2 += t * t;
            *b += t * y[i];
        }
    }
    *b /= st2;
    *a = (sy - sx * (*b)) / ss;
    *siga = sqrt ((1.0 + sx * sx / (ss * st2)) /ss);
    *sigb = sqrt (1.0 / st2);
    *chi2 = 0.0;
    *q = 1.0;
    if (mwt == 0) 
    {
        for (i = 0; i < ndata; i++)
            *chi2 += SQR(y[i] - (*a) - (*b) * x[i]);

        sigdat = sqrt ((*chi2) / (ndata - 2));
        *siga *= sigdat;
        *sigb *= sigdat;
    } 
    else 
    {
        for (i = 0; i < ndata; i++)
        {
            arg = (y[i] - (*a) - (*b) * x[i]) / sig;
            *chi2 += SQR((y[i] - (*a) - (*b) * x[i]) / sig);
        }
        if (ndata > 2) *q = midiGoodnessOfFit (0.5 * (ndata - 2), 0.5 * (*chi2), error);
        if (*error)
        {
            sprintf (midiMessage, "Cannot compute goodness-of-fit in routine '%s'", routine);
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            return;
        }
    }

    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  midiGoodnessOfFit
* Input/Output: See function arguments to avoid duplication
* Description:  Computes the goodness-of-fit
*
* History:      
* 14-June-05    (csabet) Adapted from the Numerical recipes
******************************************************************************/
float midiGoodnessOfFit (    // Ou: Goodness-of-fit
    float    a,                // In: Argument for the number of data
    float    x,                // In: Argument relating to the Chi-squared
    int        *error)            // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "midiGoodnessOfFit";
    float        gamser, gammcf, gln;

    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    if (x < 0.0 || a <= 0.0) 
    {
        *error = 1;
        sprintf (midiMessage, "Invalid arguments in routine '%s'", routine);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        return (0.0);
    }
    if (x < (a + 1.0)) 
    {
        gser (&gamser, a, x, &gln, error);
        if (*error)
        {
            sprintf (midiMessage, "Invalid arguments in routine '%s'", routine);
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            return (0.0);
        }
        return (1.0-gamser);
    } 
    else 
    {
        gcf (&gammcf, a, x, &gln, error);
        if (*error)
        {
            sprintf (midiMessage, "Invalid arguments in routine '%s'", routine);
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            return (0.0);
        }
        return (gammcf);
    }

    return (0.0); 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  gser
* Input/Output: See function arguments to avoid duplication
* Description:  
*
* History:      
* 14-June-05    (csabet) Adapted from the Numerical recipes
******************************************************************************/
void gser (
    float    *gamser,
    float    a,
    float    x,
    float    *gln,
    int        *error)

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "gser";
    int         n;
    float         sum, del, ap;

    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    *gln = gammln(a);
    if (x <= 0.0) 
    {
        if (x < 0.0) 
        {
            *error = 1;
            sprintf (midiMessage, "Invalid arguments in routine '%s'", routine);
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            return;
        }
        *gamser=0.0;
    } 
    else 
    {
        ap = a;
        del = sum = 1.0 / a;
        for (n = 0; n < ITMAX; n++) 
        {
            ++ap;
            del *= x/ap;
            sum += del;
            if (fabs(del) < fabs(sum) * EPS) 
            {
                *gamser = sum * exp (-x + a * log(x) - (*gln));
                return;
            }
        }
        *error = 1;
        sprintf (midiMessage, "Invalid arguments in routine '%s'", routine);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        return;
    }

    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  gammln
* Input/Output: See function arguments to avoid duplication
* Description:  
*
* History:      
* 14-June-05    (csabet) Adapted from the Numerical recipes
******************************************************************************/
float gammln (
    float    xx)

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "gammln";
    double        x, y, tmp, ser;
    static        double cof[6] = {76.18009172947146,-86.50532032941677,
                                24.01409824083091,-1.231739572450155,
                                0.1208650973866179e-2,-0.5395239384953e-5};
    int         j;

    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    y = x = xx;
    tmp = x + 5.5;
    tmp -= (x + 0.5) * log(tmp);
    ser = 1.000000000190015;
    for (j = 0; j <= 5; j++) ser += cof[j] / (++y);
    
    return (-tmp + log (2.5066282746310005 * ser / x));

}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  gcf
* Input/Output: See function arguments to avoid duplication
* Description:  
*
* History:      
* 14-June-05    (csabet) Adapted from the Numerical recipes
******************************************************************************/
void gcf (
    float    *gammcf,
    float    a,
    float    x,
    float    *gln,
    int        *error)

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "gcf";
    int            i;
    float         an, b, c, d, del, h;

    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;


    *gln = gammln(a);
    b = x + 1.0 - a;
    c = 1.0 / FPMIN;
    d = 1.0 / b;
    h = d;
    for (i = 0; i < ITMAX; i++) 
    {
        an = -i * (i - a);
        b += 2.0;
        d = an * d + b;
        if (fabs(d) < FPMIN) d = FPMIN;
        c = b + an / c;
        if (fabs(c) < FPMIN) c = FPMIN;
        d = 1.0 / d;
        del = d * c;
        h *= del;
        if (fabs (del - 1.0) < EPS) break;
    }
    
    if (i > ITMAX)
    {
        *error = 1;
        sprintf (midiMessage, "Invalid arguments in routine '%s'", routine);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        return;
    }
    
    *gammcf = exp (-x + a * log(x) - (*gln)) * h;

    return; 
}
/*****************************************************************************/
