/******************************************************************************
*******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software
*
* Module name:    transferFunction.c
* Description:    Contains routines for all transfer function routines
*
* History:        
* 21-Jul-03        (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*    Compiler directives
******************************************************************************/
#define NEW_PBL_x

/******************************************************************************
*    Include files
******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <cpl.h>
#include <string.h>
#include <math.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "qfits.h"
#include "errorHandling.h"
#include "diagnostics.h"
#include "fileHandling.h"
#include "transferFunction.h"
#include "midi_dfs.h"

/**********************************************************
*   Constant definitions
**********************************************************/
#define START_MIDI_RECORD        (173)            /* Start line for MIDI record in the calibrator database */
#define END_MIDI_RECORD          (620)            /* End line for MIDI record in the calibrator database */
#define CALIB_DB_LEN             (30)             /* Maximum length of each elements in the calibrator database */
#define UNWANTED_ELEMENTS        (27)             /* Number of unwanted elements in the calibrator database */
#define CALIBRATOR_MIN_THRESHOLD (0.000290888)    /* Minimum threshold for finding diameter in radians (60 arcsec) */
#define DIAMETER_FACTOR          (4.848136958e-9) /* Diameter factor = (1.0 / (1000 * 206264.8)) */
#define INVERSE_WAVELENGTH       (100000.0)       /* Inverse Effective wavelength = (1.0 / 10 micron) */

/*============================ C O D E    A R E A ===========================*/



/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    estimateTransferFunction
* Input/Output:    See function arguments to avoid duplication
* Description:    This routine computes the transfer fuction as follows: If the current
*               observation category is CALIB then the estimate of the uncalibrated
*               visibility has already been calculated and is given as an input to 
*               this routine. Here we calculate the expected visibility and thereby
*               the transfer function. The ASCCI file containing the transfer function
*               is shown below where the first line gives the number of regions and the
*                subsequent lines give the transfer functions and the errors for each channel
*
*                Record 1    Grism ID
*                Record 2    Number of Record
*                Record 3    number_of_combined_regions
*                Record 4    value_1 value_1_Err ... Value_n value_n_Error
*                .
*                .
*                Record n    value_1 value_1_Err ... Value_n value_n_Error
*
* History:        
* 07-Feb-05        (csabet) Created
******************************************************************************/
void estimateTransferFunction (
   ImageFormat        *formatInterf,        // In: Image size of interferometry data
   MidiFiles          *fileNames,            // In: Pointer to MIDI file structure
   DispersedResult    *dispResult,        // IO: Dispersed result structure
   CalibratorParam    *calibrator,        // Ou: Calibrator parameters
   int                *error,                // Ou: Error Status
   cpl_parameterlist  *parlist,
   cpl_frameset       *frameset)
{
   
    //    Local Declarations
    //    ------------------
   const char         routine[] = "estimateTransferFunction";
   FILE             * trfPtr, *trfHistoryPtr;
   int                X;
   struct stat        buf;
   cpl_table        * trftable=NULL;
   cpl_propertylist * qclist=NULL;   
   char             * dataname=NULL;
   char             * system_call=NULL;

    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nComputing Transfer Functions for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"-------------------------------------- \n");
    fprintf (midiReportPtr, "\nComputing Transfer Functions for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "-------------------------------------- \n");

    //    Reset status
    *error = 0;
    dispResult->trfExists = 0;

    //    Check if transfer function file exist. If so delete it
    if (stat (fileNames->trfNameWrite, &buf) == 0)    //    If the file exist delete it
        remove (fileNames->trfNameWrite);

    identifyCalibrator (fileNames, calibrator, error, parlist, frameset);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot identify Calibrator");
        return;
    }

    computeExpectedVis (calibrator, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot compute Expected Visibilities");
        return;
    }
    
    //    Compute Transfer Function
    if (calibrator->calibVis == 0 || calibrator->calibVisSqrd == 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "A singularity encountered\n                 Cannot compute transfer functions");
        *error = 1;
        return;
    }
    else
    {
        //    Open the transfer function file for writing
        if ((trfPtr = fopen (fileNames->trfNameWrite, "w")) == NULL)
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open file to store transfer functions");
            *error = 1;
            return;
        }
        
        //    Open a file for transfer function history
        trfHistoryPtr = fopen (fileNames->trfHistoryName, "a");
        fprintf (trfHistoryPtr, "Transfer Functions (DISPERSED) for batch %d \n", batchNumber);
        fprintf (trfHistoryPtr, "---------------------------------------- \n");
        fprintf (trfHistoryPtr, "Grism ID                   = %s \n", formatInterf->grismId);
        fprintf (trfHistoryPtr, "Number of Valid Channels   = %d \n", formatInterf->iXWidth);

        if (diagnostic)cpl_msg_info(cpl_func,"Grism ID                   = %s \n", formatInterf->grismId);
        if (diagnostic)cpl_msg_info(cpl_func,"Number of Channels         = %d \n", formatInterf->iXWidth);

        fprintf (midiReportPtr, "\nTransfer Functions (DISPERSED) for batch %d QCLOG \n", batchNumber);
        fprintf (midiReportPtr, "---------------------------------------- QCLOG \n");
        fprintf (midiReportPtr, "Grism ID                   = %s QCLOG \n", formatInterf->grismId);
        fprintf (midiReportPtr, "Number of Valid Channels   = %d QCLOG \n", formatInterf->iXWidth);

        fprintf (trfPtr, "%s \n", formatInterf->grismId);
        fprintf (trfPtr, "%d \n", formatInterf->iXWidth);

        for (X = 0; X < formatInterf->iXWidth; X++)
        {
            if (badChannelList[X])
            {
                if (diagnostic)cpl_msg_info(cpl_func,"%3d    <-- %s --> \n", X, UNAV);
                fprintf (midiReportPtr, "%3d    <-- %s -->  QCLOG \n", X, UNAV);
                fprintf (trfHistoryPtr, "%3d    <-- %s --> \n", X, UNAV);
                fprintf (trfPtr, "%f %f \n", (dispResult->trf)[X], (dispResult->trfErr)[X]);
                continue;
            }
        
            dispResult->trf[X] = dispResult->normVis2[X] / calibrator->calibVis;
            dispResult->trfErr[X] = dispResult->normVis2Err[X] / (fabs (calibrator->calibVis)) + 
                calibrator->calibVisErr * (fabs (dispResult->normVis2[X]) / 
                (calibrator->calibVis * calibrator->calibVis));

            if (diagnostic)cpl_msg_info(cpl_func,"%3d  %f %f \n", X, (dispResult->trf)[X], (dispResult->trfErr)[X]);
            fprintf (midiReportPtr, "%3d  %f %f QCLOG \n", X, (dispResult->trf)[X], (dispResult->trfErr)[X]);
            fprintf (trfHistoryPtr, "%3d  %f %f \n", X, (dispResult->trf)[X], (dispResult->trfErr)[X]);
            fprintf (trfPtr, "%f %f \n", (dispResult->trf)[X], (dispResult->trfErr)[X]);
        }

        fclose (trfPtr);
        fclose (trfHistoryPtr);
        if (diagnostic)cpl_msg_info(cpl_func,"Created Transfer Function file   %s\n", fileNames->trfNameWrite);
        fprintf (midiReportPtr, "Created Transfer Function file   %s\n", fileNames->trfNameWrite);
        dispResult->trfExists = 1;

/* -------------------------------------------------------------------------- */

/* Write transfer function in a fits table */
        trftable=cpl_table_new(formatInterf->iXWidth);
        cpl_table_new_column(trftable,"CHANNEL",CPL_TYPE_INT); 		
        cpl_table_new_column(trftable,"TRF",CPL_TYPE_FLOAT); 		
        cpl_table_new_column(trftable,"TRF_ERROR",CPL_TYPE_FLOAT); 		
        
        for (X = 0; X < formatInterf->iXWidth; X++)
        {
           cpl_table_set_int(trftable,"CHANNEL",X, X+1); 	
           cpl_table_set_float(trftable,"TRF",X, (dispResult->trf)[X]); 	
           cpl_table_set_float(trftable,"TRF_ERROR",X, (dispResult->trfErr)[X]); 	
           if (badChannelList[X])
           {
              cpl_table_set_invalid(trftable, "TRF", X);        
              cpl_table_set_invalid(trftable, "TRF_ERROR", X);        
           }
        }  
        
        
/* Save the fits table  into a fits file */
        
        qclist=cpl_propertylist_new();
        
        if (strcmp(formatInterf->grismId,"GRISM")==0) {
           cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "TRF_GRISM");
           dataname=cpl_sprintf("MIDI_trf_grism.fits");
        }
        if (strcmp(formatInterf->grismId,"PRISM")==0) {
           cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "TRF_PRISM");
           dataname=cpl_sprintf("MIDI_trf_prism.fits");
        }
        cpl_propertylist_append_string(qclist, "EXTNAME", "TRANSFER_FUNCTION");
 

        cpl_propertylist_update_float(qclist,  "ESO QC OBS CAL RA"      , calibrator->calibRA);
        cpl_propertylist_update_float(qclist,  "ESO QC OBS CAL DEC"     , calibrator->calibDEC);
        cpl_propertylist_update_float(qclist,  "ESO QC OBS AVR PBL "    , calibrator->calibPblAverage);
        cpl_propertylist_update_float(qclist,  "ESO QC OBS AVR PARANG"  , calibrator->calibParangAverage);
        cpl_propertylist_update_string(qclist, "ESO QC DB NAME"         , calibrator->calibName);                       
        cpl_propertylist_update_float(qclist,  "ESO QC DB DIAM"         , calibrator->calibDiameter);                   
        cpl_propertylist_update_float(qclist,  "ESO QC DB DIAM ERR "    , calibrator->calibDiameterErr);                
        cpl_propertylist_update_float(qclist,  "ESO QC DB DIST"         , RAD_TO_ARCSEC * calibrator->calibDistance);   
        cpl_propertylist_update_double(qclist, "ESO QC DB MAG NBAND"    , calibrator->calibFlux);                       
        cpl_propertylist_update_int   (qclist, "ESO QC DB FLAG"         , calibrator->calibFlag);                       
        cpl_propertylist_update_float(qclist,  "ESO QC EXP VIS"         , calibrator->calibVis);        
        cpl_propertylist_update_float(qclist,  "ESO QC EXP VIS ERR"     , calibrator->calibVisErr);     
        cpl_propertylist_update_float(qclist,  "ESO QC EXP VIS2"        , calibrator->calibVisSqrd);    
        cpl_propertylist_update_float(qclist,  "ESO QC EXP VIS2 ERR"    , calibrator->calibVisSqrdErr); 

        cpl_propertylist_set_comment (qclist, "ESO QC OBS CAL RA"      , "Observed Calibrator RA in radians");
        cpl_propertylist_set_comment (qclist, "ESO QC OBS CAL DEC"     , "Observed Calibrator DEC in radians");
        cpl_propertylist_set_comment (qclist, "ESO QC OBS AVR PBL "    , "Observed Calibrator Average PBL in meter");
        cpl_propertylist_set_comment (qclist, "ESO QC OBS AVR PARANG"  , "Observed Calibrator Average PARANG in degrees");
        cpl_propertylist_set_comment (qclist, "ESO QC DB NAME"         , "Closest Calibrator in the database");
        cpl_propertylist_set_comment (qclist, "ESO QC DB DIAM"         , "Calibrator Diameter in marcsec");
        cpl_propertylist_set_comment (qclist, "ESO QC DB DIAM ERR "    , "Calibrator Diameter error in marcsec");
        cpl_propertylist_set_comment (qclist, "ESO QC DB DIST"         , "Computed Distance to Observed Calibrator in arcsec");
        cpl_propertylist_set_comment (qclist, "ESO QC DB MAG NBAND"    , "Calibrator Magnitude in the N-Band");
        cpl_propertylist_set_comment (qclist, "ESO QC DB FLAG"         , "Calibrator Quality Flag");
        cpl_propertylist_set_comment (qclist, "ESO QC EXP VIS"         , "Expected Visibility");
        cpl_propertylist_set_comment (qclist, "ESO QC EXP VIS ERR"     , "Expected Visibility Error");
        cpl_propertylist_set_comment (qclist, "ESO QC EXP VIS2"        , "Expected Squared Visibility");
        cpl_propertylist_set_comment (qclist, "ESO QC EXP VIS2 ERR"    , "Expected Squared Visibility Error");


        cpl_dfs_save_table(frameset, NULL, parlist, frameset, NULL, trftable, 
                           qclist, "midi_fringe_all",
                           qclist, NULL, 
                           PACKAGE "/" PACKAGE_VERSION,
                           dataname);
        
        /* Now copy the file to tmp to be able to use the files after the
           originals are renamed by ESOREX 
        */
        system_call=cpl_sprintf("cp %s /tmp/MIDI_trf.fits", dataname);
        cpl_msg_info(cpl_func, "Copying the file into the tmp directory ...");         
        cpl_msg_info(cpl_func, "%s", system_call);         
        int ret = system(system_call);
        if (ret != 0) {
            cpl_msg_warning(cpl_func,"Failed to call the system command");
        }

        cpl_table_delete(trftable);   
        cpl_propertylist_delete(qclist);        
        cpl_free(dataname); 
        cpl_free(system_call); 



/* -------------------------------------------------------------------------- */

        //    Compute Binned Tranfer Function
        computeBinnedTrf (formatInterf, dispResult, error);
        if (*error)
        {
           midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot compute Binned Transfer Function");
           return;
        }
        
        
    }
    



    if (plotFile && diagnostic)
    {
        midiCreatePlotFile2D ("TransferFunctionConbined", "Combined Transfer Function", 
            "Channel", "Transfer Function", 0, dispResult->trf, 0, formatInterf->iXWidth, 1, 0);

        midiCreatePlotFile2D ("TransferFunctionErrConbined", "Combined Transfer Function", 
            "Channel", "Transfer Function Error", 0, dispResult->trfErr, 0, formatInterf->iXWidth, 1, 0);
    }
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    estimateTransferFunctionUndisp
* Input/Output:    See function arguments to avoid duplication
* Description:    This routine computes the transfer fuction as follows: If the current
*               observation category is CALIB then the estimate of the uncalibrated
*               visibility has already been calculated and is given as an input to 
*               this routine. Here we calculate the expected visibility and thereby
*               the transfer function. The ASCCI file containing the transfer function
*               has the following format:
*
*               transfer function region 0
*               transfer function region 0 Error
*               transfer function region 1
*               transfer function region 1 Error
*               transfer function region 2
*               transfer function region 2 Error
*               transfer function squared region 0
*               transfer function squared region 0 Error
*               transfer function squared region 1
*               transfer function squared region 1 Error
*               transfer function squared region 2
*               transfer function squared region 2 Error
*
* History:        
* 21-Jul-03        (csabet) Created
******************************************************************************/
void estimateTransferFunctionUndisp (
   MidiFiles           *fileNames,        // In: Pointer to MIDI file structure
   RawVisibility       *measVis,        // In: Measured visibilities
   TransferFunction    *trf,            // Ou: Transfer Functions
   CalibratorParam     *calibrator,    // Ou: Calibrator parameters
   int                 *error,            // Ou: Status
   cpl_parameterlist   *parlist,
   cpl_frameset        *frameset)
{

    /*    Local Declarations
    --------------------*/
    const char     routine[] = "estimateTransferFunctionUndisp";
    FILE        *trfPtr;
    int         i;
    FILE        *trfHistoryPtr;
    struct stat    buf;
    
    /*    Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nComputing Transfer Functions for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"-------------------------------------- \n");
    fprintf (midiReportPtr, "\nComputing Transfer Functions for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "-------------------------------------- \n");

    /* Reset status */
    *error = 0;
    trf->exists = 0;

    /* Check if transfer function file exist. If so delete it */
    if (stat (fileNames->trfNameWrite, &buf) == 0)    /*    If the file exist delete it */
        remove (fileNames->trfNameWrite);

    identifyCalibrator (fileNames, calibrator, error, parlist, frameset);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot identify Calibrator");
        return;
    }

    computeExpectedVis (calibrator, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot compute Expected Visibilities");
        return;
    }
    
    /* Compute Transfer Function */
    if (calibrator->calibVis == 0 || calibrator->calibVisSqrd == 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "A singularity encountered. Cannot compute transfer functions");
        *error = 1;
        return;
    }
    else
    {
        trf->trf[0] = measVis->vis / calibrator->calibVis;
        trf->trfErr[0] = measVis->visErr / (fabs (calibrator->calibVis)) + 
            calibrator->calibVisErr * (fabs (measVis->vis) / (calibrator->calibVis * calibrator->calibVis));

        trf->trf[1] = measVis->vis1 / calibrator->calibVis;
        trf->trfErr[1] = measVis->vis1Err / (fabs (calibrator->calibVis)) + 
            calibrator->calibVisErr * (fabs (measVis->vis1) / (calibrator->calibVis * calibrator->calibVis));

        trf->trf[2] = measVis->vis2 / calibrator->calibVis;
        trf->trfErr[2] = measVis->vis2Err / (fabs (calibrator->calibVis)) + 
            calibrator->calibVisErr * (fabs (measVis->vis2) / (calibrator->calibVis * calibrator->calibVis));

        trf->trf[3] = measVis->visSqrd / calibrator->calibVisSqrd;
        trf->trfErr[3] = measVis->visSqrdErr / (fabs (calibrator->calibVisSqrd)) + 
            calibrator->calibVisSqrdErr * (fabs (measVis->visSqrd) / (calibrator->calibVisSqrd * calibrator->calibVisSqrd));

        trf->trf[4] = measVis->visSqrd1 / calibrator->calibVisSqrd;
        trf->trfErr[4] = measVis->visSqrd1Err / (fabs (calibrator->calibVisSqrd)) + 
            calibrator->calibVisSqrdErr * (fabs (measVis->visSqrd1) / (calibrator->calibVisSqrd * calibrator->calibVisSqrd));

        trf->trf[5] = measVis->visSqrd2 / calibrator->calibVisSqrd;
        trf->trfErr[5] = measVis->visSqrd2Err / (fabs (calibrator->calibVisSqrd)) + 
            calibrator->calibVisSqrdErr * (fabs (measVis->visSqrd2) / (calibrator->calibVisSqrd * calibrator->calibVisSqrd));
    }
    
    /* Open the transfer function file for writing */
    if ((trfPtr = fopen (fileNames->trfNameWrite, "w")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open file to store transfer functions");
        *error = 1;
        return;
    }
    else
    {
       cpl_msg_info(cpl_func,"Transfer Function region 0               = %f \n", trf->trf[0]);
       cpl_msg_info(cpl_func,"Transfer Function region 0 Error         = %f \n", trf->trfErr[0]);
       cpl_msg_info(cpl_func,"Transfer Function region 1               = %f \n", trf->trf[1]);
       cpl_msg_info(cpl_func,"Transfer Function region 1 Error         = %f \n", trf->trfErr[1]);
       cpl_msg_info(cpl_func,"Transfer Function region 2               = %f \n", trf->trf[2]);
       cpl_msg_info(cpl_func,"Transfer Function region 2 Error         = %f \n", trf->trfErr[2]);
       cpl_msg_info(cpl_func,"Transfer Function Squared region 0       = %f \n", trf->trf[3]);
       cpl_msg_info(cpl_func,"Transfer Function Squared region 0 Error = %f \n", trf->trfErr[3]);
       cpl_msg_info(cpl_func,"Transfer Function Squared region 1       = %f \n", trf->trf[4]);
       cpl_msg_info(cpl_func,"Transfer Function Squared region 1 Error = %f \n", trf->trfErr[4]);
       cpl_msg_info(cpl_func,"Transfer Function Squared region 2       = %f \n", trf->trf[5]);
       cpl_msg_info(cpl_func,"Transfer Function Squared region 2 Error = %f \n", trf->trfErr[5]);
        fprintf (midiReportPtr, "Transfer Function region 0               = %f (QCLOG)\n", trf->trf[0]);
        fprintf (midiReportPtr, "Transfer Function region 0 Error         = %f (QCLOG)\n", trf->trfErr[0]);
        fprintf (midiReportPtr, "Transfer Function region 1               = %f (QCLOG)\n", trf->trf[1]);
        fprintf (midiReportPtr, "Transfer Function region 1 Error         = %f (QCLOG)\n", trf->trfErr[1]);
        fprintf (midiReportPtr, "Transfer Function region 2               = %f (QCLOG)\n", trf->trf[2]);
        fprintf (midiReportPtr, "Transfer Function region 2 Error         = %f (QCLOG)\n", trf->trfErr[2]);
        fprintf (midiReportPtr, "Transfer Function Squared region 0       = %f (QCLOG)\n", trf->trf[3]);
        fprintf (midiReportPtr, "Transfer Function Squared region 0 Error = %f (QCLOG)\n", trf->trfErr[3]);
        fprintf (midiReportPtr, "Transfer Function Squared region 1       = %f (QCLOG)\n", trf->trf[4]);
        fprintf (midiReportPtr, "Transfer Function Squared region 1 Error = %f (QCLOG)\n", trf->trfErr[4]);
        fprintf (midiReportPtr, "Transfer Function Squared region 2       = %f (QCLOG)\n", trf->trf[5]);
        fprintf (midiReportPtr, "Transfer Function Squared region 2 Error = %f (QCLOG)\n", trf->trfErr[5]);

        /* Write transfer function into the file */
        for (i = 0; i < 6; i++)
        {
            fprintf (trfPtr, "%f\n", trf->trf[i]);
            fprintf (trfPtr, "%f\n", trf->trfErr[i]);
        }
        fclose (trfPtr);
        if (diagnostic)cpl_msg_info(cpl_func,"Created Transfer Function file   %s\n", fileNames->trfNameWrite);
        fprintf (midiReportPtr, "Created Transfer Function file   %s\n", fileNames->trfNameWrite);
        trf->exists = 1;
    }

    /* Open a file for transfer function history */
    trfHistoryPtr = fopen (fileNames->trfHistoryName, "a");
    fprintf (trfHistoryPtr, "Transfer Functions (UNDISPERSED) Batch %d \n", batchNumber);
    fprintf (trfHistoryPtr, "-------------------------------------- \n");
    for (i = 0; i < 6; i++) fprintf (trfHistoryPtr, "%f %f \n", trf->trf[i], trf->trfErr[i]);
    fprintf (trfHistoryPtr, "\n");
    fclose (trfHistoryPtr);

    return;
}
/*****************************************************************************/

/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:  identifyCalibrator
* Input/Output: See function arguments to avoid duplication
* Description:  This routine identifies the calibrator diameter and computes
*               the average projected baseline.
*
* History:        
* 21-Jul-03        (csabet) Created
******************************************************************************/
void identifyCalibrator (
   MidiFiles          *fileNames,        // In: Pointer to MIDI file structure
   CalibratorParam    *calibrator,    // Ou: Calibrator parameters
   int                *error,
   cpl_parameterlist  *parlist,
   cpl_frameset       *frameset)
{

    /* Local Declarations
    -------------------*/
    const char     routine[] = "identifyCalibrator";
    char        *fileNameStr, *qfitsString, *cleanString, *stringTemp, *classification;
    int         fileNum;
    float       floatDummy, pblStart, pblEnd, parangStart, parangEnd;
    FILE        *inFitsBatchPtr = NULL;

    /* Algorithm
    ----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    /* Initialize parameters */
    *error = 0;
    pblStart = 0.0;
    pblEnd = 0.0;
    parangStart = 0.0;
    parangEnd = 0.0;
    fileNum = 0;

    /* Allocate memory */
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileNameStr = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    /*  Open the list of files */
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        free (cleanString);
        free (fileNameStr);
        free (classification);
        free (stringTemp);
        *error = 1;
        return;
    }

    /* Extract Projected baseline, RA and DEC for the calibrator */
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", fileNameStr, classification);

        /* Ignore the last empty file */
        qfitsString = qfits_query_ext (fileNameStr, "HIERARCH ESO DPR TYPE", 0);
        if (qfitsString == NULL)
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get DPR TYPE from");
            *error = 1;
            free (cleanString);
            free (fileNameStr);
            free (classification);
            free (stringTemp);
            fclose (inFitsBatchPtr);
            return;
        }
        else
        {
            cleanUpString (qfitsString, cleanString);
            if (strcmp (cleanString, "OTHER") != 0)
            {
                fileNum++;
                
                /* Get RA and DEC from the first file */
                if (fileNum == 1)
                {
                    /* Get RA */
                    qfitsString = qfits_query_ext (fileNameStr, "RA", 0);
                    if (qfitsString == NULL)
                    {
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get RA from");
                        *error = 1;
                        free (cleanString);
                        free (fileNameStr);
                        free (classification);
                        free (stringTemp);
                        fclose (inFitsBatchPtr);
                        return;
                    }
                    else
                    {
                        cleanUpString (qfitsString, cleanString);
                        if (qfits_is_float (cleanString))
                        {
                            sscanf(cleanString, "%f", &(calibrator->calibRA));
                            calibrator->calibRA *= DEG_TO_RAD;
                        }
                        else
                        {
                            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Found non-float RA");
                            *error = 1;
                            free (cleanString);
                            free (fileNameStr);
                            free (classification);
                            free (stringTemp);
                            fclose (inFitsBatchPtr);
                            return;
                        }
                    }
                
                    /* Get DEC */
                    qfitsString = qfits_query_ext (fileNameStr, "DEC", 0);
                    if (qfitsString == NULL)
                    {
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get DEC");
                        *error = 1;
                        free (cleanString);
                        free (fileNameStr);
                        free (classification);
                        free (stringTemp);
                        fclose (inFitsBatchPtr);
                        return;
                    }
                    else
                    {
                        cleanUpString (qfitsString, cleanString);
                        if (qfits_is_float (cleanString))
                        {
                            sscanf(cleanString, "%f", &(calibrator->calibDEC));
                            calibrator->calibDEC *= DEG_TO_RAD;
                        }
                        else
                        {
                            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Found non-float DEC");
                            *error = 1;
                            free (cleanString);
                            free (fileNameStr);
                            free (classification);
                            free (stringTemp);
                            fclose (inFitsBatchPtr);
                            return;
                        }
                    }
                    
                    /* Get target name */
                    qfitsString = qfits_query_ext (fileNameStr, "HIERARCH ESO OBS TARG NAME", 0);
                    if (qfitsString == NULL)
                    {
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get target name");
                        *error = 1;
                        free (cleanString);
                        free (fileNameStr);
                        free (classification);
                        free (stringTemp);
                        fclose (inFitsBatchPtr);
                        return;
                    }
                    else
                    {
                        cleanUpString (qfitsString, cleanString);
                        sscanf (cleanString, "%s", calibrator->calibName);
                    }    
                }

                /* Get PBL12 START */
                qfitsString = qfits_query_ext (fileNameStr, "HIERARCH ESO ISS PBL12 START", 0);
                if (qfitsString == NULL)
                {
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get PBL12 START");
                    *error = 1;
                    free (cleanString);
                    free (fileNameStr);
                    free (classification);
                    free (stringTemp);
                    fclose (inFitsBatchPtr);
                    return;
                }
                else
                {
                    cleanUpString (qfitsString, cleanString);
                    if (qfits_is_float (cleanString))
                    {
                        sscanf(cleanString, "%f", &floatDummy);
                        if (floatDummy > 0.0)
                            pblStart += floatDummy;
                        else
                        {
                            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Found non-positive PBL12 START");
                            *error = 1;
                            free (cleanString);
                            free (fileNameStr);
                            free (classification);
                            free (stringTemp);
                            fclose (inFitsBatchPtr);
                            return;
                        }
                    }
                    else
                    {
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Found non-float PBL12 START");
                        *error = 1;
                        free (cleanString);
                        free (fileNameStr);
                        free (classification);
                        free (stringTemp);
                        fclose (inFitsBatchPtr);
                        return;
                    }
                }

                /* Get PBL12 END */
                qfitsString = qfits_query_ext (fileNameStr, "HIERARCH ESO ISS PBL12 END", 0);
                if (qfitsString == NULL)
                {
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get PBL12 END");
                    *error = 1;
                    free (cleanString);
                    free (fileNameStr);
                    free (classification);
                    free (stringTemp);
                    fclose (inFitsBatchPtr);
                    return;
                }
                else
                {
                    cleanUpString (qfitsString, cleanString);
                    if (qfits_is_float (cleanString))
                    {
                        sscanf(cleanString, "%f", &floatDummy);
                        if (floatDummy > 0.0)
                            pblEnd += floatDummy;
                        else
                        {
                            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Found non-positive PBL12 END");
                            *error = 1;
                             free (cleanString);
                            free (fileNameStr);
                            free (classification);
                            free (stringTemp);
                            fclose (inFitsBatchPtr);
                            return;
                        }
                    }
                    else
                    {
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Found non-float PBL12 END");
                        *error = 1;
                        free (cleanString);
                        free (fileNameStr);
                        free (classification);
                        free (stringTemp);
                        fclose (inFitsBatchPtr);
                        return;
                    }
                }

                /* Get PARANG START */
                qfitsString = qfits_query_ext (fileNameStr, "HIERARCH ESO ISS PARANG START", 0);
                if (qfitsString == NULL)
                {
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get PARANG START");
                    *error = 1;
                    free (cleanString);
                    free (fileNameStr);
                    free (classification);
                    free (stringTemp);
                    fclose (inFitsBatchPtr);
                    return;
                }
                else
                {
                    cleanUpString (qfitsString, cleanString);
                    if (qfits_is_float (cleanString))
                    {
                        sscanf(cleanString, "%f", &floatDummy);
                        /*if (floatDummy > 0.0)*/
                            parangStart += floatDummy;
                        /*else
                        {
                            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Found non-positive PARANG START");
                            *error = 1;
                            free (cleanString);
                            free (fileNameStr);
                            free (classification);
                            free (stringTemp);
                            fclose (inFitsBatchPtr);
                            return;
                        }*/
                    }
                    else
                    {
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Found non-float PARANG START in");
                        *error = 1;
                        free (cleanString);
                        free (fileNameStr);
                        free (classification);
                        free (stringTemp);
                        fclose (inFitsBatchPtr);
                        return;
                    }
                }

                /* Get PARANG END */
                qfitsString = qfits_query_ext (fileNameStr, "HIERARCH ESO ISS PARANG END", 0);
                if (qfitsString == NULL)
                {
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get PARANG END");
                    *error = 1;
                    free (cleanString);
                    free (fileNameStr);
                    free (classification);
                    free (stringTemp);
                    fclose (inFitsBatchPtr);
                    return;
                }
                else
                {
                    cleanUpString (qfitsString, cleanString);
                    if (qfits_is_float (cleanString))
                    {
                        sscanf(cleanString, "%f", &floatDummy);
                        /*if (floatDummy > 0.0)*/
                            parangEnd += floatDummy;
                        /*else
                        {
                            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Found non-positive PARANG END");
                            *error = 1;
                             free (cleanString);
                            free (fileNameStr);
                            free (classification);
                            free (stringTemp);
                            fclose (inFitsBatchPtr);
                            return;
                        }*/
                    }
                    else
                    {
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Found non-float PARANG END");
                        *error = 1;
                        free (cleanString);
                        free (fileNameStr);
                        free (classification);
                        free (stringTemp);
                        fclose (inFitsBatchPtr);
                        return;
                    }
                }
            }
        }        
    }

    /* Compute average of projected baselines and angles */
    pblStart /= fileNum;
    pblEnd /= fileNum;
    calibrator->calibPblAverage = 0.5 * (pblStart + pblEnd);
    parangStart /= fileNum;
    parangEnd /= fileNum;
    calibrator->calibParangAverage = 0.5 * (parangStart + parangEnd);

   cpl_msg_info(cpl_func,"Observed Calibrator                      = %s         \n", calibrator->calibName);
   cpl_msg_info(cpl_func,"Observed Calibrator RA                   = %f radians \n", calibrator->calibRA);
   cpl_msg_info(cpl_func,"Observed Calibrator DEC                  = %f radians \n", calibrator->calibDEC);
   cpl_msg_info(cpl_func,"Observed Calibrator Average PBL          = %f metres  \n", calibrator->calibPblAverage);
   cpl_msg_info(cpl_func,"Observed Calibrator Average PARANG       = %f degrees \n", calibrator->calibParangAverage);
    fprintf (midiReportPtr, "Observed Calibrator                      = %s         (QCLOG)\n", calibrator->calibName);
    fprintf (midiReportPtr, "Observed Calibrator RA                   = %f radians (QCLOG)\n", calibrator->calibRA);
    fprintf (midiReportPtr, "Observed Calibrator DEC                  = %f radians (QCLOG)\n", calibrator->calibDEC);
    fprintf (midiReportPtr, "Observed Calibrator Average PBL          = %f metres  (QCLOG)\n", calibrator->calibPblAverage);
    fprintf (midiReportPtr, "Observed Calibrator Average PARANG       = %f degrees (QCLOG)\n", calibrator->calibParangAverage);

    /* Get diameter of the calibrator from the Calibrator database */
    getCalibratorDiameter (fileNames, calibrator, error, parlist, frameset);
    if (*error) 
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Calibrator Diameter");
        free (cleanString);
        free (fileNameStr);
        free (classification);
        free (stringTemp);
        fclose (inFitsBatchPtr);
        return;
    }

    /* Release memory and close files */
    free (fileNameStr);
    fclose (inFitsBatchPtr);
    free (cleanString);
    free (classification);
    free (stringTemp);

    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    getCalibratorDiameter
* Input/Output:    See function arguments to avoid duplication
* Description:    Searches through the CalibDB and extracts the calibrator
*               diameter. there are 36 elements in one line. there are
*               9 useful elements and UNWANTED_ELEMENTS dummy.
*
*
* History:        
* 26-Feb-04        (csabet) Created
******************************************************************************/
void getCalibratorDiameter (
   MidiFiles        *fileNames,
   CalibratorParam    *calibrator,
   int                *error,
   cpl_parameterlist *parlist,
   cpl_frameset      *frameset)
{
   
   /*    Local Declarations
         --------------------*/
   const char     routine[] = "getCalibratorDiameter";
   int            i;
   double          searchRA, searchDEC, diffAngle;

   cpl_errorstate    prestate = cpl_errorstate_get();   

   cpl_frame       * cur_frame;

   cpl_table       * table;

   char           ** Name=NULL;
   int             * hourRA=NULL;      
   int             * minuteRA=NULL;    
   double           * secondRA=NULL;    
   int             * signDEC=NULL;     
   int             * degreeDEC=NULL;   
   int             * minuteDEC=NULL;   
   double           * secondDEC=NULL;   
   double          * flux=NULL;        
   int             * flag=NULL;        
   double           * diameter=NULL;    
   double           * diameterErr=NULL; 

   int               ext_calibrator_data=0;
   int               dimen_calibrator_data=0;
   char            * tag=NULL;   



   /*    Algorithm
         -----------*/
   if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
   if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
   
   
   
   cur_frame = cpl_frameset_get_position(frameset, 0);
   if (cur_frame == NULL) {
      cpl_msg_error(cpl_func, "No frame found in the SOF");  
      *error = 1;
      return ;
      
   }

   /*  Find the calibrator database file in the SOF */
   cpl_frameset_iterator * it = cpl_frameset_iterator_new(frameset);
   cur_frame = cpl_frameset_iterator_get(it);
   while(cur_frame)
   {
      /*      Check the right tags */
      tag = (char*)cpl_frame_get_tag(cur_frame);
      if (strcmp(tag, MIDI_CALIB_DATABASE)) {
          cpl_frameset_iterator_advance(it, 1);
          cur_frame = cpl_frameset_iterator_get(it);
         continue;
      }
      else{
         cpl_msg_info(cpl_func, "Calibrator database found in the SOF: %s",cpl_frame_get_filename(cur_frame));   
         break;
      }
   }
   cpl_frameset_iterator_delete(it);
   
   if (strcmp(tag, MIDI_CALIB_DATABASE)) {
      cpl_msg_warning(cpl_func, "No calibrator database file found in the SOF!");   
      cpl_msg_warning(cpl_func, "Transfer function can not be calculated!");
      *error = 1;
      return;
   }
   
   
   /* Load extension CALIBRATOR_DATA */ 
   
   prestate = cpl_errorstate_get();   
   
   cpl_msg_info(cpl_func, "Reading %s",cpl_frame_get_filename(cur_frame));
   
   ext_calibrator_data=cpl_fits_find_extension(cpl_frame_get_filename(cur_frame),"CALIBRATOR_DATA");

   /* Load extension  */ 
   if((table = cpl_table_load(cpl_frame_get_filename(cur_frame), ext_calibrator_data, 1))==NULL)
   {
      cpl_msg_info(cpl_func, "No siutable table found in the file: %s",cpl_frame_get_filename(cur_frame));   
      cpl_msg_warning(cpl_func, "Transfer function can not be calculated!");
      cpl_errorstate_set(prestate);  
      *error = 1;
      return;
   }
   
   
   if (cpl_table_has_column(table, "Name"       )) {Name       =cpl_table_get_data_string(table, "Name"       );} else {cpl_msg_warning(cpl_func, "Entry not found:  Name"       );*error = 1;};
   if (cpl_table_has_column(table, "hourRA"     )) {hourRA     =cpl_table_get_data_int   (table, "hourRA"     );} else {cpl_msg_warning(cpl_func, "Entry not found:  hourRA"     );*error = 1;};
   if (cpl_table_has_column(table, "minuteRA"   )) {minuteRA   =cpl_table_get_data_int   (table, "minuteRA"   );} else {cpl_msg_warning(cpl_func, "Entry not found:  minuteRA"   );*error = 1;};
   if (cpl_table_has_column(table, "secondRA"   )) {secondRA   =cpl_table_get_data_double (table, "secondRA"   );} else {cpl_msg_warning(cpl_func, "Entry not found:  secondRA"   );*error = 1;};
   if (cpl_table_has_column(table, "signDEC"    )) {signDEC    =cpl_table_get_data_int   (table, "signDEC"    );} else {cpl_msg_warning(cpl_func, "Entry not found:  signDEC"    );*error = 1;};
   if (cpl_table_has_column(table, "degreeDEC"  )) {degreeDEC  =cpl_table_get_data_int   (table, "degreeDEC"  );} else {cpl_msg_warning(cpl_func, "Entry not found:  degreeDEC"  );*error = 1;};
   if (cpl_table_has_column(table, "minuteDEC"  )) {minuteDEC  =cpl_table_get_data_int   (table, "minuteDEC"  );} else {cpl_msg_warning(cpl_func, "Entry not found:  minuteDEC"  );*error = 1;};
   if (cpl_table_has_column(table, "secondDEC"  )) {secondDEC  =cpl_table_get_data_double (table, "secondDEC"  );} else {cpl_msg_warning(cpl_func, "Entry not found:  secondDEC"  );*error = 1;};
   if (cpl_table_has_column(table, "N_band"     )) {flux       =cpl_table_get_data_double(table, "N_band"     );} else {cpl_msg_warning(cpl_func, "Entry not found:  N_band"     );*error = 1;};
   if (cpl_table_has_column(table, "flag"       )) {flag       =cpl_table_get_data_int   (table, "flag"       );} else {cpl_msg_warning(cpl_func, "Entry not found:  flag"       );*error = 1;};
   if (cpl_table_has_column(table, "diameter"   )) {diameter   =cpl_table_get_data_double (table, "diameter"   );} else {cpl_msg_warning(cpl_func, "Entry not found:  diameter"   );*error = 1;};
   if (cpl_table_has_column(table, "diameterErr")) {diameterErr=cpl_table_get_data_double (table, "diameterErr");} else {cpl_msg_warning(cpl_func, "Entry not found:  diameterErr");*error = 1;};
   
   dimen_calibrator_data=cpl_table_get_nrow(table);
   
   cpl_msg_info(cpl_func, "Total number of Calibrators in the table: %d", dimen_calibrator_data);      

   calibrator->calibDistance =FLT_MAX;
   
      /* Search for the minimum distance and get the diameter */
      for (i=0; i<dimen_calibrator_data;i++)
      {
         
         searchRA  = HOUR_TO_RAD * ((double) hourRA[i] + minuteRA[i]/60.0 + secondRA[i]/3600.0);
         searchDEC = DEG_TO_RAD * (((double) degreeDEC[i] + minuteDEC[i]/60.0 + secondDEC[i]/3600.0) * signDEC[i]);
         diffAngle = acos ( sin(searchDEC) * sin(calibrator->calibDEC) + 
                            cos(searchDEC) * cos(calibrator->calibDEC) * cos(searchRA - calibrator->calibRA) );

         /* cpl_msg_info(cpl_func, " i: %d  diffAngle : %f    Name: %s", i, diffAngle,Name[i]); */
         if (diffAngle < calibrator->calibDistance) 
         {
            calibrator->calibDistance = (float)diffAngle;
            calibrator->calibDiameter = (float)diameter[i];
            calibrator->calibDiameterErr = (float)diameterErr[i];
            strcpy (calibrator->calibName, Name[i]);
            calibrator->calibFlux = flux[i];  // Flux in the N-Band (Actually the magnitude)
            calibrator->calibFlag = flag[i]; // Quality Parameter/Flag
            
         }
         
      }
      

    /* Check if minimum angle is within the threshold */
    if (calibrator->calibDistance <= CALIBRATOR_MIN_THRESHOLD)
    {
       cpl_msg_info(cpl_func,"Closest Calibrator in the database       = %s         \n", calibrator->calibName);
       cpl_msg_info(cpl_func,"Calibrator Diameter                      = %f marcsec \n", calibrator->calibDiameter);
       cpl_msg_info(cpl_func,"Calibrator Diameter error                = %f marcsec \n", calibrator->calibDiameterErr);
       cpl_msg_info(cpl_func,"Computed Distance to Observed Calibrator = %f arcsec  \n", RAD_TO_ARCSEC * calibrator->calibDistance);
       cpl_msg_info(cpl_func,"Calibrator Flux in the N-Band            = %g mag     \n", calibrator->calibFlux);
       cpl_msg_info(cpl_func,"Calibrator quality flag                  = %d         \n", calibrator->calibFlag);
       
       fprintf (midiReportPtr, "Closest Calibrator in the database       = %s         (QCLOG)\n", calibrator->calibName);
       fprintf (midiReportPtr, "Calibrator Diameter                      = %f marcsec (QCLOG)\n", calibrator->calibDiameter);
       fprintf (midiReportPtr, "Calibrator Diameter error                = %f marcsec (QCLOG)\n", calibrator->calibDiameterErr);
       fprintf (midiReportPtr, "Computed Distance to Observed Calibrator = %f arcsec  (QCLOG)\n", RAD_TO_ARCSEC * calibrator->calibDistance);
       fprintf (midiReportPtr, "Calibrator Flux in the N-Band            = %g mag     (QCLOG)\n", calibrator->calibFlux);
       fprintf (midiReportPtr, "Calibrator quality flag                  = %d         (QCLOG)\n", calibrator->calibFlag);
    }
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find a close match in the database");
        *error = 1;
    }

    /* Release memory */
    if(table !=NULL) cpl_table_delete(table);
    
/*     fclose (calibDbPtr); */
/*     for (i = 0; i < UNWANTED_ELEMENTS; i++) */
/*         free (dummyStr[i]); */
/*     free (dummyStr); */
/*     free (lineStr); */
    
    return;
}
/*****************************************************************************/
/* /\****************************************************************************** */
/* *                European Southern Observatory */
/* *              VLTI MIDI Data Reduction Software  */
/* * */
/* * Module name:    getCalibratorDiameter */
/* * Input/Output:    See function arguments to avoid duplication */
/* * Description:    Searches through the CalibDB and extracts the calibrator */
/* *               diameter. there are 36 elements in one line. there are */
/* *               9 useful elements and UNWANTED_ELEMENTS dummy. */
/* * */
/* * */
/* * History:         */
/* * 26-Feb-04        (csabet) Created */
/* ******************************************************************************\/ */
/* void getCalibratorDiameter ( */
/*    MidiFiles        *fileNames, */
/*    CalibratorParam    *calibrator, */
/*    int                *error, */
/*    cpl_parameterlist *parlist, */
/*    cpl_frameset      *frameset) */
/* { */

/*     /\*    Local Declarations */
/*     --------------------*\/ */
/*     const char     routine[] = "getCalibratorDiameter"; */
/*     int         i, lineNum,flag; */
/*     int         hourRA, minuteRA, signDEC, degreeDEC, minuteDEC; */
/*     float       secondRA, secondDEC, diameter, diameterErr; */
/*     double flux;  */
/*    char        **dummyStr, *lineStr; */
/*     float       searchRA, searchDEC, diffAngle; */
/*     FILE        *calibDbPtr; */
    
/*     /\*    Algorithm */
/*     -----------*\/ */
/*     if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine); */
/*     if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine); */

/*     /\* Allocate memory *\/ */
/*     lineStr = (char *) calloc (MAX_STRING_LENGTH, sizeof (char)); */
/*     dummyStr = (char **) calloc (UNWANTED_ELEMENTS, sizeof (char *)); */
/*     for (i = 0; i < UNWANTED_ELEMENTS; i++) */
/*         dummyStr[i] = (char *) calloc (CALIB_DB_LEN, sizeof (char)); */

/*     /\* Reset parameters *\/ */
/*     *error = 0; */
/*     lineNum = 0; */

/*     /\*  Open the calibDB file for reading *\/ */
/*     if ((calibDbPtr = fopen (fileNames->calibDbName, "r")) == NULL) */
/*     { */
/*         midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open calibrator database"); */
/*         *error = 1; */
/*         for (i = 0; i < UNWANTED_ELEMENTS; i++) */
/*             free (dummyStr[i]); */
/*         free (dummyStr); */
/*         free (lineStr); */
/*         return; */
/*     } */

/*     /\* Loop through the calibrator database and identify the closest RA and DEC *\/ */
/*     while (fgets (lineStr, MAX_STRING_LENGTH, calibDbPtr) != NULL) */
/*     { */
/*         sscanf (lineStr, "%s%d%d%f%d%d%d%f%s%s%s%s%s%s%s%s%lg%s%s%s%d%f%f%s%s%s%s%s%s%s%s%s%s%s%s%s",  */
/*             dummyStr[0], &hourRA, &minuteRA, &secondRA, &signDEC, &degreeDEC, &minuteDEC, &secondDEC, dummyStr[1], dummyStr[2],  */
/*             dummyStr[3], dummyStr[4], dummyStr[5], dummyStr[6], dummyStr[7], dummyStr[8], &flux, dummyStr[10],  */
/*             dummyStr[11], dummyStr[12], &flag, &diameter, &diameterErr, dummyStr[14], dummyStr[15], dummyStr[16],  */
/*             dummyStr[17], dummyStr[18], dummyStr[19], dummyStr[20], dummyStr[21], dummyStr[22], dummyStr[23], dummyStr[24], */
/*             dummyStr[25], dummyStr[26]); */
        
/*         lineNum++; */
        
/*         /\* Find initial difference *\/ */
/*         if (lineNum == START_MIDI_RECORD) */
/*         { */
/*             searchRA = HOUR_TO_RAD * ((float) hourRA + minuteRA/60.0 + secondRA/3600.0); */
/*             searchDEC = DEG_TO_RAD * (((float) degreeDEC + minuteDEC/60.0 + secondDEC/3600.0) * signDEC); */
/*             calibrator->calibDistance = acos ( sin(searchDEC) * sin(calibrator->calibDEC) +  */
/*                 cos(searchDEC) * cos(calibrator->calibDEC) * cos(searchRA - calibrator->calibRA) ); */
/*             calibrator->calibDiameter = diameter; */
/*             calibrator->calibDiameterErr = diameterErr; */
/*             sscanf (dummyStr[0], "%s", calibrator->calibName); */
/*         } */
        
/*         /\* Now search for the minimum distance and get the diameter *\/ */
/*         if (lineNum > START_MIDI_RECORD && lineNum <= END_MIDI_RECORD) */
/*         { */
/*             searchRA = HOUR_TO_RAD * ((float) hourRA + minuteRA/60.0 + secondRA/3600.0); */
/*             searchDEC = DEG_TO_RAD * (((float) degreeDEC + minuteDEC/60.0 + secondDEC/3600.0) * signDEC); */
/*             diffAngle = acos ( sin(searchDEC) * sin(calibrator->calibDEC) +  */
/*                 cos(searchDEC) * cos(calibrator->calibDEC) * cos(searchRA - calibrator->calibRA) ); */
/*             if (diffAngle < calibrator->calibDistance)  */
/*             { */
/*                 calibrator->calibDistance = diffAngle; */
/*                 calibrator->calibDiameter = diameter; */
/*                 calibrator->calibDiameterErr = diameterErr; */
/*                 sscanf (dummyStr[0], "%s", calibrator->calibName); */
/*                 calibrator->calibFlux = flux;  // Flux in the N-Band */
/*                 calibrator->calibFlag = flag; // Quality Parameter/Flag */


/*             } */
/*         } */
/*         if (lineNum > END_MIDI_RECORD) break; */
/*     } */

/*     /\* Check if there was enough record in the file *\/ */
/*     if (lineNum < START_MIDI_RECORD) */
/*     { */
/*         midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Not enough records in database"); */
/*         *error = 1; */
/*         for (i = 0; i < UNWANTED_ELEMENTS; i++) */
/*             free (dummyStr[i]); */
/*         free (dummyStr); */
/*         free (lineStr); */
/*         return; */
/*     } */
    
/*     /\* Check if minimum angle is within the threshold *\/ */
/*     if (calibrator->calibDistance <= CALIBRATOR_MIN_THRESHOLD) */
/*     { */
/*        cpl_msg_info(cpl_func,"Closest Calibrator in the database       = %s         \n", calibrator->calibName); */
/*        cpl_msg_info(cpl_func,"Calibrator Diameter                      = %f marcsec \n", calibrator->calibDiameter); */
/*        cpl_msg_info(cpl_func,"Calibrator Diameter error                = %f marcsec \n", calibrator->calibDiameterErr); */
/*        cpl_msg_info(cpl_func,"Computed Distance to Observed Calibrator = %f arcsec  \n", RAD_TO_ARCSEC * calibrator->calibDistance); */
/*        cpl_msg_info(cpl_func,"Calibrator Flux in the N-Band            = %g mag     \n", calibrator->calibFlux); */
/*        cpl_msg_info(cpl_func,"Calibrator quality flag                  = %d         \n", calibrator->calibFlag); */

/*         fprintf (midiReportPtr, "Closest Calibrator in the database       = %s         (QCLOG)\n", calibrator->calibName); */
/*         fprintf (midiReportPtr, "Calibrator Diameter                      = %f marcsec (QCLOG)\n", calibrator->calibDiameter); */
/*         fprintf (midiReportPtr, "Calibrator Diameter error                = %f marcsec (QCLOG)\n", calibrator->calibDiameterErr); */
/*         fprintf (midiReportPtr, "Computed Distance to Observed Calibrator = %f arcsec  (QCLOG)\n", RAD_TO_ARCSEC * calibrator->calibDistance); */
/*         fprintf (midiReportPtr, "Calibrator Flux in the N-Band            = %g mag     (QCLOG)\n", calibrator->calibFlux); */
/*         fprintf (midiReportPtr, "Calibrator quality flag                  = %d         (QCLOG)\n", calibrator->calibFlag); */
/*     } */
/*     else */
/*     { */
/*         midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find a close match in the database"); */
/*         *error = 1; */
/*     } */

/*     /\* Release memory and close file *\/ */
/*     fclose (calibDbPtr); */
/*     for (i = 0; i < UNWANTED_ELEMENTS; i++) */
/*         free (dummyStr[i]); */
/*     free (dummyStr); */
/*     free (lineStr); */
    
/*     return; */
/* } */
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    computeExpectedVis
* Input/Output:    See function arguments to avoid duplication
* Description:    
*
*
* History:        
* 21-Jul-03        (csabet) Created
******************************************************************************/
void computeExpectedVis (
    CalibratorParam    *calibrator,    // Ou: Calibrator parameters
    int                *error)            // Ou: Status
{
    /*    Local Declarations
    --------------------*/
    const char     routine[] = "computeExpectedVis";
    double      besselK1o1, besselK1o1Err, xArg, xArgErr;
    
    /*    Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /* Reset parameters */
    *error = 0;
    
    /* Compute Bessel function of first kind of order 1
    For faster processing most divisions are stated by multiplying factors */
    xArg = (double) (MIDI_PI * DIAMETER_FACTOR * calibrator->calibDiameter * calibrator->calibPblAverage * INVERSE_WAVELENGTH);
    xArgErr = (double) (MIDI_PI * DIAMETER_FACTOR * calibrator->calibDiameterErr * calibrator->calibPblAverage * INVERSE_WAVELENGTH);
    besselK1o1 = computeBesselK1o1 (xArg, xArgErr, &besselK1o1Err);

    /* Compute expected visibilities */
    calibrator->calibVis = (float) (2.0 * (besselK1o1 / xArg));
    calibrator->calibVisSqrd = calibrator->calibVis * calibrator->calibVis;

    /* Compute errors */
    calibrator->calibVisErr = (float) (2.0 * (besselK1o1Err/fabs(xArg) + xArgErr * (fabs(besselK1o1)/(xArg * xArg))));
    calibrator->calibVisSqrdErr = 2.0 * (calibrator->calibVisErr * fabs (calibrator->calibVis));

   cpl_msg_info(cpl_func,"Expected Visibility                      = %f \n", calibrator->calibVis);
   cpl_msg_info(cpl_func,"Expected Visibility Error                = %f \n", calibrator->calibVisErr);
   cpl_msg_info(cpl_func,"Expected Squared Visibility              = %f \n", calibrator->calibVisSqrd);
   cpl_msg_info(cpl_func,"Expected Squared Visibility Error        = %f \n", calibrator->calibVisSqrdErr);
    fprintf (midiReportPtr, "Expected Visibility                      = %f (QCLOG)\n", calibrator->calibVis);
    fprintf (midiReportPtr, "Expected Visibility Error                = %f (QCLOG)\n", calibrator->calibVisErr);
    fprintf (midiReportPtr, "Expected Squared Visibility              = %f (QCLOG)\n", calibrator->calibVisSqrd);
    fprintf (midiReportPtr, "Expected Squared Visibility Error        = %f (QCLOG)\n", calibrator->calibVisSqrdErr);

    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    computeBesselK1o1
* Input/Output:    See function arguments to avoid duplication
* Description:    Computes bessel function of first kind of order 1 and the
*               associated errors.
*
*
* History:        
* 01-Mar-04        (csabet) Created
******************************************************************************/
double computeBesselK1o1 (  /* Ou: Computed Bessel function */
    double xArg,            /* In: Bessel argument */
    double xArgErr,         /* In: Error of argument */
    double *besselK1o1Err)  /* Ou: Error of computed Bessel function */
{
    /*    Local Declarations
    --------------------*/
    const char     routine[] = "computeBesselK1o1";
    
    /*    Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    *besselK1o1Err = (
        0.5 * xArgErr +
        0.062500000000 * xArgErr * fabs (3.0 * xArg * xArg) +
        0.002604166666 * xArgErr * fabs (5.0 * xArg * xArg * xArg * xArg) +
        0.000054253472 * xArgErr * fabs (7.0 * xArg * xArg * xArg * xArg * xArg * xArg));
        
    return (
        0.5 * xArg - 
        0.062500000000 * xArg * xArg * xArg + 
        0.002604166666 * xArg * xArg * xArg * xArg * xArg -
        0.000054253472 * xArg * xArg * xArg * xArg * xArg * xArg * xArg);
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:  computeBinnedTrf
* Input/Output: See function arguments to avoid duplication
* Description:  This routine computes the transfer function at 5 spectral
*                bands as follows:
*
*                1)  8.64 microns (Channels 120 -> 126) PRISM
*                                 (Channels  38 ->  50) GRISM
*                2)  9.00 microns (Channels 113 -> 121) PRISM
*                                 (Channels  52 ->  68) GRISM
*                3) 10.46 microns (Channels  88 ->  99) PRISM
*                                 (Channels 116 -> 136) GRISM
*                4) 11.79 microns (Channels  60 ->  72) PRISM
*                                 (Channels 176 -> 200) GRISM
*                5) 12.80 microns (Channels  36 ->  50) PRISM
*                                 (Channels 217 -> 245) GRISM
*
* History:        
* 01-Jul-08        Created
******************************************************************************/
void computeBinnedTrf (
    ImageFormat        *format,        // In: General data format
    DispersedResult    *dispResult,    // In: Dispersed result structure
    int                *error)            // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "computeBinnedTrf";
    int            i, n;

    //    Algorithm
    //    ---------
   cpl_msg_info(cpl_func,"\nBinned Transfer Function for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"------------------------------------------ \n");
    fprintf (midiReportPtr, "\nBinned Transfer Function for batch  %d QCLOG \n", batchNumber);
    fprintf (midiReportPtr, "------------------------------------------ QCLOG \n");

    //    Reset parameters
    *error = 0;
    n = 0;

    dispResult->trfBinned1 = 0.0;
    dispResult->trfBinned2 = 0.0;
    dispResult->trfBinned3 = 0.0;
    dispResult->trfBinned4 = 0.0;
    dispResult->trfBinned5 = 0.0;

    if (strcmp(format->grismId, "PRISM") == 0)
    {
       n = 0;
       for (i = 36; i < 51; i++)
       {
          if (badChannelList[i])
             continue;
                
          dispResult->trfBinned5 += (dispResult->trf)[i];
          n++;
       }
       if(n>0){
          dispResult->trfBinned5 /= n;
       }
       else{
          dispResult->trfBinned5= -1.;   
       }
        
       n = 0;
       for (i = 60; i < 73; i++)
       {
          if (badChannelList[i])
             continue;
                
          dispResult->trfBinned4 += (dispResult->trf)[i];
          n++;
       }
       if(n>0){
          dispResult->trfBinned4 /= n;
       }
       else{
          dispResult->trfBinned4= -1.;   
       }

       n = 0;
       for (i = 88; i < 99; i++)
       {
          if (badChannelList[i])
             continue;
                
          dispResult->trfBinned3 += (dispResult->trf)[i];
          n++;
       }
       if(n>0){
          dispResult->trfBinned3 /= n;
       }
       else{
          dispResult->trfBinned3= -1.;   
       }

       n = 0;
       for (i = 113; i < 122; i++)
       {
          if (badChannelList[i])
             continue;
                
          dispResult->trfBinned2 += (dispResult->trf)[i];
          n++;
       }
       if(n>0){
          dispResult->trfBinned2 /= n;
       }
       else{
          dispResult->trfBinned2= -1.;   
       }

       n = 0;
       for (i = 120; i < 127; i++)
       {
          if (badChannelList[i])
             continue;
                
          dispResult->trfBinned1 += (dispResult->trf)[i];
          n++;
       }

       if(n>0){
          dispResult->trfBinned1 /= n;
       }
       else{
          dispResult->trfBinned1= -1.;   
       }
    }
    else if (strcmp(format->grismId, "GRISM") == 0)
    {
       n = 0;
       for (i = 38; i < 51; i++)
       {
          if (badChannelList[i])
             continue;
                
          dispResult->trfBinned1 += (dispResult->trf)[i];
          n++;
       }
       if(n>0){
          dispResult->trfBinned1 /= n;
       }
       else{
          dispResult->trfBinned1= -1.;   
       }

       n = 0;
       for (i = 52; i < 69; i++)
       {
          if (badChannelList[i])
             continue;
                
          dispResult->trfBinned2 += (dispResult->trf)[i];
          n++;
       }
       if(n>0){
          dispResult->trfBinned2 /= n;
       }
       else{
          dispResult->trfBinned2= -1.;   
       }

       n = 0;
       for (i = 116; i < 137; i++)
       {
          if (badChannelList[i])
             continue;
                
          dispResult->trfBinned3 += (dispResult->trf)[i];
          n++;
       }
       if(n>0){
          dispResult->trfBinned3 /= n;
       }
       else{
          dispResult->trfBinned3= -1.;   
       }

       n = 0;
       for (i = 176; i < 201; i++)
       {
          if (badChannelList[i])
             continue;
                
          dispResult->trfBinned4 += (dispResult->trf)[i];
          n++;
       }
       if(n>0){
          dispResult->trfBinned4 /= n;
       }
       else{
          dispResult->trfBinned4= -1.;   
       }

       n = 0;
       for (i = 217; i < 246; i++)
       {
          if (badChannelList[i])
             continue;
                
          dispResult->trfBinned5 += (dispResult->trf)[i];
          n++;
       }
       if(n>0){
          dispResult->trfBinned5 /= n;
       }
       else{
          dispResult->trfBinned5= -1.;   
       }
    }
    else
    {
        sprintf (midiMessage, "Unknown GRISM ID ... %s", format->grismId);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
        return;
    }

    //    Report
   cpl_msg_info(cpl_func,"Grism ID: %s \n", format->grismId);
    fprintf (midiReportPtr, "Grism ID: %s QCLOG \n", format->grismId);
    if (strcmp(format->grismId, "PRISM") == 0)
    {
       cpl_msg_info(cpl_func," 8.64 micron (Channel 120 -> 126) = %0.4f \n", dispResult->trfBinned1);
       cpl_msg_info(cpl_func," 9.00 micron (Channel 113 -> 121) = %0.4f \n", dispResult->trfBinned2);
       cpl_msg_info(cpl_func,"10.46 micron (Channel  88 ->  98) = %0.4f \n", dispResult->trfBinned3);
       cpl_msg_info(cpl_func,"11.79 micron (Channel  60 ->  72) = %0.4f \n", dispResult->trfBinned4);
       cpl_msg_info(cpl_func,"12.80 micron (Channel  36 ->  50) = %0.4f \n", dispResult->trfBinned5);

        fprintf (midiReportPtr, " 8.64 micron (Channel 120 -> 126) = %0.4f QCLOG \n", dispResult->trfBinned1);
        fprintf (midiReportPtr, " 9.00 micron (Channel 113 -> 121) = %0.4f QCLOG \n", dispResult->trfBinned2);
        fprintf (midiReportPtr, "10.46 micron (Channel  88 ->  98) = %0.4f QCLOG \n", dispResult->trfBinned3);
        fprintf (midiReportPtr, "11.79 micron (Channel  60 ->  72) = %0.4f QCLOG \n", dispResult->trfBinned4);
        fprintf (midiReportPtr, "12.80 micron (Channel  36 ->  50) = %0.4f QCLOG \n", dispResult->trfBinned5);
    }
    else
    {
       cpl_msg_info(cpl_func," 8.64 micron (Channel  38 ->  50) = %0.4f \n", dispResult->trfBinned1);
       cpl_msg_info(cpl_func," 9.00 micron (Channel  52 ->  68) = %0.4f \n", dispResult->trfBinned2);
       cpl_msg_info(cpl_func,"10.46 micron (Channel 116 -> 136) = %0.4f \n", dispResult->trfBinned3);
       cpl_msg_info(cpl_func,"11.79 micron (Channel 176 -> 200) = %0.4f \n", dispResult->trfBinned4);
       cpl_msg_info(cpl_func,"12.80 micron (Channel 217 -> 245) = %0.4f \n", dispResult->trfBinned5);

        fprintf (midiReportPtr, " 8.64 micron (Channel  38 ->  50) = %0.4f QCLOG \n", dispResult->trfBinned1);
        fprintf (midiReportPtr, " 9.00 micron (Channel  52 ->  68) = %0.4f QCLOG \n", dispResult->trfBinned2);
        fprintf (midiReportPtr, "10.46 micron (Channel 116 -> 136) = %0.4f QCLOG \n", dispResult->trfBinned3);
        fprintf (midiReportPtr, "11.79 micron (Channel 176 -> 200) = %0.4f QCLOG \n", dispResult->trfBinned4);
        fprintf (midiReportPtr, "12.80 micron (Channel 217 -> 245) = %0.4f QCLOG \n", dispResult->trfBinned5);
    }
    
    return;
}
/*****************************************************************************/




#ifdef NEW_PBL
From: Anders Wallander [awalland@eso.org]
Sent: Wednesday, 12 July, 2006 10:46
To: Isabelle Percheron
Cc: tlicha@eso.org; csabet@eso.org; awalland@eso.org
Subject: Re: VLTI header keyword ISS PBLAxy START/END

Attachments: VLTSW20060066.PDF
Isabelle at al,

The keyword still represents the value of projected baseline at the beginning and end of integration.
The format is the same, but the definition of projected baseline is different, hence a new algorithm.
Attached is the change request. The plan is to install this at Paranal end of September.
The point is that pipelines should not break. I am waiting for your comments on this

Below the 2 different algorithms (GIP is current, RMA is future) to compute PBLA.

Cheers,
Anders

double pblaGIP (double alt, double az, double T1[3], double T2[3])
{

    vltDOUBLE           star[]          = {0.0, 0.0, 0.0};
    vltDOUBLE           baseline[]      = {0.0, 0.0, 0.0};
    vltDOUBLE           projBaseline[]  = {0.0, 0.0, 0.0};
    vltDOUBLE           sidOpd          = 0.0;
    vltDOUBLE           cosAlt          = 0.0;
    vltDOUBLE           sinAlt          = 0.0;
    vltDOUBLE           cosAz           = 0.0;
    vltDOUBLE           sinAz           = 0.0;

    double              gip;

    cosAlt  = cos(alt * DDEG2RAD);
    sinAlt  = sin(alt * DDEG2RAD);
    cosAz   = cos(az * DDEG2RAD);
    sinAz   = sin(az * DDEG2RAD);

    star[0] = sinAz * cosAlt;
    star[1] = (- cosAz) * cosAlt;
    star[2] = sinAlt;

    baseline[0] = T2[0] - T1[0];
    baseline[1] = T2[1] - T1[1];
    baseline[2] = T2[2] - T1[2];

    /* scalar product <star>*<baseline> */
    sidOpd = star[0] * baseline[0] + star[1] * baseline[1] + star[2] * baseline[2];
    /* projected baseline vector in XYZ coordinate system */
    projBaseline[0] = baseline[0] - sidOpd * star[0];
    projBaseline[1] = baseline[1] - sidOpd * star[1];
    projBaseline[2] = baseline[2] - sidOpd * star[2];
            
    gip = atan(projBaseline[0]/projBaseline[1]) * DRAD2DEG;

    return gip;

}


double pblaRMA (double alt, double az, double prltic, double T1[3], double T2[3])
{

    vltDOUBLE           a, A, A_b, p;
    vltDOUBLE           sinpsi, cospsi, psi;

    double              rma;

    /*
     * 1. Compute A, star azimuth, with FITS convention; 0=south, +90 west
     *    VLT convention is 0=south, +90 east
     */
    A = (360 - az) * DDEG2RAD;

    /*
     * 2. Compute baseline azimuth on the platform
     *    This is zero if T2 is south of T1 and +90 deg if T2 is west of T1.
     *    It seems that DLCS applies the convention X pos to west and Y pos to south
     *    for station coordinates (T vectors).
     *    In a south-over-west convention for 'A_b' and 'a', the y-component
     *    of the atan2(y,x) must represent +West and the x-component +South.
     *    The vector from T1[.] to T2[.] is obtained as T2[.]-T1[.].
     */
    A_b = atan2( (T2[0] - T1[0]), (T2[1] - T1[1]) );
    
    /*
     * 3. Get a, star elevation
     *    This is directly given by VLT altitude
     */
    a = alt * DDEG2RAD;

    /*
     * 4. Compute auxiliary angle psi
     */
    sinpsi = sin(A_b - A);
    cospsi = cos(A_b - A) * sin(a);
    psi = atan2(sinpsi,cospsi);

    /*
     * 5. Get parallactic angle.
     *    This value is computed by ISS using standard slaPa 
     */
    p = prltic * DDEG2RAD;

    /*
     * 6. Compute final angle
     *
     */
    rma = slaDranrm(p + psi + DPI) * DRAD2DEG;

    return rma;

}

Isabelle Percheron wrote: 
Anders

I was checking with Cyrus about the uses of the keywords, the pipelines (MIDI
and AMBER for sure) read it and in the case of MIDI uses it, I am stilll not
sure if AMBER pipeline just retrieves it or uses it for other calculations, I
guess they have to retrieve the baseline from somewhere maybe from the stations,
object ...

So just to be sure, could you confirm if this keyword will still represent the
value of the projected baseline at the beginning of the integration and at the
end. How is the format affected (still the same?). And last could you please
tell us when this is implemented on Paranal for Cyrus to check.
Thanks
Isabelle

#endif
  
