/* $Id: midi_acq.c,v 1.23 2011-11-21 09:40:20 agabasch Exp $
 *
 * This file is part of the MIDI Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: agabasch $
 * $Date: 2011-11-21 09:40:20 $
 * $Revision: 1.23 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <stdio.h>
#include "midi_utils.h"
#include "midi_pfits.h"
#include "midi_dfs.h"
#include "midiControl.h"
#include "midiGlobal.h"
#include "midiAppendPropertylist.h"
#include <string.h>
#include "midi_cplupgrade.h"

/*-----------------------------------------------------------------------------
                            Functions prototypes
 -----------------------------------------------------------------------------*/

static int midi_acq_create(cpl_plugin *) ;
static int midi_acq_exec(cpl_plugin *) ;
static int midi_acq_destroy(cpl_plugin *) ;
static int midi_acq(cpl_parameterlist *, cpl_frameset *) ;
static int table_to_imglst(const char * columname,
		const char * columntype,
		cpl_imagelist * imglst_target,
		const char * columnentry,
		cpl_table * table);
static cpl_error_code midi_qc_acq(const cpl_image * image,
		cpl_propertylist * pro_list);
static cpl_error_code midi_image_get_fwhm(
		const cpl_image *   in,
		int                 xpos,
		int                 ypos,
		double          *   fwhm_x,
		double          *   fwhm_y);
static double midi_vector_get_fwhm(
		const cpl_vector    *   vec,
		int                     pos,
		double                  half_max);
static int midi_isnan(double value);


/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

static char midi_acq_description[] =
		"The purpose of this recipe is to assess the position, size and the flux\n"
		"intensity of the target.\n\n"
		"Input files:\n\n"
		"  DO category:               Type:       Explanation:         Required:\n"
		"  ACQ                        Raw         Raw data frame          Y\n\n"
		"Output files:\n\n"
		"  DO category:               Data type:  Explanation:\n"
		"  IMAGE_QUALITY              FITS image  Image of the beam\n\n";

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Build the list of available plugins, for this module. 
  @param    list    the plugin list
  @return   0 if everything is ok, -1 otherwise

  Create the recipe instance and make it available to the application using the 
  interface. This function is exported.
 */
/*----------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
	cpl_recipe  *   recipe = cpl_calloc(1, sizeof *recipe ) ;
	cpl_plugin  *   plugin = &recipe->interface ;

	cpl_plugin_init(plugin,
			CPL_PLUGIN_API,
			MIDI_BINARY_VERSION,
			CPL_PLUGIN_TYPE_RECIPE,
			"midi_acq",
			"Image quality assessment",
			midi_acq_description,
			"Coorosh Sabet",
			PACKAGE_BUGREPORT,
			midi_get_license(),
			midi_acq_create,
			midi_acq_exec,
			midi_acq_destroy) ;

	cpl_pluginlist_append(list, plugin) ;

	return 0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Setup the recipe options    
  @param    plugin  the plugin
  @return   0 if everything is ok

  Defining the command-line/configuration parameters for the recipe.
 */
/*----------------------------------------------------------------------------*/
static int midi_acq_create(cpl_plugin * plugin)
{
	cpl_recipe      * recipe ;
	/*    cpl_parameter   * p ;*/

	/* Check that the plugin is part of a valid recipe */
	if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE)
		recipe = (cpl_recipe *)plugin ;
	else return -1 ;

	/* Create the parameters list in the cpl_recipe object */
	recipe->parameters = cpl_parameterlist_new() ;


	/* Return */
	return 0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Execute the plugin instance given by the interface
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_acq_exec(cpl_plugin * plugin)
{
	cpl_recipe  *   recipe ;

	/* Get the recipe out of the plugin */
	if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE)
		recipe = (cpl_recipe *)plugin ;
	else return -1 ;

	batchNumber=0;
	return midi_acq(recipe->parameters, recipe->frames) ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Destroy what has been created by the 'create' function
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_acq_destroy(cpl_plugin * plugin)
{
	cpl_recipe  *   recipe ;

	/* Get the recipe out of the plugin */
	if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE)
		recipe = (cpl_recipe *)plugin ;
	else return -1 ;

	cpl_parameterlist_delete(recipe->parameters) ;
	return 0 ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    parlist     the parameters list
  @param    frameset   the frames list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_acq(
		cpl_parameterlist	*parlist,
		cpl_frameset		*frameset)
{

	cpl_frame		 * current_frame;
	int				   error=0;
	FILE			 * sofPtr=NULL;
	int				   plotDuration;
	cpl_propertylist * header=NULL;

	cpl_frame        * cur_frame=NULL;
	cpl_table        * table=NULL;
	cpl_imagelist    * imglst_data1;
	cpl_imagelist    * imglst_data2;
	cpl_image        * image_data1, * image_data2;
	cpl_image        * image_data1_mask, * image_data2_mask;
	cpl_mask         * mask_data1, * mask_data2;
	cpl_matrix       * filter;
	cpl_errorstate     prestate = cpl_errorstate_get();
	int                ext_imaging_data=0;
	double             median_data1, median_data2;
	cpl_propertylist * qclist_data1=NULL;
	cpl_propertylist * qclist_data2=NULL;
	char               station[]= "AT";
	plotDuration=0;


	sofPtr = fopen ("MIDI_sof.log", "w");
	cpl_frameset_iterator * it = cpl_frameset_iterator_new(frameset);
	current_frame = cpl_frameset_iterator_get(it);
	while ( current_frame && sofPtr )
	{	
		fprintf (sofPtr,"%s \n", (char *)cpl_frame_get_filename(current_frame));
		cpl_frameset_iterator_advance(it, 1);
		current_frame = cpl_frameset_iterator_get(it);
	} /* All frames from frameset */
	fclose (sofPtr);
	cpl_frameset_iterator_delete(it);

	/* NOW PERFORMING THE DATA REDUCTION */
	executeDataReduction ("", "","./", plotDuration,"MIDI_sof.log",&error,
			parlist,frameset);
	if (error) return -1;
	remove ("MIDI_sof.log");

	if (CPL_ERROR_NONE !=  appendPropertylist("MIDI_b1_acq_DATA1.pro.fits",
			CPL_FRAME_TYPE_IMAGE, "IMAGE_QUALITY",frameset,parlist))
	{
		cpl_msg_error(cpl_func,"Error in appendPropertylist");
	}

	if (CPL_ERROR_NONE !=  appendPropertylist("MIDI_b1_acq_DATA2.pro.fits",
			CPL_FRAME_TYPE_IMAGE, "IMAGE_QUALITY",frameset,parlist))
	{
		cpl_msg_error(cpl_func,"Error in appendPropertylist");
	}


	/*------------------------------------------------------------------------*/



	imglst_data1=cpl_imagelist_new();
	imglst_data2=cpl_imagelist_new();


	cur_frame=cpl_frameset_find(frameset,MIDI_ACQ);
	if (cur_frame == NULL) {
		return (int)cpl_error_set_message(cpl_func, CPL_ERROR_DATA_NOT_FOUND,
				"SOF does not have any file");
	}
	/* Append the images from the tables to the imagelists */

	cpl_msg_info(cpl_func,"Processing file %s",
			cpl_frame_get_filename(cur_frame));

	header = cpl_propertylist_load(cpl_frame_get_filename(cur_frame), 0);


	/*       Checking if UT or AT */
	strcpy(station,"AT");

	if (cpl_propertylist_has(header, "ESO ISS CONF STATION1") == 1)
	{
		if(strcmp(cpl_propertylist_get_string(header,
				"ESO ISS CONF STATION1"),"U1")==0 ||
				strcmp(cpl_propertylist_get_string(header,
						"ESO ISS CONF STATION1"),"U2")==0 ||
						strcmp(cpl_propertylist_get_string(header,
								"ESO ISS CONF STATION1"),"U3")==0 ||
								strcmp(cpl_propertylist_get_string(header,
										"ESO ISS CONF STATION1"),"U4")==0){
			strcpy(station,"UT");
		}
	}


	/* Load extension  Imaging Data  */
	ext_imaging_data=cpl_fits_find_extension(cpl_frame_get_filename(cur_frame),
			"IMAGING_DATA");
	table =cpl_table_load(cpl_frame_get_filename(cur_frame),ext_imaging_data,1);
	if (table == NULL) {
		cpl_imagelist_delete(imglst_data1);
		cpl_imagelist_delete(imglst_data2);
		cpl_propertylist_delete(header);
		return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
				"Could not load the table");
	}
	cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());

	cpl_msg_indent_more();
	cpl_msg_info(cpl_func,"Reading DATA1 and DATA2 frames into imagelists ...");
	/* Adding the data1/data2 sky and target images to the imagelist*/
	if (cpl_table_has_column(table,"DATA1")){
		table_to_imglst("DATA1","TARTYP1",imglst_data1,"S",table);
		table_to_imglst("DATA1","TARTYP1",imglst_data1,"T",table);
	}
	if (cpl_table_has_column(table,"DATA2")){
		table_to_imglst("DATA2","TARTYP2",imglst_data2,"S",table);
		table_to_imglst("DATA2","TARTYP2",imglst_data2,"T",table);

	}

	if (cpl_msg_get_level() <= CPL_MSG_DEBUG) {
		cpl_msg_info(cpl_func, "Processed DATA1 frames: % " CPL_SIZE_FORMAT "",
				cpl_imagelist_get_size(imglst_data1));
		cpl_msg_info(cpl_func, "Processed DATA2 frames: % " CPL_SIZE_FORMAT "",
				cpl_imagelist_get_size(imglst_data2));
		cpl_imagelist_save(imglst_data1,"imglst_data1.fits", CPL_BPP_IEEE_FLOAT,
				NULL, CPL_IO_CREATE);
		cpl_imagelist_save(imglst_data2,"imglst_data2.fits", CPL_BPP_IEEE_FLOAT,
				NULL, CPL_IO_CREATE);
	}

	cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
	cpl_table_delete(table);


	/*Average the imagelist to get an cpl_image*/
	cpl_msg_info(cpl_func, "Averaging the imagelists ...");
	image_data1=cpl_imagelist_collapse_create(imglst_data1);
	image_data2=cpl_imagelist_collapse_create(imglst_data2);

	cpl_msg_info(cpl_func, "Calculating the median ...");
	median_data1=cpl_image_get_median(image_data1);
	median_data2=cpl_image_get_median(image_data2);
	if (cpl_msg_get_level() <= CPL_MSG_DEBUG) {
		cpl_msg_info(cpl_func,"First median DATA1: %f DATA12:%f", median_data1,
				median_data2);
	}


	/* Create a bad pixel mask and connect it with the image*/
	cpl_msg_info(cpl_func, "Creating a mask by using the median ...");
	mask_data1=cpl_mask_threshold_image_create(image_data1,0., median_data1);
	cpl_image_reject_from_mask(image_data1, mask_data1) ;
	mask_data2=cpl_mask_threshold_image_create(image_data2,0., median_data2);
	cpl_image_reject_from_mask(image_data2, mask_data2) ;

	/*Threshold the images */
	cpl_msg_info(cpl_func, "Thresholding the images by using the median ...");
	cpl_image_threshold(image_data1,median_data1,FLT_MAX,0.,FLT_MAX);
	cpl_image_threshold(image_data2,median_data2,FLT_MAX,0.,FLT_MAX);

	if (cpl_msg_get_level() <= CPL_MSG_DEBUG) {
		cpl_image_save(image_data1, "image_data1.fits", CPL_BPP_IEEE_FLOAT,
				NULL, CPL_IO_CREATE);
		cpl_image_save(image_data2, "image_data2.fits", CPL_BPP_IEEE_FLOAT,
				NULL, CPL_IO_CREATE);
	}


	/* Re Calculate the median deviation using the mask file*/

	cpl_msg_info(cpl_func, "Re-calculating the median after thresholding ...");
	median_data1=cpl_image_get_median(image_data1);
	median_data2=cpl_image_get_median(image_data2);

	if (cpl_msg_get_level() <= CPL_MSG_DEBUG) {
		cpl_msg_info(cpl_func,"Second median DATA1: %f DATA2:%f", median_data1,
				median_data2);
	}


	/* Apply dilatation to decrease the good pixel area */
	cpl_msg_info(cpl_func, "Dilate the mask ...");
	filter = cpl_matrix_new(7, 7);
	cpl_matrix_fill(filter, 1.0);
	cpl_mask_dilation(mask_data1, filter);
	cpl_mask_dilation(mask_data2, filter);
	cpl_matrix_delete(filter);

	cpl_mask_not(mask_data1);
	cpl_mask_not(mask_data2);

	/*convert the mask to a cpl_image*/
	image_data1_mask=cpl_image_new_from_mask(mask_data1);
	image_data2_mask=cpl_image_new_from_mask(mask_data2);

	if (cpl_msg_get_level() <= CPL_MSG_DEBUG) {
		cpl_image_save(image_data1_mask, "image_data1_mask.fits",
				CPL_BPP_IEEE_FLOAT, NULL, CPL_IO_CREATE);
		cpl_image_save(image_data2_mask, "image_data2_mask.fits",
				CPL_BPP_IEEE_FLOAT, NULL, CPL_IO_CREATE);
	}




	/* multiply mask and image */
	cpl_msg_info(cpl_func, "Multiply the dilated mask and the image ...");
	cpl_image_multiply(image_data1,image_data1_mask);
	cpl_image_multiply(image_data2,image_data2_mask);

	cpl_msg_indent_less();
	if (strcmp(station,"AT")==0)
	{

		cpl_image_subtract_scalar(image_data1,median_data1);
		cpl_image_subtract_scalar(image_data2,median_data2);

		/*Threshold the images */
		cpl_image_threshold(image_data1, (median_data1*(-1.))+1., FLT_MAX,
				0., FLT_MAX);
		cpl_image_threshold(image_data2, (median_data2*(-1.))+1. ,FLT_MAX,
				0., FLT_MAX);

		cpl_image_multiply_scalar(image_data1,-1.);
		cpl_image_multiply_scalar(image_data2,-1.);
	}


	if (cpl_msg_get_level() <= CPL_MSG_DEBUG) {
		cpl_image_save(image_data1,"image_data1_final.fits", CPL_BPP_IEEE_FLOAT,
				NULL, CPL_IO_CREATE);
		cpl_image_save(image_data2,"image_data2_final.fits", CPL_BPP_IEEE_FLOAT,
				NULL, CPL_IO_CREATE);
	}

	qclist_data1=cpl_propertylist_new();
	qclist_data2=cpl_propertylist_new();


	/*Derive QC parameters*/

	prestate = cpl_errorstate_get();

	midi_qc_acq(image_data1,qclist_data1);
	midi_qc_acq(image_data2,qclist_data2);


	if (!cpl_errorstate_is_equal(prestate)) {
		/*An error happened*/
		cpl_msg_warning(cpl_func, "An error occurred! ");
		cpl_msg_indent_more();
		cpl_errorstate_dump(prestate, CPL_FALSE, NULL);
		cpl_msg_indent_less();
	}

	cpl_msg_warning(cpl_func, "Trying to recover ... ");

	cpl_errorstate_set(prestate);




	cpl_propertylist_update_string(qclist_data1, CPL_DFS_PRO_CATG,
			"MIDI_ACQ_FOV_DATA1");
	if (cpl_dfs_save_image(frameset, header, parlist, frameset, NULL,
			image_data1, CPL_BPP_IEEE_FLOAT, "midi_acq", qclist_data1, NULL,
			PACKAGE "/" PACKAGE_VERSION, "midi_acq_fov_data1.fits")) {
		/* Propagate the error */
		(void)cpl_error_set_where(cpl_func);
	}

	cpl_propertylist_update_string(qclist_data2, CPL_DFS_PRO_CATG,
			"MIDI_ACQ_FOV_DATA2");
	if (cpl_dfs_save_image(frameset, header, parlist, frameset, NULL,
			image_data2, CPL_BPP_IEEE_FLOAT, "midi_acq", qclist_data2, NULL,
			PACKAGE "/" PACKAGE_VERSION, "midi_acq_fov_data2.fits")) {
		/* Propagate the error */
		(void)cpl_error_set_where(cpl_func);
	}
	/*
	cpl_image_save(image_data1, "armin1_delayline.fits", CPL_BPP_IEEE_FLOAT,
			qclist_data1, CPL_IO_CREATE);
	cpl_image_save(image_data2, "armin2_delayline.fits", CPL_BPP_IEEE_FLOAT,
			qclist_data2, CPL_IO_CREATE);
	 */

	if (cpl_error_get_code() != CPL_ERROR_NONE) {
		cpl_msg_error(cpl_func, "%s():%d: An error is already set: %s",
				cpl_func, __LINE__, cpl_error_get_message());

		cpl_msg_error(cpl_func,"%s %s",cpl_error_get_message(),
				cpl_error_get_where());
	}
	/* un-connect the mask and free the memory*/
	cpl_image_accept_all(image_data1) ;
	cpl_mask_delete(mask_data1);
	cpl_image_accept_all(image_data2) ;
	cpl_mask_delete(mask_data2);

	/* Free the memory */

	cpl_imagelist_delete(imglst_data1);
	cpl_imagelist_delete(imglst_data2);
	cpl_image_delete(image_data1);
	cpl_image_delete(image_data2);
	cpl_image_delete(image_data1_mask);
	cpl_image_delete(image_data2_mask);
	cpl_propertylist_delete(header);
	cpl_propertylist_delete(qclist_data1);
	cpl_propertylist_delete(qclist_data2);


	/*---------------------------------------------------------------------*/



	/* Return */
	if (cpl_error_get_code())
		return -1 ;
	else
		return 0 ;
}

static int table_to_imglst(const char * columname,
		const char * columntype,
		cpl_imagelist * imglst_target,
		const char * columnentry,
		cpl_table * table)
{

	int dimenDATA;
	int   i, ctarget=0;
	cpl_array * array_data=NULL;
	cpl_image * image_data=NULL;
	cpl_errorstate          prestate = cpl_errorstate_get();
	char ** target_type;
	cpl_type ctype;

	/* Load extension  Imaging Data  */

	dimenDATA=cpl_table_get_column_dimensions(table, columname);
	cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
	if (dimenDATA != 2) {
		return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
				"DATA has a wrong dimension");
	}
	/* Read target type columnentry and store them in the image list */
	/* Loop over all "images" stored in DATA and add them to the imagelist */

	if (cpl_table_has_column(table, columntype))
	{
		target_type=cpl_table_get_data_string(table, columntype);

	}
	else
	{
		return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
				"TYPE of the Column not found");
	}

	ctype=cpl_table_get_column_type(table, columname);
	/*    cpl_msg_info(cpl_func, "Type of the table column: %d",ctype); */

	ctarget=cpl_imagelist_get_size(imglst_target);

	for (i=0; i<cpl_table_get_nrow(table);i++)
	{

		array_data=(cpl_array *)cpl_table_get_array(table,columname, i);
		if(ctype&CPL_TYPE_INT){
			image_data=cpl_image_wrap_int(
					cpl_table_get_column_dimension(table,columname,0 ),
					cpl_table_get_column_dimension(table,columname,1 ),
					cpl_array_get_data_int(array_data) );
		}
		if(ctype&CPL_TYPE_FLOAT){
			image_data=cpl_image_wrap_float(
					cpl_table_get_column_dimension(table,columname,0 ),
					cpl_table_get_column_dimension(table,columname,1 ),
					cpl_array_get_data_float(array_data) );
		}
		/* Cast the image to float */
		/*       image_data=cpl_image_cast(image_data, CPL_TYPE_FLOAT); */

		/*        Append the image to the imagelists */


		if(strcmp(target_type[i],columnentry)== 0){
			cpl_imagelist_set(imglst_target,
					cpl_image_cast(image_data, CPL_TYPE_FLOAT),ctarget++);
			cpl_image_unwrap(image_data);
			continue;
		}

		/*Unwrap processed image manualy if nor Target nor Sky*/
		if(image_data!=NULL){
			cpl_image_unwrap(image_data);
		}

	}

	return (int)cpl_error_get_code();
}

static cpl_error_code midi_qc_acq(const cpl_image * image,
		cpl_propertylist * pro_list)
{

	cpl_array  *parameters = NULL;
	cpl_array  *err_params = NULL;
	cpl_array  *fit_params = NULL;
	double      major=0.0,minor=0.0,angle=0.0,rms=0.0;
	double      centroid_x=0.0, centroid_y=0.0;
	double      centroid_fwhm_x=0.0, centroid_fwhm_y=0.0;
	double      gauss_x=0.0, gauss_y=0.0;
	double      gauss_sigma_x=0.0, gauss_sigma_y=0.0;
	double      gauss_fwhm_x=0.0, gauss_fwhm_y=0.0;
	double      dummy=0.;
	int         i=0;



	const char  *p[7] = { /* Parameter names */
			"Background       ",
			"Normalisation    ",
			"Correlation      ",
			"Center position x",
			"Center position y",
			"Sigma x          ",
			"Sigma y          "};

	parameters = cpl_array_new(7, CPL_TYPE_DOUBLE);
	err_params = cpl_array_new(7, CPL_TYPE_DOUBLE);
	fit_params = cpl_array_new(7, CPL_TYPE_INT);

	/*All parameter should be fitted*/
	for (i = 0; i < 7; i++)
		cpl_array_set(fit_params, i, 1);


	/*Freez the background to a value of zero*/
	//cpl_array_set(fit_params, 0, 0);
	//cpl_array_set(parameters, 0,0.);

	/*Fit a gaussian*/
	if(cpl_fit_image_gaussian(image, NULL, 30, 30, 20, 20, parameters, NULL,
			fit_params, &rms, NULL, NULL, &major, &minor, &angle, NULL)
			== CPL_ERROR_NONE &&
			!midi_isnan(cpl_array_get(parameters,0,NULL)) &&
			!midi_isnan(cpl_array_get(parameters,1,NULL)) &&
			!midi_isnan(cpl_array_get(parameters,2,NULL)) &&
			!midi_isnan(cpl_array_get(parameters,3,NULL)) &&
			!midi_isnan(cpl_array_get(parameters,4,NULL)) &&
			!midi_isnan(cpl_array_get(parameters,5,NULL)) &&
			!midi_isnan(cpl_array_get(parameters,6,NULL)))
	{


		for (i = 0; i < 7; i++){
			cpl_msg_info(cpl_func,"%s: %f",
					p[i], cpl_array_get(parameters,i,NULL));
		}

		gauss_x=cpl_array_get(parameters,3,NULL);
		gauss_y=cpl_array_get(parameters,4,NULL);
		gauss_sigma_x=cpl_array_get(parameters,5,NULL);
		gauss_sigma_y=cpl_array_get(parameters,6,NULL);

		/*Get the FWHM at the center of the gaussian fit*/
		cpl_image_get_fwhm(image, gauss_x, gauss_y, &gauss_fwhm_x,
				&gauss_fwhm_y);

		if(gauss_fwhm_x<0){
			midi_image_get_fwhm(image, gauss_x, gauss_y, &gauss_fwhm_x,
					&dummy);
		}
		if(gauss_fwhm_y<0){
			midi_image_get_fwhm(image, gauss_x, gauss_y, &dummy, &gauss_fwhm_y);

		}

	}
	/*Get the centroid of the image*/
	centroid_x=cpl_image_get_centroid_x(image);
	centroid_y=cpl_image_get_centroid_y(image);
	/*Get the FWHM at the centroid position*/
	cpl_image_get_fwhm(image, centroid_x, centroid_y, &centroid_fwhm_x,
			&centroid_fwhm_y);


	if(centroid_fwhm_x<0){
		midi_image_get_fwhm(image, centroid_x, centroid_y, &centroid_fwhm_x,
				&dummy);
	}
	if(centroid_fwhm_y<0){
		midi_image_get_fwhm(image, centroid_x, centroid_y, &dummy,
				&centroid_fwhm_y);
	}



	cpl_propertylist_update_float(pro_list,
			"ESO QC GAUSS FIT X", (float)gauss_x);
	cpl_propertylist_update_float(pro_list,
			"ESO QC GAUSS FIT Y", (float)gauss_y);
	cpl_propertylist_update_float(pro_list,
			"ESO QC GAUSS FIT SIGMA X", (float)gauss_sigma_x);
	cpl_propertylist_update_float(pro_list,
			"ESO QC GAUSS FIT SIGMA Y", (float)gauss_sigma_y);
	cpl_propertylist_update_float(pro_list,
			"ESO QC GAUSS MEASURED FWHM AT X", (float)gauss_fwhm_x);
	cpl_propertylist_update_float(pro_list,
			"ESO QC GAUSS MEASURED FWHM AT Y", (float)gauss_fwhm_y);

	cpl_propertylist_update_float(pro_list,
			"ESO QC CENTROID X", (float)centroid_x);
	cpl_propertylist_update_float(pro_list,
			"ESO QC CENTROID Y", (float)centroid_y);
	cpl_propertylist_update_float(pro_list,
			"ESO QC CENTROID MEASURED FWHM AT X", (float)centroid_fwhm_x);
	cpl_propertylist_update_float(pro_list,
			"ESO QC CENTROID MEASURED FWHM AT Y", (float)centroid_fwhm_y);

	if (cpl_msg_get_level() <= CPL_MSG_DEBUG) {
		cpl_msg_info(cpl_func,"centroid_x: %f centroid_y: %f", centroid_x,
				centroid_y);
		cpl_msg_info(cpl_func,"centroid_fwhm_x: %f centroid_fwhm_y: %f",
				centroid_fwhm_x, centroid_fwhm_y);
	}

	/*Free the memory*/
	cpl_array_delete(parameters);
	cpl_array_delete(err_params);
	cpl_array_delete(fit_params);

	return cpl_error_get_code();
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Compute FWHM values in x and y for an object
  @param    in      the input image
  @param    xpos    the x position of the object (1 for the first pixel)
  @param    ypos    the y position of the object (1 for the first pixel)
  @param    fwhm_x  the computed FWHM in x or -1 on error
  @param    fwhm_y  the computed FWHM in y or -1 on error
  @return   CPL_ERROR_NONE or the relevant #_cpl_error_code_

  This function uses a basic method: start from the center of the object
  and go away until the half maximum value is reached in x and y.

  For the FWHM in x (resp. y) to be computed, the image size in the x (resp. y)
  direction should be at least of 5 pixels.

  If for any reason, one of the FHWMs cannot be computed, its returned value
  is -1.0, but an error is not necessarily raised. For example, if a 4 column
  image is passed, the fwhm_x would be -1.0, the fwhm_y would be correctly
  computed, and no error would be raised.

  Possible #_cpl_error_code_ set in this function:
  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  - CPL_ERROR_DATA_NOT_FOUND if (xpos, ypos) specifies a rejected pixel or a
      pixel with a non-positive value
  - CPL_ERROR_ACCESS_OUT_OF_RANGE if xpos or ypos is outside the image
    size range
 */
/*----------------------------------------------------------------------------*/
static cpl_error_code midi_image_get_fwhm(
		const cpl_image *   in,
		int                 xpos,
		int                 ypos,
		double          *   fwhm_x,
		double          *   fwhm_y)
{
	double              half_max;
	const int           minimum_size = 5;
	int                 is_rejected;


	/* Check entries - and initialize *fwhm_{x,y} */
	if (fwhm_y != NULL) *fwhm_y = -1;
	cpl_ensure_code(fwhm_x,         CPL_ERROR_NULL_INPUT);
	*fwhm_x = -1;
	cpl_ensure_code(fwhm_y,         CPL_ERROR_NULL_INPUT);

	/* This call will check the validity of image, xpos and ypos */
	half_max = 0.5 * cpl_image_get(in, xpos, ypos, &is_rejected);

	cpl_ensure_code(is_rejected >= 0, cpl_error_get_code());
	cpl_ensure_code(!is_rejected, CPL_ERROR_DATA_NOT_FOUND);

	cpl_ensure_code(half_max > 0, CPL_ERROR_DATA_NOT_FOUND);

	/* FWHM in x */
	if (cpl_image_get_size_x(in) >= minimum_size) {
		cpl_errorstate pstate;

		/* Extract the vector centered on the maximum */
		cpl_vector * row = cpl_vector_new_from_image_row(in, ypos);

		/* If an error happened, update its location */
		cpl_ensure_code(row, cpl_error_get_code());

		pstate = cpl_errorstate_get();


		/* Compute the FWHM */
		if (cpl_errorstate_is_equal(pstate))
			*fwhm_x = midi_vector_get_fwhm(row, xpos, half_max);

		cpl_vector_delete(row);

		/* Propagate the error, if any */
		cpl_ensure_code(cpl_errorstate_is_equal(pstate), cpl_error_get_code());

	}

	/* FWHM in y */
	if (cpl_image_get_size_y(in)>= minimum_size) {
		cpl_errorstate pstate;

		/* Extract the vector centered on the maximum */
		cpl_vector * col = cpl_vector_new_from_image_column(in, xpos);

		/* If an error happened, update its location */
		cpl_ensure_code(col, cpl_error_get_code());

		pstate = cpl_errorstate_get();



		/* Compute the FWHM */
		if (cpl_errorstate_is_equal(pstate))
			*fwhm_y = midi_vector_get_fwhm(col, ypos, half_max);

		cpl_vector_delete(col);

		/* Propagate the error, if any */
		cpl_ensure_code(cpl_errorstate_is_equal(pstate), cpl_error_get_code());
	}

	return CPL_ERROR_NONE;
}
/*----------------------------------------------------------------------------*/
/**
  @brief    Compute the FWHM of an object in a cpl_vector
  @param    vec         the input cpl_vector
  @param    pos         the object position (from 1 to the vector size)
  @param    half_max    the half maximum value
  @return   the FWHM or a negative value on error
  @note The return value may be -1 with no error condition

  Possible #_cpl_error_code_ set in this function:
  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  - CPL_ERROR_ILLEGAL_INPUT if pos is less than 1 or greater than the vec-length
 */
/*----------------------------------------------------------------------------*/
static double midi_vector_get_fwhm(
		const cpl_vector    *   vec,
		int                     pos,
		double                  half_max)
{
	const double * vec_data;
	int            nelem;
	double         x_left, x_right;
	double         y_1, y_2;
	int            i;

	/* Check entries */
	cpl_ensure(vec,          CPL_ERROR_NULL_INPUT, -1.0);
	nelem = cpl_vector_get_size(vec);
	cpl_ensure(pos >= 1,     CPL_ERROR_ILLEGAL_INPUT, -1.0);
	cpl_ensure(pos <= nelem, CPL_ERROR_ILLEGAL_INPUT, -1.0);

	vec_data = cpl_vector_get_data_const(vec);

	/* Object may be too noisy - or strange in some other way */
	if (vec_data[pos - 1] <= half_max) return -1.0;

	/* Find first pair of values, y(i) <= half_max < y(i+1)
         on the left of the maximum */
	i = pos - 1;

	while ((vec_data[i] > half_max) && (i > 0)) i--;
	if (vec_data[i] > half_max) return -1.0;  /* y_1 could not be found */

	y_1 = vec_data[i];
	y_2 = vec_data[i+1];

	/* assert ( y_1 <= half_max && half_max < y_2 ); */

	/* Assume linearity between y_1 and y_2 */
	x_left = i + (half_max-y_1) / (y_2-y_1);

	/* assert( x_left >= i ); */

	/* Find first pair of values, y(i-1) > half_max >= y(i)
         on the right of the maximum */
	i = pos - 1;

	while ((vec_data[i] > half_max) && (i < nelem-1)) i++;
	if (vec_data[i] > half_max) return -1.0;   /* y_2 could not be found */

	y_1 = vec_data[i-1];
	y_2 = vec_data[i];

	/* assert( y_1 > half_max && half_max >= y_2 ); */

	/* Assume linearity between y_1 and y_2 */
	x_right = i + (half_max-y_2) / (y_2-y_1);

	/* assert( x_right < i ); */

	if (x_right < x_left || x_right - x_left > FLT_MAX) return -1;

	return x_right - x_left;
}
/*----------------------------------------------------------------------------*/
/**
   @brief   portable isnan
 */
/*----------------------------------------------------------------------------*/
static int midi_isnan(double value)
{
#if defined HAVE_ISNAN && HAVE_ISNAN
	return isnan(value);
#else
	return value != value;
#endif
}
