/* $Id: midi_fringe_all.c,v 1.14 2011-02-10 14:46:35 agabasch Exp $
 *
 * This file is part of the MIDI Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: agabasch $
 * $Date: 2011-02-10 14:46:35 $
 * $Revision: 1.14 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <stdio.h>
#include <string.h>
#include "midi_utils.h"
#include "midi_pfits.h"
#include "midi_dfs.h"
#include "midiControl.h"
#include "midiGlobal.h"
#include "midiAppendPropertylist.h"

/*-----------------------------------------------------------------------------
                            Functions prototypes
 -----------------------------------------------------------------------------*/

static int midi_fringe_all_create(cpl_plugin *) ;
static int midi_fringe_all_exec(cpl_plugin *) ;
static int midi_fringe_all_destroy(cpl_plugin *) ;
static int midi_fringe_all(cpl_parameterlist *, cpl_frameset *) ;

/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

static char midi_fringe_all_description[] =
"This recipe is able to compute Uncalibrated Visibilities, Transfer Functions\n"
"and Calibrated Visibilities for HIGH_SENS and SCI_PHOT input data.\n"
"As the input frames and the output products of this recipe are manifold, the\n"
"user should refer to the pipeline manual for a detailed description \n\n";

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Build the list of available plugins, for this module. 
  @param    list    the plugin list
  @return   0 if everything is ok, -1 otherwise

  Create the recipe instance and make it available to the application using the 
  interface. This function is exported.
 */
/*----------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
    cpl_recipe  *   recipe = cpl_calloc(1, sizeof *recipe ) ;
    cpl_plugin  *   plugin = &recipe->interface ;

    cpl_plugin_init(plugin,
                    CPL_PLUGIN_API,
                    MIDI_BINARY_VERSION,
                    CPL_PLUGIN_TYPE_RECIPE,
                    "midi_fringe_all",
                    "Computes Visibilities and Transfer Functions",
                    midi_fringe_all_description,
                    "Armin Gabasch",
                    PACKAGE_BUGREPORT,
                    midi_get_license(),
                    midi_fringe_all_create,
                    midi_fringe_all_exec,
                    midi_fringe_all_destroy) ;

    cpl_pluginlist_append(list, plugin) ;
    
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Setup the recipe options    
  @param    plugin  the plugin
  @return   0 if everything is ok

  Defining the command-line/configuration parameters for the recipe.
 */
/*----------------------------------------------------------------------------*/
static int midi_fringe_all_create(cpl_plugin * plugin)
{
    cpl_recipe      * recipe ;
    cpl_parameter   * p ;

    /* Check that the plugin is part of a valid recipe */
    if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE) 
        recipe = (cpl_recipe *)plugin ;
    else return -1 ;

    /* Create the parameters list in the cpl_recipe object */
    recipe->parameters = cpl_parameterlist_new() ; 

    /* Fill the parameters list */

	p = cpl_parameter_new_value("midi.midi_fringe_all.checkSof",
			CPL_TYPE_BOOL, "Do homogeneity checks on the SOF ", "midi.midi_fringe_all", TRUE) ;
	cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "checkSof") ;
	cpl_parameterlist_append(recipe->parameters, p) ;


/*
	p = cpl_parameter_new_value("midi.midi_fringe_all.checkSof", CPL_TYPE_BOOL,
			"Do homogeneity checks on the SOF","midi.midi_fringe_all", TRUE);
	cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "checkSof") ;
	cpl_parameterlist_append(recipe->parameters, p) ;
*/

    /* Return */
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Execute the plugin instance given by the interface
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_fringe_all_exec(cpl_plugin * plugin)
{
    cpl_recipe  *   recipe ;
    
    /* Get the recipe out of the plugin */
    if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE) 
        recipe = (cpl_recipe *)plugin ;
    else return -1 ;

    batchNumber=0;
    return midi_fringe_all(recipe->parameters, recipe->frames) ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Destroy what has been created by the 'create' function
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_fringe_all_destroy(cpl_plugin * plugin)
{
    cpl_recipe  *   recipe ;
    
    /* Get the recipe out of the plugin */
    if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE) 
        recipe = (cpl_recipe *)plugin ;
    else return -1 ;

    cpl_parameterlist_delete(recipe->parameters) ; 
    return 0 ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    parlist     the parameters list
  @param    frameset   the frames list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_fringe_all(
	cpl_parameterlist	*parlist, 
	cpl_frameset		*frameset)
{
/*
	const char		*fctid = "midi_fringe_all" ;
	cpl_parameter	*param;
*/
	cpl_frame		*current_frame;
	int				error=0;
	FILE			*sofPtr=NULL;
	int				plotDuration;
        char                    * system_call=NULL;


  /* Identify the RAW and CALIB frames in the input frameset */
   cpl_ensure_code(midi_dfs_set_groups(frameset) == CPL_ERROR_NONE,
                   cpl_error_get_code());




    /* RETRIEVE INPUT PARAMETERS */

    plotDuration = 0;
    cpl_frameset_iterator * it = cpl_frameset_iterator_new(frameset);
    current_frame = cpl_frameset_iterator_get(it);
    sofPtr = fopen ("MIDI_sof.log", "w");
	while ( current_frame && sofPtr )
	{	
		fprintf (sofPtr, "%s \n", (char *)cpl_frame_get_filename( current_frame ));
        cpl_frameset_iterator_advance(it, 1);
        current_frame = cpl_frameset_iterator_get(it);
    } /* All frames from frameset */
	fclose (sofPtr);
    cpl_frameset_iterator_delete(it);
	int ret = system("sort -o MIDI_sof.log MIDI_sof.log");
    if (ret != 0) {
        cpl_msg_warning(cpl_func,"Failed to call the system command");
    }

	/* NOW PERFORMING THE DATA REDUCTION */
	executeDataReduction (MIDI_CALIBTEMP, MIDI_MASKS, "./", plotDuration, "MIDI_sof.log", &error,parlist,frameset);
/* 	executeDataReduction (calibTempDir, maskFileDir, productDir, plotDuration, "MIDI_sof.log", &error); */
	if (error) return -1;
	remove ("MIDI_sof.log");


        if (strcmp (batchTemplate, "SCI_PHOT_CALIB") == 0){
           if (CPL_ERROR_NONE !=  appendPropertylist_table("MIDI_b1_spc.pro.fits", CPL_FRAME_TYPE_TABLE, "REDUCED_DISPERSED_SCIPHOT",frameset,parlist))
           {
              cpl_msg_error(cpl_func,"Error in appendPropertylist_table");
           }
        }

        if (strcmp (batchTemplate, "SCI_PHOT_CALIB") == 0){
           if (CPL_ERROR_NONE !=  appendPropertylist("MIDI_b1_spc.waf.fits", CPL_FRAME_TYPE_IMAGE, "WATERFALL",frameset,parlist))
           {
              cpl_msg_error(cpl_func,"Error in appendPropertylist");
           }
        }


       if (strcmp (batchTemplate, "SCI_PHOT_SCIENCE") == 0){
           if (CPL_ERROR_NONE !=  appendPropertylist_table("MIDI_b1_sps.pro.fits", CPL_FRAME_TYPE_TABLE, "REDUCED_DISPERSED_SCIPHOT",frameset,parlist))
           {
              cpl_msg_error(cpl_func,"Error in appendPropertylist_table");
           }
        }

        if (strcmp (batchTemplate, "SCI_PHOT_SCIENCE") == 0){
           if (CPL_ERROR_NONE !=  appendPropertylist("MIDI_b1_sps.waf.fits", CPL_FRAME_TYPE_IMAGE, "WATERFALL",frameset,parlist))
           {
              cpl_msg_error(cpl_func,"Error in appendPropertylist");
           }
        }

        if (strcmp (batchTemplate, "SCI_PHOT_KAPPA") == 0){
           if (CPL_ERROR_NONE !=  appendPropertylist("MIDI_b1_spk.pro.fits", CPL_FRAME_TYPE_IMAGE, "KAPPA_MATRIX",frameset,parlist))
           {
              cpl_msg_error(cpl_func,"Error in appendPropertylist");
           }
           
        }
        

        if (strcmp (batchTemplate, "HIGH_SENS_CALIB") == 0){
           if (CPL_ERROR_NONE !=  appendPropertylist_table("MIDI_b1_hsc.pro.fits", CPL_FRAME_TYPE_TABLE, "REDUCED_DISPERSED",frameset,parlist))
           {
              cpl_msg_error(cpl_func,"Error in appendPropertylist_table");
           }
        }

        if (strcmp (batchTemplate, "HIGH_SENS_CALIB") == 0){
           if (CPL_ERROR_NONE !=  appendPropertylist("MIDI_b1_hsc.waf.fits", CPL_FRAME_TYPE_IMAGE, "WATERFALL",frameset,parlist))
           {
              cpl_msg_error(cpl_func,"Error in appendPropertylist");
           }
        }


       if (strcmp (batchTemplate, "HIGH_SENS_SCIENCE") == 0){
           if (CPL_ERROR_NONE !=  appendPropertylist_table("MIDI_b1_hss.pro.fits", CPL_FRAME_TYPE_TABLE, "REDUCED_DISPERSED",frameset,parlist))
           {
              cpl_msg_error(cpl_func,"Error in appendPropertylist_table");
           }
        }

        if (strcmp (batchTemplate, "HIGH_SENS_SCIENCE") == 0){
           if (CPL_ERROR_NONE !=  appendPropertylist("MIDI_b1_hss.waf.fits", CPL_FRAME_TYPE_IMAGE, "WATERFALL",frameset,parlist))
           {
              cpl_msg_error(cpl_func,"Error in appendPropertylist");
           }
        }

/* Create a clone of the Product file and append plt files for visualisation  */
        
        
        if (strcmp (batchTemplate, "HIGH_SENS_CALIB") == 0){
           
           struct plots plotfiles_lambda[] = 
              {{"MIDI_b1_hsc.WaveCalibCombined.plt",               "Lambda"},	   
               {"MIDI_b1_hsc.NormalizedUncalibVisCombined.plt",    "Visibility"},	   
               {"MIDI_b1_hsc.NormalizedUncalibVisErrCombined.plt", "VisibilityErr"},   
               {"MIDI_b1_hsc.PhotomADATA1.plt",                    "PhotomADATA1"},    
               {"MIDI_b1_hsc.PhotomAErrDATA1.plt",                 "PhotomAErrDATA1"}, 
               {"MIDI_b1_hsc.PhotomADATA2.plt",                    "PhotomADATA2"},    
               {"MIDI_b1_hsc.PhotomAErrDATA2.plt",                 "PhotomAErrDATA2"}, 
               {"MIDI_b1_hsc.PhotomBDATA1.plt",                    "PhotomBDATA1"},    
               {"MIDI_b1_hsc.PhotomBErrDATA1.plt",                 "PhotomBErrDATA1"}, 
               {"MIDI_b1_hsc.PhotomBDATA2.plt",                    "PhotomBDATA2"},    
               {"MIDI_b1_hsc.PhotomBErrDATA2.plt",                 "PhotomBErrDATA2"}, 
               {"MIDI_b1_hsc.MeanInterfCleanDATA1.plt",            "MeanInterfDATA1"}, 
               {"MIDI_b1_hsc.MeanInterfCleanDATA2.plt",            "MeanInterfDATA2"}}; 
           int  dimen_lambda = sizeof(plotfiles_lambda)/sizeof(struct plots);
           
           struct plots plotfiles_frames[] = 
              {{"MIDI_b1_hsc.PhotomACleanDATA1.plt", "PhotomA_DATA1"},  
               {"MIDI_b1_hsc.PhotomACleanDATA2.plt", "PhotomA_DATA2"}, 
               {"MIDI_b1_hsc.PhotomBCleanDATA1.plt", "PhotomB_DATA1"},  
               {"MIDI_b1_hsc.PhotomBCleanDATA2.plt", "PhotomB_DATA2"}};
           int  dimen_frames = sizeof(plotfiles_frames)/sizeof(struct plots);
           
 
           struct plots plotfiles_scans[] = 
              {{"MIDI_b1_hsc.PeakScanChannelaveraged.plt", "Peakflux_Scan"}};
           int  dimen_scans = sizeof(plotfiles_scans)/sizeof(struct plots);




          if (CPL_ERROR_NONE != appendPropertylist_statistics("MIDI_b1_hsc.pro.fits","MIDI_b1_hsc.stat.fits",CPL_FRAME_TYPE_TABLE,"REDUCED_STAT_HS",frameset,parlist,plotfiles_lambda,plotfiles_frames,plotfiles_scans,dimen_lambda,dimen_frames,dimen_scans))
           {
              cpl_msg_error(cpl_func,"Error in appending aditional files");
           }
           
           
        }
        
        
        if (strcmp (batchTemplate, "HIGH_SENS_SCIENCE") == 0){
           
           struct plots plotfiles_lambda[] = 
              {{"MIDI_b1_hss.WaveCalibCombined.plt",                "Lambda"},	   
               {"MIDI_b1_hss.NormalizedUncalibVisCombined.plt",     "Visibility"},	   
               {"MIDI_b1_hss.NormalizedUncalibVisErrCombined.plt",  "VisibilityErr"},   
               {"MIDI_b1_hss.PhotomADATA1.plt",                     "PhotomADATA1"},    
               {"MIDI_b1_hss.PhotomAErrDATA1.plt",                  "PhotomAErrDATA1"}, 
               {"MIDI_b1_hss.PhotomADATA2.plt",                     "PhotomADATA2"},    
               {"MIDI_b1_hss.PhotomAErrDATA2.plt",                  "PhotomAErrDATA2"}, 
               {"MIDI_b1_hss.PhotomBDATA1.plt",                     "PhotomBDATA1"},    
               {"MIDI_b1_hss.PhotomBErrDATA1.plt",                  "PhotomBErrDATA1"}, 
               {"MIDI_b1_hss.PhotomBDATA2.plt",                     "PhotomBDATA2"},    
               {"MIDI_b1_hss.PhotomBErrDATA2.plt",                  "PhotomBErrDATA2"}, 
               {"MIDI_b1_hss.MeanInterfCleanDATA1.plt",             "MeanInterfDATA1"}, 
               {"MIDI_b1_hss.MeanInterfCleanDATA2.plt",             "MeanInterfDATA2"}}; 
           int  dimen_lambda = sizeof(plotfiles_lambda)/sizeof(struct plots);
           
           struct plots plotfiles_frames[] = 
              {{"MIDI_b1_hss.PhotomACleanDATA1.plt", "PhotomA_DATA1"},  
               {"MIDI_b1_hss.PhotomACleanDATA2.plt", "PhotomA_DATA2"}, 
               {"MIDI_b1_hss.PhotomBCleanDATA1.plt", "PhotomB_DATA1"},  
               {"MIDI_b1_hss.PhotomBCleanDATA2.plt", "PhotomB_DATA2"}};
           int  dimen_frames = sizeof(plotfiles_frames)/sizeof(struct plots);
           
           struct plots plotfiles_scans[] = 
              {{"MIDI_b1_hss.PeakScanChannelaveraged.plt", "Peakflux_Scan"}};
           int  dimen_scans = sizeof(plotfiles_scans)/sizeof(struct plots);
           
           
           
          
           if (CPL_ERROR_NONE != appendPropertylist_statistics("MIDI_b1_hss.pro.fits","MIDI_b1_hss.stat.fits",CPL_FRAME_TYPE_TABLE,"REDUCED_STAT_HS",frameset,parlist,plotfiles_lambda,plotfiles_frames,plotfiles_scans,dimen_lambda,dimen_frames,dimen_scans))
           {
              cpl_msg_error(cpl_func,"Error in appending aditional files");
           }
           
           
        }




        if (strcmp (batchTemplate, "SCI_PHOT_CALIB") == 0){
           
           struct plots plotfiles_lambda[] = 
              {{"MIDI_b1_spc.WaveCalibCombined.plt",		       "Lambda"},	   
               {"MIDI_b1_spc.NormalizedUncalibVisCombined.plt",        "Visibility"},	   
               {"MIDI_b1_spc.NormalizedUncalibVisErrCombined.plt",     "VisibilityErr"},   
               {"MIDI_b1_spc.PhotomADATA1.plt",		               "PhotomADATA1"},    
               {"MIDI_b1_spc.PhotomAErrDATA1.plt",		       "PhotomAErrDATA1"}, 
               {"MIDI_b1_spc.PhotomIDATA2.plt",		               "PhotomIDATA2"},    
               {"MIDI_b1_spc.PhotomIErrDATA2.plt",		       "PhotomIErrDATA2"}, 
               {"MIDI_b1_spc.PhotomIDATA3.plt",                        "PhotomIDATA3"},    
               {"MIDI_b1_spc.PhotomIErrDATA3.plt",		       "PhotomIErrDATA3"}, 
               {"MIDI_b1_spc.PhotomBDATA4.plt",   		       "PhotomBDATA4"},    
               {"MIDI_b1_spc.PhotomBErrDATA4.plt",		       "PhotomBErrDATA4"}, 
               {"MIDI_b1_spc.MeanInterfCleanDATA1.plt",	               "MeanInterfDATA1"}, 
               {"MIDI_b1_spc.MeanInterfCleanDATA2.plt",                "MeanInterfDATA2"}}; 
           int  dimen_lambda = sizeof(plotfiles_lambda)/sizeof(struct plots);
           
           struct plots plotfiles_frames[] = 
              {{"MIDI_b1_spc.PhotomACleanDATA1.plt", "PhotomA_DATA1"},  
               {"MIDI_b1_spc.InterfCleanDATA2.plt",  "PhotomI_DATA2"}, 
               {"MIDI_b1_spc.InterfCleanDATA3.plt",  "PhotomI_DATA3"},  
               {"MIDI_b1_spc.PhotomBCleanDATA4.plt", "PhotomB_DATA4"}};
           int  dimen_frames = sizeof(plotfiles_frames)/sizeof(struct plots);
           
           
           struct plots plotfiles_scans[] = 
              {{"MIDI_b1_spc.PeakScanChannelaveraged.plt", "Peakflux_Scan"}};
           int  dimen_scans = sizeof(plotfiles_scans)/sizeof(struct plots);
           
           
           
           
           if (CPL_ERROR_NONE != appendPropertylist_statistics("MIDI_b1_spc.pro.fits","MIDI_b1_spc.stat.fits",CPL_FRAME_TYPE_TABLE,"REDUCED_STAT_SP",frameset,parlist,plotfiles_lambda,plotfiles_frames,plotfiles_scans,dimen_lambda,dimen_frames,dimen_scans))
           {
              cpl_msg_error(cpl_func,"Error in appending aditional files");
           }
           
           
        }

        if (strcmp (batchTemplate, "SCI_PHOT_SCIENCE") == 0){
           
           struct plots plotfiles_lambda[] = 
              {{"MIDI_b1_sps.WaveCalibCombined.plt",		       "Lambda"},	   
               {"MIDI_b1_sps.NormalizedUncalibVisCombined.plt",        "Visibility"},	   
               {"MIDI_b1_sps.NormalizedUncalibVisErrCombined.plt",     "VisibilityErr"},   
               {"MIDI_b1_sps.PhotomADATA1.plt",		               "PhotomADATA1"},    
               {"MIDI_b1_sps.PhotomAErrDATA1.plt",		       "PhotomAErrDATA1"}, 
               {"MIDI_b1_sps.PhotomIDATA2.plt",		               "PhotomIDATA2"},    
               {"MIDI_b1_sps.PhotomIErrDATA2.plt",		       "PhotomIErrDATA2"}, 
               {"MIDI_b1_sps.PhotomIDATA3.plt",                        "PhotomIDATA3"},    
               {"MIDI_b1_sps.PhotomIErrDATA3.plt",		       "PhotomIErrDATA3"}, 
               {"MIDI_b1_sps.PhotomBDATA4.plt",   		       "PhotomBDATA4"},    
               {"MIDI_b1_sps.PhotomBErrDATA4.plt",		       "PhotomBErrDATA4"}, 
               {"MIDI_b1_sps.MeanInterfCleanDATA1.plt",	               "MeanInterfDATA1"}, 
               {"MIDI_b1_sps.MeanInterfCleanDATA2.plt",                "MeanInterfDATA2"}}; 
           int  dimen_lambda = sizeof(plotfiles_lambda)/sizeof(struct plots);
           
           struct plots plotfiles_frames[] = 
              {{"MIDI_b1_sps.PhotomACleanDATA1.plt", "PhotomA_DATA1"},  
               {"MIDI_b1_sps.InterfCleanDATA2.plt",  "PhotomI_DATA2"}, 
               {"MIDI_b1_sps.InterfCleanDATA3.plt",  "PhotomI_DATA3"},  
               {"MIDI_b1_sps.PhotomBCleanDATA4.plt", "PhotomB_DATA4"}};
           int  dimen_frames = sizeof(plotfiles_frames)/sizeof(struct plots);
           
           
           struct plots plotfiles_scans[] = 
              {{"MIDI_b1_sps.PeakScanChannelaveraged.plt", "Peakflux_Scan"}};
           int  dimen_scans = sizeof(plotfiles_scans)/sizeof(struct plots);
           
           
           
           
           if (CPL_ERROR_NONE != appendPropertylist_statistics("MIDI_b1_sps.pro.fits","MIDI_b1_sps.stat.fits",CPL_FRAME_TYPE_TABLE,"REDUCED_STAT_SP",frameset,parlist,plotfiles_lambda,plotfiles_frames,plotfiles_scans,dimen_lambda,dimen_frames,dimen_scans))
           {
              cpl_msg_error(cpl_func,"Error in appending aditional files");
           }
           
           
        }
 
        
        /* Be sure to remove a possible transfer function file /tmp/MIDI_trf.fits !! */
        if (strcmp (batchTemplate, "SCI_PHOT_SCIENCE") == 0 || strcmp (batchTemplate, "HIGH_SENS_SCIENCE") == 0){
           
           system_call=cpl_sprintf("rm -f /tmp/MIDI_trf.fits");
           if (system(system_call)==0){
              cpl_msg_info(cpl_func, "Removing possible transfer function file /tmp/MIDI_trf.fits");         
              cpl_msg_info(cpl_func, "%s", system_call);         
              cpl_free(system_call);                   
           }
        }


    /* Return */
    if (cpl_error_get_code()) 
        return -1 ;
    else 
        return 0 ;
}
